PROGRAM PartInventoryTracker (Input, Output, PartFile, SalesFile, LogFile);

{***************************************************************************
 *                                                                         *
 *   Author     : Charlie Wiederhold                                       *
 *   Date       : January 7, 1996                                          *
 *   TA         : Gooty                                                    *
 *   TA Section : Wed. 3:00 - 4:00                                         *
 *   Assignment : Program 1 Inventory Tracker - Part Inventory Tracker     *
 *                                                                         *
 ***************************************************************************}

USES Crt,          {* Used for basic screen actions               *}
     Utility,      {* Used for basic screen routines/actions      *}
     Screen,       {* Prints to screen, without interaction       *}
     Routines,     {* Performs most of the background actions     *}
     List2Data,    {* Defines the types used                      *}
     List2ADT;     {* Abstract data type, used here for inventory *}

{***************************************************************************}
PROCEDURE Sales
  (VAR Inventory : ListType);
  {* Purpose : Read all sales made for the day, and insert data into *}
  {*           sales.rpt with final calculations                     *}
  {*     Pre : Sales.rpt is empty                                    *}
  {*    Post : Sales.rpt is full of all day's sales                  *}

VAR
  Amount : Integer;    {* Amount On Hand temp position *}
  Part   : PartType;   {* Part being used              *}
  Sales  : SalesFile;  {* File of sale.rpt             *}
  Exit,                {* Boolean exit condition       *}
  Found  : Boolean;    {* True if part is in list      *}

BEGIN {* Sales *}

  {* Initializations *}
  writeln (Log, 'Sales option selected');
  assign (Sales, 'sales.rpt');
  rewrite (Sales);
  Exit := False;

  {* Read all sales for the day *}
  WHILE NOT Exit DO
    BEGIN {* WHILE *}

      {* Get information *}
      Clear (MainTextColor,MainBackColor,1,9,80,24);
      GetInfo (Amount,Part.Number,'Enter amount sold ==> ');

      {* Test for exit, update, invalid *}
      IF Part.Number = EmptyString THEN
          Exit := True
        ELSE
          BEGIN {* ELSE *}
            RetrieveElement (Inventory, Part, Found);
            IF Found THEN
                BEGIN
                  ChangeInfo (Inventory,Part,Amount,'S');
                  WriteToSalesRpt (Sales, Part,Amount);
                END
              ELSE
                Invalid (Part);
            PrintOne (6,8,10,Part);
          END; {* ELSE *}
    END;

  {* Write to files, clear screen *}
  close (Sales);
  PrintReport (Sales);
  Clear (MainTextColor,MainBackColor,1,9,80,24);
  writeln (Log, 'Sales option exited');

END; {* Sales *}

{***************************************************************************}
PROCEDURE CheckStock
  (Inventory : ListType);
  {* Purpose : To read a part number and print info if found         *}
  {*    Post : Part informartion printed if it exists                *}

VAR
  Strng : STRING;    {* Temporary String value       *}
  Part  : PartType;  {* Part being used              *}
  Found : Boolean;   {* True if part is in inventory *}

BEGIN {* CheckStock *}

  {* Initialize and get part number *}
  Clear (MainTextColor,MainBackColor,1,9,80,24);
  writeln (Log, 'Check Stock option selected');
  ReadString (3,3,Strng,'Part number ==> ');
  Part.Number := Strng;

  {* Locate and output results *}
  RetrieveElement (Inventory, Part, Found);
  IF NOT Found THEN
      Invalid (Part);
  PrintOne (6,8,10,Part);
  writeln (Log, 'Check Stock option exited');

END; {* CheckStock *}

{***************************************************************************}
PROCEDURE Reorder
  (Inventory : ListType);
  {* Purpose : To take a threshhold and output a table of all parts  *}
  {*           that fall below that threshhold                       *}
  {*     Pre : Orders needed unknown                                 *}
  {*    Post : All parts needed ording printed to screen             *}

VAR
  Part       : PartType;   {* Part being used              *}
  Found      : Boolean;    {* True if part is in inventory *}
  Thresh     : STRING;     {* Threshhold string value      *}
  Count,                   {* Loop counter variable        *}
  Limit,                   {* Length of inventory          *}
  Where,                   {* Where to place the info      *}
  ThreshHold : Integer;    {* Threshhold integer value     *}

BEGIN {* Reorder *}

  {* Initialize and get threshhold *}
  Clear (MainTextColor,MainBackColor,1,9,80,24);
  writeln (Log, 'Reorder option selected');
  ReadString (3,3,Thresh,'Minimum Quantity ==> ');
  WHILE NOT Convert (Thresh, Threshhold) DO
    InvalidData (3,Thresh);

  {* Output all parts needing reordering *}
  Clear (MainTextColor,MainBackColor,1,9,80,24);
  Limit := Length (Inventory);
  Header (1);
  Where := 3;
  ResetList (Inventory);

  {* Printouts *}
  FOR Count := 1 TO Limit DO
    BEGIN {* FOR *}
      GetNextItem (Inventory, Part);
      IF Part.OnHand < ThreshHold THEN
        BEGIN
          OutputPart (Part,Where);
          inc(Where);
        END;
    END; {* FOR *}
  PressEnter (Where + 1);
  writeln (Log, 'Reorder option exited');

END; {* Update *}

{***************************************************************************}
PROCEDURE NewStock
  (VAR Inventory : ListType);
  {* Purpose : To get new stock for existing and new parts           *}
  {*     Pre : Stock low, empty, or non-existent                     *}
  {*    Post : All new stock and parts added to inventory            *}

VAR
  Amount : Integer;   {* On Hand amount temp position *}
  Part   : PartType;  {* Part being used              *}
  Exit,               {* Boolean exit condition       *}
  Found  : Boolean;   {* True if part is in inventory *}

BEGIN {* NewStock *}

  {* Initialize *}
  writeln (Log, 'New Stock option selected');
  Exit := False;
  amount := Length (inventory);

  {* Read in all new stock *}
  WHILE NOT Exit DO
    BEGIN {* WHILE *}

      {* Get new info *}
      Clear (MainTextColor,MainBackColor,1,9,80,24);
      GetInfo (Amount, Part.Number, 'Enter number to be added ==> ');
      IF Part.Number = EmptyString THEN
          Exit := True

        {* Modify as needed *}
        ELSE
          BEGIN {* ELSE *}
            RetrieveElement (Inventory, Part, Found);
            IF Found THEN
                ChangeInfo (Inventory,Part,Amount,'A')
              ELSE IF NewPart (12) THEN
                BEGIN
                  Part.OnHand := 0;
                  UpdateOnHand (Amount, Part, 'A');
                  UpdatePart (13,Part);
                  InsertElement (Inventory, Part);
                END
              ELSE
                Invalid (Part);
            PrintOne (6,8,10,Part);
          END; {* ELSE *}
    END; {* WHILE *}

  Clear (MainTextColor,MainBackColor,1,9,80,24);
  writeln (Log, 'New Stock option exited');

END; {* NewStock *}

{***************************************************************************}
PROCEDURE Update
  (Inventory : ListType);
  {* Purpose : Update existing parts                                 *}
  {*     Pre : Parts needing updating unchanged                      *}
  {*    Post : Parts needing updating updated to current values      *}

VAR
  Strng : STRING;    {* Teporary string value        *}
  Part  : PartType;  {* Part being used              *}
  Found,             {* True if part is in inventory *}
  Exit  : Boolean;   {* Boolean exit condition       *}

BEGIN {* Update *}

  {* Initialize *}
  writeln (Log, 'Update option selected');
  Exit := False;

  {* Read all parts needing updating *}
  WHILE NOT Exit DO
    BEGIN {* WHILE *}

      {* Read data and update as needed *}
      Clear (MainTextColor,MainBackColor,1,9,80,24);
      ReadString (3,2,Strng,'Part number ==> ');
      Part.Number := Strng;
      IF Part.Number = EmptyString THEN
          Exit := True
        ELSE
          BEGIN
            RetrieveElement (Inventory,Part,Found);
            IF Found THEN
                UpdateTotalPart (Inventory,Part)
              ELSE
                Invalid (Part);
            PrintOne (6,8,10,Part);
          END;

    END; {* WHILE *}

  Clear (MainTextColor,MainBackColor,1,9,80,24);
  writeln (Log, 'Update option exited');

END; {* Update *}

{***************************************************************************}
PROCEDURE OpenInventory
  (VAR Inventory : ListType);
  {* Purpose : To open all files and get inventory for program       *}
  {*     Pre : No information in inventory and all files closed      *}
  {*    Post : Inventory created and files opened and ready          *}

VAR
  Sales : SalesFile;       {* Sales.rpt variable *}
  Parts : PartFileType;    {* Parts.bin variable *}
  Part  : PartType;        {* Part being used    *}

BEGIN {* Open Inventory *}

  {* Get sales.log started *}
  assign (Log, 'sales.log');
  rewrite (Log);
  writeln (Log, 'Program started');

  {* Clear sales.rpt *}
  assign (Sales, 'sales.rpt');
  rewrite (Sales);
  close (Sales);

  {* Read from parts.bin the inventory *}
  assign (Parts, 'parts.bin');
  reset (Parts);
  CreateList (Inventory);
  WHILE NOT EOF(Parts) DO
    BEGIN
      read (Parts, Part);
      InsertElement (Inventory, Part);
    END;
  close (Parts);

  writeln (Log, 'Inventory opened');

END; {* OpenInventory *}

{***************************************************************************}
PROCEDURE ProcessCommands
  (VAR Inventory : ListType);
  {* Purpose : Controls all selection of varios options              *}
  {*     Pre : Program just starting                                 *}
  {*    Post : Program finished and data updated                     *}

VAR
  Exit   : Boolean;  {* Boolean exit condition  *}
  Cursor : byte;     {* Cursor position in menu *}

BEGIN {* ProcessCommands *}

  {* Initialize *}
  ScreenSetup;
  PrintMenu;
  Exit := FALSE;
  Cursor := 1;

  {* Continue to allow selections until cursor is 6 (Exit) *}
  WHILE NOT Exit DO
    BEGIN {* WHILE *}
      Select (MenuSelectColor,MenuBackColor,7,6,34,2,Cursor);
      CASE Cursor OF
        1 : Sales (Inventory);
        2 : CheckStock (Inventory);
        3 : Reorder (Inventory);
        4 : NewStock (Inventory);
        5 : Update (Inventory);
        6 : Exit := TRUE;
      END; {* CASE *}
      Clear (MainTextColor,MainBackColor,1,9,80,24);
      window (1,1,80,25);
    END; {* WHILE *}

END; {* ProcessCommands *}

{***************************************************************************}
PROCEDURE CloseInventory
  (Inventory : ListType);
  {* Purpose : Save parts.bin and sales.log and other last actions   *}
  {*     Pre : Parts.bin and sales.log unfinished                    *}
  {*    Post : Parts.bin and sales.log saved and closed              *}

VAR
  Parts : PartFileType;  {* Parts.bin variable    *}
  Part  : PartType;      {* Part being used       *}
  Limit,                 {* Lenght of inventory   *}
  Count : Integer;       {* Loop counter variable *}

BEGIN {* CloseInventory *}

  {* Prepare sales.log and parts.bin *}
  writeln (Log, 'Exit program selected');
  ResetList (Inventory);
  assign (Parts, 'parts.bin');
  rewrite (Parts);
  Limit := Length (Inventory);

  {* Write inventory to parts.bin *}
  FOR Count := 1 TO Limit DO
    BEGIN
      GetNextItem (Inventory, Part);
      write (Parts, Part);
    END;

  {* Close up last files and stuff *}
  close (Parts);
  write (Log, 'Inventory saved and program exited');
  close (Log);
  Clear (7,0,1,1,80,25);

END; {* CloseInventory *}

{***************************************************************************}
PROCEDURE ControlProcessing;
  {* Purpose : To control and run the main modules                     *}

VAR
  Inventory : ListType;    {* Inventory of all parts *}

BEGIN {* ControlProcessing *}
  OpenInventory (Inventory);
  ProcessCommands (Inventory);
  CloseInventory (Inventory);
END; {* ControlProcessing *}

{***************************************************************************}

BEGIN {* Main *}

  ControlProcessing;

END. {* Main *}
