{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} UNIT Postfix; {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


{///////////////////////////////////////////////////////////////////////////}
{/                                                                         /}
{/  Author     : Charlie Wiederhold                                        /}
{/  Date       : March 04, 1996                                            /}
{/  Assignment : Program assignment 2                                      /}
{/  TA         : Gooty                                                     /}
{/  Purpose    : To read in an ifnfix expression, and convert it to a      /}
{/               postfix expression and then evaluate that postfix         /}
{/               expression to obtain a real result.                       /}
{/                                                                         /}
{///////////////////////////////////////////////////////////////////////////}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} INTERFACE {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


USES
  Utility,                    {* Performs most of my routine tasks         *}
  Prog2Data,                  {* Contains the Type Data for Program 2      *}
  StackADT;                   {* Pre-written Stack ADT for our use         *}

VAR
  Operators,                  {* Stack of Operators in the expression      *}
  Operands  : StackType;      {* Stack of Operands in the expression       *}

{***************************************************************************}
PROCEDURE InfixToPostfix
  (    InfixExp   : STRING;   {* Infix expression entered by the user      *}
   VAR PostfixExp : STRING;   {* Postfix expression converted by Program 2 *}
   VAR Error      : Boolean); {* Error checking parameter                  *}
{* Purpose : To convert the Infix expression to Postfix                    *}
{*     Pre : Infix expression read, and not converted                      *}
{*    Post : Postfix expression created, and passed back to call           *}

{***************************************************************************}
PROCEDURE EvaluatePostfix
  (    PostfixExp : STRING;   {* Infix expression entered by the user      *}
   VAR Result     : Real;     {* The real result calculated from Postfix   *}
   VAR Error      : Boolean); {* Boolean error checking varaible           *}

{* Purpose : To evalute the postfix expression and get a real result       *}
{*     Pre : Value of postfix expression unknown                           *}
{*    Post : Postfix expression evaluated and real value passed to call    *}

{***************************************************************************}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} IMPLEMENTATION {>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


{***************************************************************************}
PROCEDURE Combine
  (VAR Token : TokenType);    {* Specific element from the expression      *}
{* Purpose : To combine two operands with their operator in postfix        *}
{*     Pre : Token is looking for operand and parts un-combined            *}
{*    Post : Token is looking for an operator and parts combined           *}

VAR
  Op1,                        {* First operator in the expression          *}
  Op2,                        {* Second operator in the expression         *}
  Oper : TokenType;           {* Operator for Op1 and Op2                  *}

BEGIN {* Combine *}

  Pop (Operators, Oper);

  {* Gets the operands and combines them with the operator in postix *}
  WHILE Oper.Character <> '(' DO
    BEGIN {* While *}
      Pop (Operands,Op2);
      Pop (Operands,Op1);
      Op1.StrPart := concat(Op1.StrPart,' ',Op2.StrPart,' ',Oper.Character);
      Push (Operands,Op1);
      Pop (Operators,Oper);
    END; {* While *}

  {* Switch to next expected token type *}
  Token.Contents := Operator;

END; {* Combine *}

{***************************************************************************}
PROCEDURE EndExp;
{* Purpose : To continue to combine elements if needed                     *}
{*     Pre : Operator and operands not combined                            *}
{*    Post : Operator and operands combined in postfix                     *}

VAR
  Op1,                        {* First operand in the expression           *}
  Op2,                        {* Second operand in the expression          *}
  Oper: TokenType;            {* Operator for Op1 and Op2                  *}

BEGIN {* EndExp *}

  {* Extra combining procedure in case the stack is not empty *}
  Pop (Operators,Oper);
  Pop (Operands,Op2);
  Pop (Operands,Op1);
  Op1.StrPart := concat(Op1.StrPart,' ',Op2.StrPart,' ',Oper.Character);
  Push (Operands,Op1);

END; {* EndExp *}

{***************************************************************************}
FUNCTION Precedes
  (Oper1,                     {* First operator to be compared             *}
   Oper2 : Char) : Boolean;   {* Second operator to be compared            *}
{* Purpose : Checks precedence between two operators                       *}
{*     Pre : Precedence unknown                                            *}
{*    Post : Precedence known                                              *}

BEGIN {* Precedes *}

  {* Multiplication precedence *}
  IF (Oper1='*') AND (Oper2 IN ['+','-']) THEN
      Precedes := True
    ELSE
      {* Division precedence *}
      IF (Oper1='/') AND (Oper2 IN ['+','-']) THEN
          Precedes := True
      ELSE
        {* Additon/subtraction precedence *}
        IF (Oper1 IN ['+','-']) AND (Oper2 IN ['+','-']) THEN
            Precedes := True
        ELSE
          {* Subtraction/addition precedence *}
          IF (Oper1 IN ['*','/']) AND (Oper2 IN ['*','/']) THEN
              Precedes := True
          ELSE
            {* Precedence fails *}
            Precedes := False;

END; {* Precedes *}

{***************************************************************************}
PROCEDURE ProcessOperator
  (VAR Token : TokenType);    {* Specific element from the expression      *}
{* Purpose : To check the operator, including precedence and evaluate      *}
{*     Pre : Unknown precedence and postfix value                          *}
{*    Post : Precedence known and postfix value created                    *}

VAR
  PrevOper,                   {* Previous operator in the stack            *}
  Op1,                        {* First operand in the expression           *}
  Op2      : TokenType;       {* Second operand in the expression          *}

BEGIN {* ProcessOperator *}

  REPEAT
    IF NOT EmptyStack(Operators) THEN
      BEGIN {* If Then *}
        Pop (Operators,PrevOper);
        {* Checks for precedence, if true, then performs the combination... *}
        IF Precedes (PrevOper.Character,Token.Character) THEN
          BEGIN {* If Then *}
            Pop (Operands,Op2);
            Pop (Operands,Op1);
            Op1.StrPart := concat(Op1.StrPart,' ',Op2.StrPart,' ',
                                  PrevOper.Character);
            Push (Operands,Op1);
          END {* If Then *}
        ELSE
          {* ...Otherwise, push it onto the stack for later use *}
          Push (Operators,PrevOper);
      END; {* If Then *}
  UNTIL (NOT Precedes (PrevOper.Character,Token.Character)) OR
        (EmptyStack (Operators));
  Push (Operators,Token);
  Token.Contents := Operand;

END; {* ProcessOperator *}

{***************************************************************************}
PROCEDURE ProcessToken
  (VAR Token : TokenType);    {* Specific element from the expression      *}
{* Purpose : To perform operation on token, depending on what it is        *}
{*     Pre : Token undefined                                               *}
{*    Post : Token defined and data sent back to call                      *}

BEGIN {* ProcessToken *}

  {* Depending on the Type of token, decides what actions to take *}
  CASE Token.Contents OF
    Operator     : ProcessOperator (Token);
    Operand      : BEGIN {* Case Operand *}
                     Push (Operands,Token);
                     Token.Contents := Operator;
                   END; {* Case Operand *}
    LeftParen    : BEGIN {* Case LeftParen *}
                     Push (Operators,Token);
                     Token.Contents := Operand;
                   END; {* Case LeftParen *}
    RtParen      : Combine (Token);
    EndExpression: EndExp;
  END; {* Case *}

END; {* ProcessToken *}

{***************************************************************************}
PROCEDURE GetOperandToken
  (VAR Token      : TokenType;{* Specific element from the expression      *}
   VAR Counter    : Byte;     {* Loop counter variable                     *}
       Expression : STRING;   {* The expression to get the operand from    *}
   VAR Error      : Boolean;  {* Error checking boolean variable           *}
       Infix      : Boolean); {* Used to check for blank spaces in infix   *}
{* Purpose : Main control to read and get an operand token                 *}
{*     Pre : Operand token in expression unkown                            *}
{*    Post : Operand token in expression created and passed back to call   *}

VAR
 Len : Integer;               {* The length of the expression              *}

BEGIN (* GetOperandToken *)

  {* Checking for double negative signs. Ex 5--3 *}
  IF (Expression[Counter] = '-') OR (Expression[Counter] = '') THEN
    BEGIN {* If Then *}
      Token.StrPart := '';
      Inc(Counter)
    END {* If Then *}
  ELSE
    Token.StrPart := '';

  {* Initialize *}
  Error := False;
  Token.Value := 0;
  Len := length(Expression);

  {* If the character is in the valid set of operand parts, it will build *}
  {* the operand from this part, with the next parts, until it reaches a  *}
  {* character that is not in the valid set.                              *}
  WHILE ((((Expression[Counter] IN ['0'..'9','E','e','.']) OR
          ((Expression[Counter] = '-') AND
           (Expression[Counter - 1] IN ['E','e'])) AND NOT Error)
       OR ((Expression[Counter] = ' ') AND Infix))) AND
           (Counter <= Len) DO
    BEGIN {* While *}
      {* Combine to create the string for the operand *}
      IF Expression[Counter] IN ['0'..'9','E','e','.','-'] THEN
          BEGIN {* If Then *}
            Token.StrPart := concat(Token.StrPart,Expression[Counter]);
            inc(Counter);
          END {* If Then *}
        ELSE
          {* Check for blank spaces *}
          IF (Expression[Counter] = ' ') AND Infix THEN
              inc(Counter)
            ELSE Error := True;
    END; (* While *)

END;  (* GetOperandToken *)

{***************************************************************************}
PROCEDURE GetOperatorToken
  (VAR Token      : TokenType;{* Specific element from the expression      *}
   VAR Counter    : Byte;     {* Loop counter variable                     *}
       Expression : STRING;   {* The expression to get the operator from   *}
   VAR Error      : Boolean); {* Error checking boolean variable           *}
{* Purpose : Main control to read and get an operator token                *}
{*     Pre : Token undefined                                               *}
{*    Post : Token defined and passed back to call                         *}

BEGIN {* GetOperatorToken *}

  {* Initialize *}
  Error := False;

  {* Check if it is in the proper operator set *}
  IF Expression[Counter] IN ['+','-','*','/'] THEN
      BEGIN {* If Then *}
        IF Counter = length(Expression) THEN
            Error := True
          ELSE
            BEGIN
              Token.Character := Expression[Counter];
              inc(Counter);
            END;
      END {* If Then *}
    ELSE Error := True;

END; {* GetOperatorToken *}

{***************************************************************************}
PROCEDURE GetToken
  (VAR Token    : TokenType;  {* Specific element from the expression      *}
       InfixExp : STRING;     {* Infix expression entered from user        *}
   VAR Counter  : Byte;       {* Loop counter variable                     *}
   VAR Error    : Boolean);   {* Error checking boolean variable           *}
{* Purpose : Get a token from an expression                                *}
{*     Pre : Token is undefined and unknown                                *}
{*    Post : Token is defined, known, and passed back to call              *}

BEGIN {* GetToken *}

  {* Assign Parentheses to token for use *}
  IF InfixExp[Counter] = '(' THEN
    BEGIN {* If Then *}
      Token.Contents := LeftParen;
      Token.Character := '(';
    END; {* If Then *}
  IF InfixExp[Counter] = ')' THEN
      Token.Contents := RtParen;

  {* Depending on what type Token is, the needed action is taken *}
  CASE Token.Contents OF
    Operand   : GetOperandToken (Token,Counter,InfixExp,Error,True);
    Operator  : GetOperatorToken (Token,Counter,InfixExp,Error);
    LeftParen,
    RtParen   : inc(Counter);
  END; {* Case *}

END;

{***************************************************************************}
PROCEDURE EvaluateOperator
  (VAR EvalStack  : StackType;{* Stack to be evaluated from                *}
   VAR Count      : Byte;     {* Loop counter variable                     *}
       PostfixExp : STRING;   {* Converted postfix expression              *}
   VAR Error      : Boolean); {* Error checking boolean variable           *}
{* Purpose : To evaluate the real value depending on the operator          *}
{*     Pre : Current value in stack unkown                                 *}
{*    Post : Current value in stack known and pushed back onto EvalStack   *}

VAR
  Op1,                        {* First operand in the expression           *}
  Op2 : TokenType;            {* Second operand in the expression          *}

BEGIN

  {* Get the operators *}
  Pop (EvalStack,Op2);
  Pop (EvalStack,Op1);

  {* Depending on the sign, the proper arithmetic action is taken *}
  CASE PostfixExp[Count] OF
    '+' : Op1.Value := Op1.Value + Op2.Value;
    '-' : Op1.Value := Op1.Value - Op2.Value;
    '*' : Op1.Value := Op1.Value * Op2.Value;
    '/' : IF Op2.Value <> 0 THEN
              Op1.Value := Op1.Value / Op2.Value
            ELSE Error := True;
  END; {* Case *}

  {* Push the result on the evaluated stack *}
  Push (EvalStack,Op1);
  Count := Count + 2;

END; {* EvaluateOperator *}

{***************************************************************************}
PROCEDURE CheckParen
  (    InfixExp : STRING;     {* Infix expression entered by the user      *}
   VAR Error : Boolean);      {* Error checking boolean variable           *}
{* Purpose : Check to see if the number of Rt Paren = Lft Paren            *}
{*     Pre : Correct amount of Paren's unkown                              *}
{*    Post : Correct amount of Paren's known and boolean sent back to call *}

VAR
  RightParen,                 {* Number of right parentheses               *}
  LeftParen,                  {* Number of left parentheses                *}
  Len,                        {* Length of the expression                  *}
  Counter    : Byte;          {* Loop counter variable                     *}

BEGIN {* CheckParen *}

  {* Initialize *}
  RightParen := 0;
  LeftParen := 0;
  Len := length(InfixExp);

  {* Count the number of parentheses *}
  FOR Counter := 1 TO Len DO
    BEGIN {* For To Do *}
      IF InfixExp[Counter] = ')' THEN inc(RightParen);
      IF InfixExp[Counter] = '(' THEN inc(LeftParen);
    END; {* For To Do *}

  {* Check to see if they match *}
  IF RightParen <> LeftParen THEN
      Error := TRUE
    ELSE Error := False;

END; {* CheckParen *}

{***************************************************************************}
PROCEDURE InfixToPostfix
  (    InfixExp   : STRING;   {* Infix expression entered by the user      *}
   VAR PostfixExp : STRING;   {* Converted postfix expression              *}
   VAR Error      : Boolean); {* Error checking boolean variable           *}
{* Purpose : To convert the Infix expression to Postfix                    *}
{*     Pre : Infix expression read, and not converted                      *}
{*    Post : Postfix expression created, and passed back to call           *}

VAR
  Len,                        {* Length of the expression                  *}
  Counter : Byte;             {* Loop counter variable                     *}
  Token   : Tokentype;        {* Specific element from the expression      *}

BEGIN {* InfixToPostfix *}

  {* Initialize *}
  CreateStack (Operators);
  CreateStack (Operands);
  Len := length(InfixExp);
  Counter := 1;
  Token.Contents := Operand;
  Error := False;
  CheckParen (InfixExp,Error);

  {* Get all tokens, and construct the stacks *}
  WHILE (Counter <= Len) AND NOT Error DO
    BEGIN {* While *}
      GetToken (Token,InfixExp,Counter,Error);
      ProcessToken (Token);
    END; {* While *}

  {* Put together the postfix expression *}
  WHILE NOT (EmptyStack (Operators)) AND NOT Error DO
    BEGIN {* While *}
      Token.Contents := EndExpression;
      ProcessToken (Token);
    END; {* While *}

  {* Get the final postfix expression from the stack, and output result *}
  IF NOT Error THEN
      BEGIN
        Pop (Operands,Token);
        PostfixExp := Token.StrPart;
      END
    ELSE PostfixExp := ' Invalid expression entered ';

END; {* InfixToPostfix *}

{***************************************************************************}
PROCEDURE EvaluatePostfix
  (    PostfixExp : STRING;   {* Converted postifix expression             *}
   VAR Result     : Real;     {* The evaluated result from the postfix     *}
   VAR Error      : Boolean); {* Boolean error checking varaible           *}
{* Purpose : To evalute the postfix expression and get a real result       *}
{*     Pre : Value of postfix expression unknown                           *}
{*    Post : Postfix expression evaluated and real value passed to call    *}

VAR
  Token     : TokenType;      {* Specific element from the expressions     *}
  EvalStack : StackType;      {* Evalution stack to hold the values        *}
  ErrorPos  : Integer;        {* Position of an error in the string value  *}
  Counter,                    {* Loop counter variable                     *}
  Len       : Byte;           {* The length of the expression              *}

BEGIN {* EvaluatePostfix *}

  {* Initialize *}
  CreateStack (EvalStack);
  Len := length(PostfixExp);
  Counter := 1;
  Error := False;
  ErrorPos := 0;

  {* Controls the outer most exit checks for evaluation *}
  WHILE ((Counter <= Len) AND NOT Error) DO
    BEGIN {* While *}

      {* If it is an operator, evaluate the sub expression *}
      IF PostfixExp[Counter] IN ['+','-','*','/'] THEN
          EvaluateOperator (EvalStack,Counter,PostfixExp,Error)
        ELSE

          {* Check for blank spaces *}
          IF PostfixExp[Counter] = ' ' THEN
              inc(Counter)
          ELSE

            {* Gets an operator from the postfix expression *}
            BEGIN {* Else *}
              Token.Contents := Operand;
              GetOperandToken (Token,Counter,PostfixExp,Error,False);
              inc(Counter);

              {* Check for negatives *}
              IF Token.StrPart[1] = '' THEN
                  Token.StrPart[1] := '-';
              val (Token.StrPart,Token.Value,ErrorPos);
              IF ErrorPos = 0 THEN
                  Push (EvalStack,Token)
                ELSE Error := True;
            END; {* Else *}
    END; {* While *}

  {* Get value from stack if there is no error in the expression *}
  IF NOT Error THEN
      BEGIN
        Pop (EvalStack,Token);
        Result := Token.Value;
      END;

END; {* EvaluatePostfix *}

{***************************************************************************}

END. {* Unit Postfix *}