UNIT StackADT;

INTERFACE

USES
  Prog2Data;

CONST
  MaxStack = 50;

TYPE

  StackType = RECORD
    Elements : ARRAY [1 .. MaxStack] OF StackElementType;
    Top      : 0 .. MaxStack
  END; (* StackType *)

(*********************************************************)

PROCEDURE CreateStack 
  (VAR Stack: StackType);
(* Function: Initializes Stack to an empty state.*)
(* Preconditions: None                           *)
(* Postconditions:  Stack is empty.              *)

(*********************************************************)

PROCEDURE DestroyStack 
  (VAR Stack: StackType);
(* Function: Removes all elements from Stack,    *)
(*   leaving the stack empty.                    *)
(* Preconditions: Stack has been created.        *)
(* Postconditions: Stack is empty.               *)

(*********************************************************)

FUNCTION EmptyStack
  (Stack: StackType) : Boolean;
(* Function: Tests whether Stack is empty.       *)
(* Preconditions: Stack has been created.        *)
(* Postconditions: EmptyStack = (stack is empty) *)

(*********************************************************)

FUNCTION FullStack 
  (Stack: StackType) : Boolean;
(* Function: Tests whether Stack is full.        *)
(* Preconditions: Stack has been created.        *)
(* Postconditions: FullStack = (stack is full)   *)

(*********************************************************)

PROCEDURE Push 
  (VAR Stack: StackType;
   NewElement: StackElementType);
(* Function: Adds NewElement to the top of Stack.*)
(* Preconditions: Stack has been created and it  *)
(*   is not full.                                *)
(* Postconditions: Stack = original Stack with   *)
(*   NewElement added on top.                    *)

(*********************************************************)

PROCEDURE Pop 
  (VAR Stack: StackType;
   VAR PoppedElement: StackElementType);
(* Function: Removes top element from Stack and    *)
(*   returns it in PoppedElement.                  *)
(* Preconditions: Stack has been created and is    *)
(*   not empty.                                    *)
(* Postconditions: Stack = original Stack with     *)
(*   top element removed.                          *)
(*   PoppedElement = top element of original Stack *)

(*********************************************************)
(*-------------------------------------------------------*)
(*********************************************************)

IMPLEMENTATION

PROCEDURE CreateStack
  (VAR Stack : StackType);
  (* Pre:  None                          *)
  (* Post: Stack is initialized to empty *)
  (*       by setting Stack.Top to 0.    *)

BEGIN (* CreateStack *)
  Stack.Top := 0
END;  (* CreateStack *) 

(*********************************************************)

PROCEDURE DestroyStack
  (VAR Stack : StackType);
  (* Pre:  Stack has been initialized. *)
  (* Post: Stack is empty.             *)
BEGIN (* DestroyStack *)
  Stack.Top := 0
END;  (* DestroyStack *) 

(*********************************************************)

FUNCTION EmptyStack
  (Stack : StackType) : Boolean;
  (* Pre:  Stack has been initialized.  *)
  (* Post: EmptyStack = (Stack.Top = 0) *)

BEGIN (* EmptyStack. *)
  EmptyStack := (Stack.Top = 0)
END; (* EmptyStack *) 

(*********************************************************)

FUNCTION FullStack 
  (Stack : StackType) : Boolean;
  (* Pre:  Stack has been initialized.        *)
  (* Post: FullStack = (Stack.Top = MaxStack) *)

BEGIN (* FullStack *)
  FullStack := (Stack.Top = MaxStack)
END; (* FullStack *) 

(*********************************************************)

PROCEDURE Push 
  (VAR Stack  : StackType;
   NewElement : StackElementType);
  (* Pre:  Stack has been initialized. *)
  (*       NOT FullStack (Stack)       *)
  (* Post: Stack.Top is incremented.   *)
  (*       Stack.Elements[Stack.Top]   *)
  (*       is set to NewElement.       *)

BEGIN (* Push *)
  Stack.Top := Stack.Top + 1;
  Stack.Elements[Stack.Top] := NewElement
END; (* Push *)

(*********************************************************)

PROCEDURE Pop
  (VAR Stack         : StackType;
   VAR PoppedElement : StackElementType);
  (* Pre:  Stack has been initialized.   *)
  (*       NOT EmptyStack(Stack)         *)
  (* Post: PoppedElement is set to       *)
  (*       Stack.Elements[Stack.Top].    *)
  (*       Stack.Top is decremented.     *)

BEGIN (* Pop *)
  PoppedElement := Stack.Elements[Stack.Top];
  Stack.Top := Stack.Top - 1
END; (* Pop *) 

END.  (* StackAdt *)
