{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} UNIT CPU; {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}

              {* Most variables identified in Program3.pas *}

{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} INTERFACE {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


USES
  Prog3Data,   {* Contains basic initializations *}
  Queue2ADT,   {* Contains the queue types       *}
  CircleADT;   {* Contains the circle list type  *}

TYPE
  ReadyType = ARRAY [PriorityRange] OF QueueType; {* Type for ReadyQueue *}

{***************************************************************************}
PROCEDURE NewProgram
  (VAR Output : Text;
   VAR ReadyQueue : ReadyType;
   VAR CurrentPriority : ListType;
   VAR IDNumber : Integer;
       ProgramTime,
       ArrivalTime : Integer);
  {* Purpose : To assign a new program, and put it in its queue           *}
  {*     Pre : New program needed but not created                         *}
  {*    Post : Program created, initialized, and put in its queue         *}

{***************************************************************************}
PROCEDURE CPUFree
  (VAR CPU : Integer;
   VAR InExecution : QueueElementType;
   VAR ReadyQueue : ReadyType;
   VAR ProgramGoing : Boolean;
       TimeSlice : Integer);
  {* Purpose : Finds the next program to enter into CPU if it exists      *}
  {*     Pre : CPU is free and program is not in CPU                      *}
  {*    Post : CPU has a new program if it exits and is ready to comput   *}

{***************************************************************************}
PROCEDURE ProgramNotDone
  (VAR ReadyQueue : ReadyType;
       InExecution : QueueElementType);
  {* Purpose : Put the program back into the ready queue that it belongs  *}
  {*     Pre : Program time slice is over but program not finished        *}
  {*    Post : Program is enered back into the ready queue it belongs     *}

{***************************************************************************}
PROCEDURE ProgramFinish
  (VAR Output : Text;
   VAR TotalWaitTime : Integer;
   VAR ProgsFinished : Integer;
       InExecution : QueueElementType;
       Clock : Integer);
  {* Purpose : Dispose of the finished program and update wait times      *}
  {*     Pre : Program is finished                                        *}
  {*    Post : Wait times updated and program is eleminated               *}

{***************************************************************************}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} IMPLEMENTATION {>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


{***************************************************************************}
PROCEDURE NewProgram
  (VAR Output : Text;
   VAR ReadyQueue : ReadyType;
   VAR CurrentPriority : ListType;
   VAR IDNumber : Integer;
       ProgramTime,
       ArrivalTime : Integer);
  {* Purpose : To assign a new program, and put it in its queue           *}
  {*     Pre : New program needed but not created                         *}
  {*    Post : Program created, initialized, and put in its queue         *}

VAR
  TempProgram : QueueElementType;   {* Temp program to be added *}

BEGIN {* NewProgram *}

  {* Assign the ID Number *}
  IF IDNumber = 9999 THEN
      IDNumber := 1001
    ELSE inc(IDNumber);

  {* Assign all specified values of the programs *}
  TempProgram.IDNum := IDNumber;
  TempProgram.CPUNeed := ProgramTime;
  GetNextItem (CurrentPriority, TempProgram.Priority);
  TempProgram.Arrive := ArrivalTime;

  {* Insert into proper Ready queue *}
  Enqueue (ReadyQueue[TempProgram.Priority], TempProgram);
  writeln (Output, ' Program ', TempProgram.IDNum, ' has arrived');

END; {* NewProgram *}

{***************************************************************************}
PROCEDURE CPUFree
  (VAR CPU : Integer;
   VAR InExecution : QueueElementType;
   VAR ReadyQueue : ReadyType;
   VAR ProgramGoing : Boolean;
       TimeSlice : Integer);
  {* Purpose : Finds the next program to enter into CPU if it exists      *}
  {*     Pre : CPU is free and program is not in CPU                      *}
  {*    Post : CPU has a new program if it exits and is ready to comput   *}

VAR
  Count : PriorityRange;  {* Loop control variable *}

BEGIN {* CPUFree *}

  {* Find the highest program priority *}
  Count := 4;
  WHILE EmptyQueue(ReadyQueue[Count]) DO dec(Count);

  {* If a program exists, then get it into exectution *}
  IF Count > 0 THEN
      BEGIN
        Dequeue (ReadyQueue[Count], InExecution);
        CPU := TimeSlice * InExecution.Priority;
        ProgramGoing := True;
      END
    ELSE ProgramGoing := False;

END; {* CPUFree *}

{***************************************************************************}
PROCEDURE ProgramNotDone
  (VAR ReadyQueue : ReadyType;
       InExecution : QueueElementType);
  {* Purpose : Put the program back into the ready queue that it belongs  *}
  {*     Pre : Program time slice is over but program not finished        *}
  {*    Post : Program is enered back into the ready queue it belongs     *}

BEGIN {* ProgramNotDone *}

  {* Put the program back in its queue to wait for the next time slice *}
  Enqueue (ReadyQueue[InExecution.Priority], InExecution);

END; {* ProgramNotDone *}

{***************************************************************************}
PROCEDURE ProgramFinish
  (VAR Output : Text;
   VAR TotalWaitTime : Integer;
   VAR ProgsFinished : Integer;
       InExecution : QueueElementType;
       Clock : Integer);
  {* Purpose : Dispose of the finished program and update wait times      *}
  {*     Pre : Program is finished                                        *}
  {*    Post : Wait times updated and program is eleminated               *}

BEGIN {* ProgramFinish *}

  {* Update wait times and dispose of program *}
  inc(ProgsFinished);
  TotalWaitTime := TotalWaitTime + (Clock - InExecution.Arrive);
  writeln (Output, ' Program ', InExecution.IDNum, ' has finished');

END; {* ProgramFinish *}

{***************************************************************************}

END. {* CPU *}