PROGRAM Program3;

{**************************************************************************}
{*                                                                        *}
{* Author     : Charlie Wiederhold                                        *}
{* Date       : April 1, 1996                                             *}
{* TA         : Gooty                                                     *}
{* Section    : Wed 3-4                                                   *}
{* Assingment : Program 3 - MultiTasking Simulation                       *}
{* Purpose    : To simulate a multitasking OS using a circular linked     *}
{*              and FIFO queues.                                          *}
{* Note       : All variables are labeled and identified in the main      *}
{*              procedure, and thus, are not labeled at each procedure    *}
{*              header because this would require repeating myself a few  *}
{*              hundred times. Thus, unless expressly stated, all         *}
{*              parameters are as descibed in Control Processing.         *}
{*                                                                        *}
{**************************************************************************}

USES
  Crt,        {* Allows many basic routines, such as ClrScr         *}
  Prog3Data,  {* Contains the type declarations for the ADT's, etc. *}
  Queue2ADT,  {* The ADT that implements the FIFO Queue             *}
  CircleADT,  {* The ADT that implements the Circular linked list   *}
  CPU,        {* Controls the operations of the CPU                 *}
  IOServer;   {* Controls the operations of the IO Server           *}

VAR
  Output : Text;   {* The file where the output results are printed *}

{**************************************************************************}
PROCEDURE ReadInputs
  (VAR ArrivalProbability,
       IORequestProbability : Real;
   VAR ProgramTime,
       IOServiceTime,
       TimeSlice,
       TimeLimit : Integer);
  {* Purpose : To read in the initial values needed for limits within the *}
  {*           program. Read in from the keyboard                         *}
  {*     Pre : Needed values unkown...                                    *}
  {*    Post : Initial values received and program specs known            *}

{**************************************************************************}
FUNCTION GetValue
  (Prompt : String;         {* The prompt to be used for the input *}
   MinValue,                {* Minimum value that can be entered   *}
   MaxValue : Real) : Real; {* Maximum value that can be entered   *}
  {* Purpose : To get a singe value and test for validity                 *}
  {*     Pre : Needed value unknown, and validity unknown                 *}
  {*    Post : Value found and is valed                                   *}

VAR
  ErrorPos : Integer;    {* Error position in the entered string  *}
  TempStr : STRING;      {* Temp String to be test for conversion *}
  Value : Real;          {* The resulting REAL value              *}

BEGIN {* GetValue *}

  REPEAT
    {* Write the prompt and read in test value *}
    write (Prompt);
    readln (TempStr);
    val (TempStr, Value, ErrorPos);

    {* If it is a valid number, check for range, otherwise error *}
    IF ErrorPos <> 0 THEN
        BEGIN
          writeln;
          writeln ('* ERROR, invalid data entered: Please try again *')
        END
      ELSE
        IF (Value > MinValue) AND (Value < MaxValue) THEN
            GetValue := Value
          ELSE
            BEGIN
              ErrorPos := 1;
              writeln;
              writeln ('* ERROR, the value must be between ',
                        trunc(MinValue), ' and ', trunc(MaxValue), ' *');
            END;
  UNTIL ErrorPos = 0;

END; {* GetValue *}

{**************************************************************************}

BEGIN {* ReadInputs *}

  {* Reads all the values needed from the user. Self explanitory *}
  clrscr;
  TimeLimit :=
    trunc(GetValue('Enter the Time Limit of this Program ==> ',0,MaxInt));
  ProgramTime :=
    trunc(GetValue('Enter the Program Service Time ==> ',0,TimeLimit));
  IOServiceTime :=
    trunc(GetValue('Enter the I/O Service Time ==> ',0,ProgramTime));
  TimeSlice :=
    trunc(GetValue('Enter the Time Slice Length ==> ',
          0,(ProgramTime/MaxPriority)));
  ArrivalProbability :=
    GetValue('Enter (Arrival Probability/Clock Tick) ==> ',0,1);
  IORequestProbability :=
    GetValue('Enter (I/O Request Probability/Clock Tick) ==> ',0,1);

END; {* ReadInputs *}

{***************************************************************************}
PROCEDURE PrintHeader
  (ArrivalProbability : Real;
   IORequestProbability : Real;
   ProgramTime : Integer;
   IOServiceTime : Integer;
   TimeSlice : Integer;
   TimeLimit : Integer);
  {* Purpose : To print to the Output file the input data                 *}
  {*     Pre : Input data has been read and is valid                      *}
  {*    Post : Printed to the Output file                                 *}

BEGIN {* PrintHeader *}
  {* Prints all the values read in from the user to Output *}
  writeln (Output, '----------------------------------');
  writeln (Output, ' INPUT DATA FROM USER');
  writeln (Output, '----------------------------------');
  writeln (Output, ' Arrival Probability......... ',
           trunc(ArrivalProbability*100), '%');
  writeln (Output, ' I/O Request Probability..... ',
           trunc(IORequestProbability*100), '%');
  writeln (Output, ' Program Time................ ', ProgramTime);
  writeln (Output, ' I/O Service Time............ ', IOServiceTime);
  writeln (Output, ' Time Slice.................. ', TimeSlice);
  writeln (Output, ' Time Limit.................. ', TimeLimit);
  writeln (Output);
  writeln (Output);
  writeln (Output, '----------------------------------');
  writeln (Output, ' RUNNING PROGRAM DATA');
  writeln (Output, '----------------------------------');
END; {* PrintHeader *}

{***************************************************************************}
PROCEDURE PrintStatus
  (ReadyQueue : ReadyType;
   Blocked : QueueType;
   Clock : Integer);
  {* Purpose : Prints the entire contents of the queues and clock         *}
  {*     Pre : Program has begun and clock begins ticking                 *}
  {*    Post : Information at this clock tick is printed to Output        *}

VAR
  Count : PriorityRange;  {* Loop control variable *}

BEGIN {* PrintStatus *}

  {* Print the clock counter *}
  writeln (Output);
  writeln (Output, ' Clock ==> ', Clock);

  {* Print the data in ReadyQueue *}
  FOR Count := 1 TO MaxPriority DO
    BEGIN
      write (Output, ' Priority ', Count, ' ==> ');
      IF NOT EmptyQueue(ReadyQueue[Count]) THEN
          PrintQueue (ReadyQueue[Count],Output)
        ELSE write (Output, 'Empty');
      writeln (Output);
    END;

  {* Print the data in Blocked *}
  write (Output, ' I/O Requests ==> ');
  IF NOT EmptyQueue(Blocked) THEN
      PrintQueue (Blocked,Output)
    ELSE write (Output, 'Empty');
  writeln (Output);

END; {* PrintStatus *}

{***************************************************************************}
PROCEDURE PrintSummary
  (ProgramsServiced : Integer; {* Number of programs serviced by OS *}
   TotalWaitTime : Integer);   {* Total overall wait time           *}
  {* Purpose : To print the results of the program                        *}
  {*     Pre : Program has finished and proper execution                  *}
  {*    Post : Results have been calculated and outputed                  *}

BEGIN {* PrintSummary *}

  {* Write the summary to the Output File *}
  writeln (Output);
  writeln (Output, '-------------------------------------');
  writeln (Output, ' Programs Finished ==> ', ProgramsServiced);
  IF ProgramsServiced > 0 THEN
      writeln (Output, ' Average Wait Time ==> ',
               (TotalWaitTime/ProgramsServiced):0:2, ' clicks')
    ELSE writeln (Output, ' Average Wait Time ==> 0');
  writeln (Output, ' Total Wait Time ==> ', TotalWaitTime);

  {* Write the summary to the screen *}
  writeln;
  writeln ('-------------------------------------');
  writeln ('Programs Finished ==> ', ProgramsServiced);
  IF ProgramsServiced > 0 THEN
      writeln ('Average Wait Time ==> ',
               (TotalWaitTime/ProgramsServiced):0:2, ' clicks')
    ELSE writeln ('Average Wait Time ==> 0');
  writeln ('Total Wait Time ==> ', TotalWaitTime);
  readln;

END; {* PrintSummary *}

{***************************************************************************}
FUNCTION Random
  (VAR Seed : Real) : Real;
  {* Purpose : To get a new value based on seed to get a semi random num  *}
  {*     Pre : Seed is initialized and has a value                        *}
  {*    Post : Seed has been midified and Random is := midification       *}

CONST
  Pi = 3.14159;  {* Equivalent to Pi in mathematics *}

VAR
  Temp : Real;   {* Temporary value to be updated *}

BEGIN {* Random *}

  Temp := Seed + Pi;
  Temp := exp(5.0 * ln(Temp));
  Seed := Temp - trunc(Temp);
  Random := Seed;

END; {* Random *}

{***************************************************************************}
PROCEDURE CreatePriority
  (VAR Priority : ListType);
  {* Purpose : Creat the priority list from Priority.seq                  *}
  {*     Pre : Priority is needed and Priority.seq is error free          *}
  {*    Post : Priority is complete and circle list is ready              *}

VAR
  Temp : PriorityRange;  {* Temp value to be inserted    *}
  PriorityFile : Text;   {* File with priorityies listed *}

BEGIN {* CreatePriority *}

  {* Get file ready to be read *}
  assign (PriorityFile, 'priority.seq');
  reset (PriorityFile);
  CreateList(Priority);

  {* Read in values from file *}
  WHILE NOT Eof(PriorityFile) DO
    BEGIN
      readln (PriorityFile, Temp);
      InsertElement (Priority, Temp);
    END;
  close (PriorityFile);

END; {* CreatePriority *}

{***************************************************************************}
PROCEDURE Initializations
  (VAR Seed,
       AverageWaitTime : Real;
   VAR CPU,
       IOChannel,
       IOServiceTime,
       ProgramsServiced,
       TotalWaitTime,
       CurrentID,
       CPUNeed : Integer;
   VAR ProgramGoing,
       IOGoing : Boolean);
  {* Purpose : To initializa all relevent data at beginning of program    *}
  {*     Pre : All vars have not been initialized                         *}
  {*    Post : Vars are initialized and ready to begin program            *}

BEGIN {* Initializations *}

  {* Set all vars to needed starting values *}
  Seed := 4.0;
  CPU := 0;
  IOChannel := IOServiceTime;
  ProgramsServiced := 0;
  TotalWaitTime := 0;
  AverageWaitTime := 0;
  CurrentID := 1000;
  CPUNeed := 0;
  ProgramGoing := False;
  IOGoing := False;

END; {* Initializations *}

{***************************************************************************}
PROCEDURE Create
  (VAR ReadyQueue : ReadyType;
   VAR Blocked : QueueType;
   VAR Priority : ListType);
  {* Purpose : To call all creating procedures for list and queues        *}
  {*     Pre : Queues and lists are not created                           *}
  {*    Post : Queues and lists are created and empty                     *}

VAR
  Count : PriorityRange;  {* Loop control variable *}

BEGIN {* Create *}

  {* Create the Priority list *}
  CreatePriority (Priority);

  {* Create the queues *}
  FOR Count := 1 TO MaxPriority DO CreateQueue (ReadyQueue[Count]);
  CreateQueue (Blocked);

END; {* Create *}

{***************************************************************************}
PROCEDURE Destroy
  (VAR ReadyQueue : ReadyType;
   VAR Blocked : QueueType;
   VAR Priority : ListType);
  {* Purpose : To destroy all dynamic data structures                     *}
  {*     Pre : Data structures have been created                          *}
  {*    Post : Data structures are destroyed and memory is freed          *}

VAR
  Count : PriorityRange;  {* Loop control variable *}

BEGIN {* Destroy *}

  {* Destroy the Priority list *}
  DestroyList (Priority);

  {* Destroy the queues *}
  FOR Count := 1 TO MaxPriority DO DestroyQueue (ReadyQueue[Count]);
  DestroyQueue (Blocked);

END; {* Destroy *}

{***************************************************************************}
PROCEDURE ControlProcessing;
  {* Purpose : Main controlling procedure where decisions are made        *}
  {*     Pre : Program is ready to start                                  *}
  {*    Post : Program completed and all results are outputed             *}

VAR
  Priority,                        {* The list of priorities              *}
  CurrentPriority : ListType;      {* The current priority being accessed *}
  ReadyQueue : ReadyType;          {* The array of queues of programs     *}
  Blocked : QueueType;             {* Queue for IO operation programs     *}
  InIO,                            {* Element currently doing I/O action  *}
  InExecution : QueueElementType;  {* Element currently being processed   *}
  TimeLimit,                       {* Time limit of the clock             *}
  Clock,                           {* Clock to control look length        *}
  CPU,                             {* The CPU counter for time slices     *}
  IOChannel,                       {* Time counter for IO operations      *}
  TimeSlice,                       {* Time given to a program for exec    *}
  ProgramTime,                     {* Time needed for program to complete *}
  IOServiceTime,                   {* Length of an I/O operation          *}
  ProgramsServiced,                {* Total of programs completed run     *}
  TotalWaitTime,                   {* Total wait time of programs done    *}
  CurrentID,                       {* Most recent ID Number               *}
  Count : Integer;                 {* Loop control variable               *}
  ArrivalProbability,              {* Program Arrive/Clock Tick           *}
  IORequestProbability,            {* I/O Request/Clock Tick              *}
  AverageWaitTime,                 {* Average wait time of programs done  *}
  Seed : Real;                     {* Seed value for random num generator *}
  IOGoing,                         {* Boolean for if IO program running   *}
  ProgramGoing : Boolean;          {* Boolean for if program executing    *}

BEGIN {* ControlProcessing *}

  {* Get output file ready for data *}
  assign (Output, 'output.rep');
  rewrite (Output);

  {* All creations and initializations *}
  Create (ReadyQueue,Blocked,Priority);
  CurrentPriority := Priority;
  ReadInputs (ArrivalProbability,IORequestProbability,ProgramTime,
              IOServiceTime,TimeSlice,TimeLimit);
  Initializations (Seed,AverageWaitTime,CPU,IOChannel,IOServiceTime,
                   ProgramsServiced,TotalWaitTime,CurrentID,
                   InExecution.CPUNeed,ProgramGoing,IOGoing);

  {* Print header for Output *}
  PrintHeader (ArrivalProbability,IORequestProbability,ProgramTime,
               IOServiceTime,TimeSlice,TimeLimit);

  {* Main program time limit control loop *}
  FOR Clock := 1 TO TimeLimit DO
    BEGIN {* For Clock *}

      {* Print out current status of queues and programs in operation *}
      PrintStatus (ReadyQueue,Blocked,Clock);
      IF ProgramGoing THEN
          writeln (Output, ' Program ', InExecution.IDNum, ' is running')
        ELSE writeln (Output,' No program in execution');

      {* Check for a new program arrival *}
      IF Random(Seed) <= ArrivalProbability THEN
          NewProgram (Output,ReadyQueue,CurrentPriority,CurrentID,
                      ProgramTime,Clock);

      {* Check for operating program actions *}
      IF ProgramGoing THEN
          BEGIN {* IF ProgramGoing *}

            {* Reset CPU if Program finished *}
            IF (InExecution.CPUNeed = 0) AND (CPU > 0) THEN CPU := 0;

            {* If end program then dispose of it otherwise it waits again *}
            IF (CPU = 0) THEN
                BEGIN {* IF CPU=0 *}
                  IF (InExecution.CPUNeed = 0) THEN
                      ProgramFinish (Output,TotalWaitTime,ProgramsServiced,
                                     InExecution,Clock)
                    ELSE ProgramNotDone (ReadyQueue,InExecution);
                  writeln (Output, ' End of time slice for Program ',
                           InExecution.IDNum);
                END; {* IF CPU=0 *}

          END; {* IF ProgramGoing *}

      {* If CPU free then get another program in execution, if needed *}
      IF (CPU = 0) THEN
          CPUFree (CPU,InExecution,ReadyQueue,ProgramGoing,TimeSlice);

      {* If I/O program running and done then I/O channel is freed *}
      IF (IOGoing) AND (IOChannel = 0) THEN
          IOFree (Output,IOChannel,Blocked,ReadyQueue,IOGoing,IOServiceTime);

      {* If a program is operating, check for I/O Request *}
      IF ProgramGoing THEN
          IF (Random(Seed) <= IORequestProbability) THEN
              BEGIN {* IF Random(Seed)... *}
                IORequest (Output,CPU,Blocked,InExecution);
                IOGoing := True;
                ProgramGoing := False;
              END; {* IF Random(Seed)... *}

      {* If a program is operating, decrement CPU and CPU Needed *}
      IF ProgramGoing THEN
          BEGIN {* IF ProgramGoing *}
            IF CPU > 0 THEN dec(CPU);
            IF InExecution.CPUNeed > 0 THEN dec(InExecution.CPUNeed);
          END; {* IF ProgramGoing *}

      {* If I/O program going, decrement the I/O Needed *}
      IF IOGoing AND (IOChannel > 0) THEN dec(IOChannel);

    END; {* For Clock *}

  {* Print final results to screen and Output *}
  PrintSummary (ProgramsServiced,TotalWaitTime);

  {* Destroy all dynamic structures and close Output file *}
  Destroy (ReadyQueue,Blocked,Priority);
  close (Output);

END; {* ControlProcessing *}

{***************************************************************************}

BEGIN {* Program3 *}

  ControlProcessing;

END. {* Program3 *}