                 {* Charlie Wiederhold - Gooty Wed. 3-4 *}
{***************************************************************************}
{*                                                                         *}
{* Author     : Charlie Wiederhold                                         *}
{* TA         : Gooty - Wed. 3-4                                           *}
{* Assignment : Program 4                                                  *}
{* Date       : April 20, 1996                                             *}
{* Purpose    : To maintain a catalog of books for a library using two     *}
{*              binary search trees to access the books and a linked list  *}
{*              to store the actual data.                                  *}
{*                                                                         *}
{*(*************************************************************************}

PROGRAM Program4;

USES
  Utility,      {* Contains many of my often used routines                 *}
  ListData,     {* Declaration of ListElementType stuff                    *}
  TreeData,     {* Declaration of TreeElementType stuff                    *}
  Prog4Data,    {* Main declarations for Program4                          *}
  ListADT,      {* ADT for the lists                                       *}
  TreeADT,      {* ADT for the search trees                                *}
  Basics,       {* The very basic routines for Program4                    *}
  Advanced;     {* The more in depth routines for Program4                 *}

VAR
  LogFile : Text;

{***************************************************************************}
PROCEDURE InsertBook
  (VAR BookList  : ListType;     {* Master list of books                   *}
   VAR Trees     : IndexTrees;   {* Trees to be inserted into              *}
   VAR CurrentID : Integer);     {* ID number of the current book loaded   *}
{* Purpose : To read a book from the keyboard and insert into the program  *}
{* Pre     : Book is not in the program records                            *}
{* Post    : Book is in the trees and the master list                      *}

VAR
  OldNode,                       {* Old node in tree if it exists          *}
  TreeNode    : TreeElementType; {* Node within the tree to be accessed    *}
  NewBook     : ListElementType; {* New book for insertion                 *}
  Count       : Integer;         {* Loop control variable                  *}
  Found       : Boolean;         {* Boolean checking value                 *}
  AuthorNames : AuthorArray;     {* Array of authors                       *}

BEGIN {* InsertBook *}

  writeln (LogFile,'InsertBook Selected');
  writeln (LogFile,'-------------------');

  {* Get the book to be inserted and upcase it all for ease of comparing *}
  InputWindow (Insert,NewBook);
  FOR Count := 1 TO 60 DO
    BEGIN {* For *}
      NewBook.BookNode.Title[Count] := upcase(NewBook.BookNode.Title[Count]);
      NewBook.BookNode.Authors[Count] := upcase(NewBook.BookNode.Authors[Count]);
    END; {* For *}
  NewBook.BookNode.IDNum := CurrentID + 1;
  RetrieveElement (BookList,NewBook,Found);

  {* Assign an ID, and update the trees and the master list *};
  IF NOT(Found) THEN
      BEGIN {* If *}
        inc (CurrentID);
        InsertElement (BookList,NewBook);
        ResetList (BookList);
        UpdateTree (NewBook.BookNode.Title,TreeNode,OldNode,BookList,Trees[Title]);

        BreakUpAuthors (NewBook.BookNode.Authors,AuthorNames);
        Count := 1;
        WHILE AuthorNames[Count] <> '' DO
          BEGIN {* While *}
            UpdateTree (AuthorNames[Count],TreeNode,OldNode,BookList,Trees[Author]);
            inc(Count);
          END; {* While *}

        writeln (LogFile,TitleHead,NewBook.BookNode.Title);
        writeln (LogFile,AuthorHead,NewBook.BookNode.Authors);
      END {* If *}
    ELSE writeln (LogFIle,'Book Already Inserted');

  writeln (LogFile);

END; {* InsertBook *}

{***************************************************************************}
PROCEDURE Search_Delete
  (VAR BookList : ListType;      {* Master list of books                   *}
   VAR Trees    : IndexTrees;    {* Trees the be traversed                 *}
       Process  : ProcessMethod);{* What process to do on the books        *}
{* Purpose : To search for a delete a book, depending on the user choice   *}
{* Pre     : Book is unknown or in the list                                *}
{* Post    : Book is known and printed or deleted dependant on process     *}

VAR
  SearchBook : ListElementType;  {* Book to be searching for               *}
  Count      : Integer;          {* Loop control variable                  *}

BEGIN {* Search_Delete *}

  CASE Process OF
    Remove   : writeln (LogFile,'DeleteBook Selected');
    PrintOut : writeln (LogFIle,'SearchBook Selected');
  END;

  writeln (LogFile,'-------------------');

  {* Find the search book and upcase for ease of comparing *}
  InputWindow (Process,SearchBook);
  FOR Count := 1 TO 60 DO
    BEGIN {* For *}
      SearchBook.BookNode.Title[Count] := upcase(SearchBook.BookNode.Title[Count]);
      SearchBook.BookNode.Authors[Count] := upcase(SearchBook.BookNode.Authors[Count]);
    END; {* For *}

  {* Determine the method of serching, depending on the user input *}
  IF (SearchBook.BookNode.Title <> '') AND (SearchBook.BookNode.Authors <> '') THEN
      CheckBoth (SearchBook,BookList,Trees,Process,LogFile);
  IF (SearchBook.BookNode.Title <> '') AND (SearchBook.BookNode.Authors = '') THEN
      CheckTitle (SearchBook,BookList,Trees,Process,LogFIle);
  IF (SearchBook.BookNode.Title = '') AND (SearchBook.BookNode.Authors <> '') THEN
      CheckAuthor (SearchBook,BookList,Trees,Process,LogFile);

  writeln (LogFile);
END; {* Search_Delete *}

{***************************************************************************}
PROCEDURE Initialize
  (VAR CurrentID : Integer;      {* ID number of the current book          *}
   VAR Cursor    : Byte;         {* Cursor position for the selection bar  *}
   VAR Quit      : Boolean;      {* Determines wheter to terminate prgrm   *}
   VAR BookList  : ListType;     {* Master list of books                   *}
   VAR Trees     : IndexTrees);  {* Search trees                           *}
{* Purpose : To initialize all values and create lists and trees           *}
{* Pre     : No initializations performed                                  *}
{* Post    : Program is ready to begin main execution                      *}

VAR
  BookFile : Text;               {* Text file to read data from            *}

BEGIN {* Initialize *}

  {* Initializes all values for new program exectution *}
  CurrentID := 1000;
  CreateList (BookList);
  CreateTree (Trees[Author]);
  CreateTree (Trees[Title]);
  GetBooks (Trees[Author],BookList,BookFile,CurrentID);
  CreateTitleTree (Trees[Author],Trees[Title]);
  Cursor := 1;
  Quit := False;

END; {* Initialize *}

{***************************************************************************}
PROCEDURE ShutDown
  (BookList : ListType;          {* Master list of books                   *}
   Trees    : IndexTrees);       {* Search trees                           *}
{* Purpose : To save current data, delete trees/lists, and exit            *}
{* Pre     : Data still unsaved etc.                                       *}
{* Post    : Info saved and program exited cleanly                         *}

VAR
  BookFile : Text;               {* Text file to save data to              *}

BEGIN {* ShutDown *}

  {* Saves all values and destroys all dynamic structures to free memory *}
  SaveBooks (Trees[Title],BookFile);
  DestroyList (BookList);
  DestroyTree (Trees[Author]);
  DestroyTree (Trees[Title]);
  Clear (DOSText,DOSBack,1,1,80,25);

END; {* ShutDown *}

{***************************************************************************}
PROCEDURE ControlProcessing;
{* Purpose : Main controlling module of the entire program                 *}

VAR
  BookList  : ListType;              {* Master list of books               *}
  Trees     : IndexTrees;            {* Search trees contain book nodes    *}
  Quit      : Boolean;               {* Boolean checking value             *}
  CurrentID : Integer;               {* ID Number of the current book      *}
  Cursor    : Byte;                  {* Position of the cursor in the menu *}

BEGIN {* ControlProcessing *}

  writeln (LogFile,'Program entered');
  writeln (LogFile,'---------------');
  writeln (LogFile);

  Initialize (CurrentID,Cursor,Quit,BookList,Trees);

  REPEAT
    ScreenSetup;

    {* Find out selection and act upon it *}
    Select (SelectColor,WinColor,6,4,2,1,Cursor);
    CASE Cursor OF
      1 : InsertBook (BookList,Trees,CurrentID);
      2 : Search_Delete (BookList,Trees,Remove);
      3 : Search_Delete (BookList,Trees,PrintOut);
      4 : Quit := QuitProgram;
    END; {* Case *}
  UNTIL Quit;

  ShutDown (BookList,Trees);

  writeln (LogFile,'Program exited');
  writeln (LogFile,'--------------');
END; {* ControlProcessing *}

{***************************************************************************}

BEGIN {* Program4 *}

  assign (LogFile, 'program4.log');
  rewrite (LogFile);

  ControlProcessing;

  close (LogFile);
END. {* Program4 *}