PROGRAM CrossWords;

{****************************************************************************}
{*                                                                          *}
{*  Author     : Charlie Wiederhold                                         *}
{*  Assignment : Supplemental Program 17                                    *}
{*  Purpose    : This program reads two words from the user, and prints     *}
{*               them in a cross pattern or prints an error message if no   *}
{*               common letters are found.                                  *]
{*                                                                          *}
{****************************************************************************}

USES
  Crt;  {* Used to clear the screen *}

CONST
  OrdEnter  = false;       {* Used to check ord value of Enter on machine *}
  MaxLength = 27;          {* Maximum length of each word *}

TYPE
  WordLength = 1..MaxLength;      {* Range of length of word *}
  TestLength = 0..MaxLength;      {* Range of testing lengths for words *}
  {* Array of all capital letters for each word *}
  Words      = ARRAY [WordLength] OF 'A'..'Z';
  {* Record of word and length of the word *}
  WordType   = RECORD
                 Letters : Words;
                 Length  : integer;
               END;

VAR
  Word1,                   {* First word read *}
  Word2   : WordType;      {* Second word read *}
  Common1,                 {* Common position in first word *}
  Common2 : TestLength;    {* Common position in second word *}


{****************************************************************************}
{*                                                                          *}
{* Procedure FindOrdEnter reads the value of Test and prints the ordinal    *}
{* value of it. I used this to test what the ordinal value of enter was to  *}
{* test for the end of the word.                                            *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE FindOrdEnter;

VAR
  Test : char;  {* Char to be tested for ordinal value *}

BEGIN {* FindOrdEnter *}

  {* Clears screen -non standard pascal-*}
  clrscr;

  {* Reads the key and prints the ordinal value *}
  read (Test);
  writeln (ord(Test));
  readln;

END; {* FindOrdEnter *}

{****************************************************************************}
{*                                                                          *}
{* Procedure ReadIt reads the word from the user, and applies it to array   *}
{* letters, and finds the length of the word.                               *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE ReadIt (VAR Word : WordType);

VAR
  Count : TestLength;  {* Counter variable to keep track of position *}

BEGIN {* ReadIt *}

  {* Initialize Count *}
  Count := 0;

  {* Reads each char and assigns it to its position in the array *}
  REPEAT
    Count := Count + 1;
    read (Word.Letters[Count]);
  UNTIl (Word.Letters[Count] = chr(10)) OR (Count = 27);

  {* I had to use the watch function on the compiler to find what the extra *}
  {* value I was receiving after every word was. By doing so I noticed that *}
  {* the read function was reading the Line Feed Char #10 after enter was   *}
  {* pressed. The reason the extra 2 values are allowed in the maximum      *}
  {* is because of the characters #13 and #10 being read and assigned to    *}
  {* the array of chars. This allows the user to actually type 25 letters   *}
  {* as specified in the prompt, without any errors occuring in the reading *}
  {* of the words. 25 just seemed like a reasonable number considering few  *}
  {* words would be over 25 letters long. *}

  {* Finds the length of the word *}
  Word.Length := Count - 2;

END; {* ReadIt *}

{****************************************************************************}
{*                                                                          *}
{* Procedure ReadWords calls procedure ReadIt to read the words and find    *}
{* length of the words for both words.                                      *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE ReadWords (VAR Word1, Word2 : WordType);

BEGIN {* ReadWords *}

  {* Clears screen -non standard pascal-*}
  clrscr;

  {* Prompts for word and reads word by calling procedure Readit *}
  writeln ('Each word can only be a max of 25 letters and all capitals.');
  writeln;
  write ('Enter first word  ==> ');
  ReadIt (Word1);
  write ('Enter second word ==> ');
  ReadIt (Word2);

END; {* ReadWords *}

{****************************************************************************}
{*                                                                          *}
{* Procedure FindCommon finds the common letter positions in each of the    *}
{* words so that they can either be connected at those links or an error    *}
{* message can be displayed if no common letters are found.                 *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE FindCommon (VAR Common1, Common2 : TestLength;
                          Word1, Word2 : WordType);

VAR
  Count,                {* Counter variable for word 1 *}
  Counter : WordLength; {* Counter variable for word 2 *}

BEGIN {* FindCommon *}

  {* Initialize common positions to 0 for no common results *}
  Common1 := 0;
  Common2 := 0;

  {* Finds the last common letter in both words, or no common letter *}
  FOR Count := 1 TO Word1.Length DO
    FOR Counter := 1 TO Word2.Length DO
      IF Word1.Letters[Count] = Word2.Letters[Counter]
        THEN
          BEGIN
            Common1 := Count;
            Common2 := Counter;
          END;

END; {* FindCommon *}

{****************************************************************************}
{*                                                                          *}
{* Procedure WriteWords writes the words in the cross form with word1 going *}
{* down, and word2 written horizontally at the point where they cross.      *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE WriteWords (Common1, Common2 : WordLength; Word1, Word2 : WordType);

VAR
  Count,                {* Counter variable for word 1 *}
  Counter : WordLength; {* Counter variable for wrod 2 *}

BEGIN {* WriteWords *}

  {* Spaces output away from prompts *}
  writeln;
  writeln;

  {* Writes letters of word 1 vertically until the common letter is found *}
  {* Writes spaces before word 1 to space out for common length in word 2 *}
  FOR Count := 1 TO (Common1 - 1) DO
    FOR Counter := 1 TO Common2 DO
      IF Counter = Common2
        THEN Writeln (Word1.Letters[Count])
        ELSE Write (' ');

  {* Writes all of word 2 at common link in word 1 *}
  FOR Count := 1 TO Word2.Length DO
    write (Word2.Letters[Count]);
  writeln;

  {* Finishes word 1 in same manner as first loop *}
  FOR Count := (Common1 + 1) TO Word1.Length DO
    FOR Counter := 1 to Common2 DO
      IF Counter = Common2
        THEN writeln (Word1.Letters[Count])
        ELSE write (' ');

  {* Hold output on screen *}
  readln;

END; {* WriteWords *}

{****************************************************************************}
{*                                                                          *}
{* Procedure Error prints an error message if no common letters are found.  *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE Error;

BEGIN {* Error *}


  {* Spaces output away from prompts *}
  writeln;
  writeln;

  {* Error message *}
  writeln ('There were no letters that matched between these two words.');
  readln;

END; {* Error *}

{****************************************************************************}

BEGIN {* CrossWords *}

  {* Test for value of enter *}
  IF OrdEnter THEN FindOrdEnter;

  {* Reads the words *}
  ReadWords (Word1,Word2);

  {* Finds the common letter positions for the words *}
  FindCommon (Common1,Common2,Word1,Word2);

  {* Prints error message if no common letters found, or else prints the *}
  {* words in a cross pattern *}
  IF (Common1 <> 0) AND (Common2 <> 0)
    THEN WriteWords (Common1,Common2,Word1,Word2)
    ELSE Error;

END. {* CrossWords *}