PROGRAM LINKS;

{//////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 /                                                                          \
 / Program   : LINKS                                                        \
 / Author    : Charles T. Wiederhold                                        \
 / Date      : 06/27/95                                                     \
 / Purpose   : This program allows the user to enter as many string values  \
 /             as they need, and then the values are stored in a text file  \
 /             DATA.TXT. Then the values are read into a linked list, and   \
 /             printed out on the screen.                                   \
 / Structure : 4 Modules                                                    \
 /             GetNodes  : Reads values enterd by user and saves them.      \
 /             ReadNodes : Reads from the file and creates the linked list. \
 /             PrintNodes: Prints out all nodes in the linked list.         \
 /             Main      : Calls all of the modules to be run.              \
 /                                                                          \
 //////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\}

USES CRT;

TYPE
  {** Pointer type for nodes created **}
  NodePointer = ^NodeType;

  {** Record data pointed to by NodePointer **}
  NodeType    = RECORD
                  Component : STRING;
                  Link : NodePointer;
                END;

VAR
  List,                      {** External pointer and first component      **}
  Current,                   {** Current pointer and component in list     **}
  NewNode : NodePointer;     {** Newly created node to be added to list    **}
  Data    : Text;            {** File of string components created by user **}

{//////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 /                                                                          \
 / This procedure reads the components entered by the user and saves them   \
 / to a text file name DATA.TXT to be read later to print out the data.     \
 /                                                                          \
 //////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\}

PROCEDURE GetNodes;

VAR
  Component : string;        {** Entries entered by user and saved to Data **}

BEGIN {** GetNodes **}

  {* SET UP SCREEN *}
  textcolor(15);
  textbackground(1);
  clrscr;

  {* OPEN FILE *}
  assign (Data, 'c:data.txt');
  rewrite (Data);

  {* READ AND WRITE ALL COMPONENTS TO FILE *}
  write ('Enter new component ==> ');
  readln (Component);
  IF ((Component = 'x') OR (Component = 'x'))
    THEN writeln (Data, 'No Entries')
    ELSE
      WHILE ((Component <> 'X') AND (Component <> 'x')) DO
        BEGIN
          writeln (Data, Component);
          write ('Enter new component ==> ');
          readln (Component);
        END;

  {* CLOSE FILE *}
  close (Data);

END; {** GetNodes **}

{//////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 /                                                                          \
 / This procedure reads all the data from DATA.TXT and creates the linked   \
 / list to be printed out later.                                            \
 /                                                                          \
 //////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\}

PROCEDURE ReadNodes (VAR List : NodePointer);

VAR
  Current : NodePointer;

BEGIN {** ReadNodes **}

  {* OPEN FILE *}
  assign (Data, 'c:data.txt');
  reset (Data);

  {* CREATE NEW LIST *}
  new (List);
  Current := List;
  readln (Data, List^.Component);

  {* READ ALL NODES IN FILE *}
  WHILE NOT EOF (Data) DO
    BEGIN

      {* CREATE NEW NODE TO LIST *}
      new (NewNode);

      {* GET INFO FOR NODE *}
      readln (Data, NewNode^.Component);

      {* LINK NODE TO LIST *}
      Current^.Link := NewNode;

      {* UPDATE CURRENT POSITION TO NEW NODE *}
      Current := NewNode;

    END;

  {* SET LINK TO END OF LIST MARKER *}
  Current^.Link := NIL;

  {* CLOSE FILE *}
  close (Data);

END; {** ReadNodes **}

{/////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 /                                                                          \
 / This procedure simply prints the linked list created from DATA.TXT.      \
 /                                                                          \
 //////////////////////////////////////\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\}

PROCEDURE PrintNodes (List : NodePointer);

VAR
  Count   : Byte;            {** Counting variable                         **}
  Current : NodePointer;     {** Current node being printed                **}

BEGIN {** PrintNodes **}

  {* SET UP SCREEN *}
  textcolor(15);
  textbackground(1);
  clrscr;

  {* WRITE ALL COMPONENTS IN FILE *}
  Count := 0;
  Current := List;
  WHILE Current <> NIL DO
    BEGIN
      inc(Count);
      writeln ('Component ', count:2, ' : ', Current^.Component);
      current := Current^.link;
    END;

  {** HOLD OUTPUT ON SCREEN **}
  writeln;
  write ('Press any key to exit.');
  repeat until keypressed;

END; {** PrintNodes **}

{============================================================================}

BEGIN {** LINKS **}

  {** CREATE NEW FILE **}
  GetNodes;

  {* READ FILE AND CREATE LINKED LIST *}
  ReadNodes (List);

  {* PRINT OUT DATA *}
  PrintNodes (List);

END. {** LINKS **}