PROGRAM RomanNumeral;

{****************************************************************************}
{*                                                                          *}
{*  Author     : Charlie Wiederhold                                         *}
{*  Date       : October 23, 1995                                           *}
{*  Assignment : Supplemental Problem Number 4                              *}
{*  Purpose    : This program reads an integer between 1 and 1000 and       *}
{*               converts it to roman numeral format.                       *}
{*                                                                          *}
{****************************************************************************}

USES
  Crt;                          {* Unit used to clear the screen *}

CONST
  One         = 'I';            {* These next few constants are *}
  Five        = 'V';            {* representations of roman numerals *}
  Ten         = 'X';            {* compared to their integer counterparts. *}
  Fifty       = 'L';            {* These are used to construct the entire *}
  OneHundred  = 'C';            {* roman numeral for any integer entered *}
  FiveHundred = 'D';            {* between 1 and 1000. *}
  OneThousand = 'M';
  Max         = 9;              {* Longest roman numeral from 1 to 1000 *}

TYPE
  Integers = 1..1000;           {* Range of integers entered by user *}
  Length   = 1..Max;            {* Length of longest roman numeral *}
  {* String representation of the roman numerals *}
  Roman    = PACKED ARRAY [Length] OF char;

{****************************************************************************}
{*                                                                          *}
{*  Proceure ReadNumer is designed to read an integer and pass it back to   *}
{*  the MainModule for modification.                                        *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE ReadNumber (VAR DecimalNumeral : Integers);

BEGIN {* Read Number *}

  {* Prompts the user to enter a number, and then reads it. *}
  writeln;
  write ('Enter an integer between 1 and 1000 ==> ');
  readln (DecimalNumeral);

END; {* Read Number *}

{****************************************************************************}
{*                                                                          *}
{*  Procedure AddLetter is where the roman numeral is inserted into the     *}
{*  array in its proper position to represent the integer number. Then the  *}
{*  variable test is modified to be current test value minus the value that *}
{*  is added to the roman numeral.                                          *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE AddLetter (VAR Position : Length; VAR RomanNumeral : Roman;
                     AddTo : char; VAR Test : Integers; Update : integer);

BEGIN {* Add Letter *}

  {* Adds the new letter to the array *}
  RomanNumeral[Position] := AddTo;

  {* Updates the position in the array *}
  inc(Position);

  {* Updates test to the new test value *}
  Test := Test - Update;

END; {* Add Letter *}

{****************************************************************************}
{*                                                                          *}
{*
{*                                                                          *}
{****************************************************************************}

PROCEDURE ConvertToRomanNumeral (DecimalNumeral : Integers;
                                 VAR RomanNumeral : Roman);

VAR
  Test     : Integers;   {* Value modified while adding letters *}
  Count,                 {* Loop control variable *}
  Position : Length;     {* Current position in the array *}

BEGIN {* ConvertToRomanNumeral *}

  {* Initialize the entire array to blanks to get rid of giberish *}
  FOR Position := 1 TO Max DO
    RomanNumeral[Position] := ' ';

  {* Initialize Test to starting value and beginning position in array *}
  Test := DecimalNumeral;
  Position := 1;

  {* The next IF THEN statements all are used to slowly reduce the test *}
  {* to 0 while testing for the current value, and adding the corresponding *}
  {* roman numeral value when needed *}
  IF Test = 1000 THEN AddLetter (Position,RomanNumeral,OneThousand,Test,1000);

  IF Test >= 900 THEN
    BEGIN
      AddLetter (Position,RomanNumeral,OneHundred,Test,900);
      AddLetter (Position,RomanNumeral,OneThousand,Test,0);
    END;

  IF Test >= 500 THEN AddLetter (Position,RomanNumeral,FiveHundred,Test,500);

  IF Test >= 400 THEN
    BEGIN
      AddLetter (Position,RomanNumeral,OneHundred,Test,400);
      AddLetter (Position,RomanNumeral,FiveHundred,Test,0);
    END;

  IF Test >= 300 THEN FOR Count := 1 TO 3 DO
      AddLetter (Position,RomanNumeral,OneHundred,Test,100);

  IF Test >= 200 THEN FOR Count := 1 TO 2 DO
      AddLetter (Position,RomanNumeral,OneHundred,Test,100);

  IF Test >= 100 THEN AddLetter (Position,RomanNumeral,OneHundred,Test,100);

  IF Test >= 90 THEN
    BEGIN
      AddLetter (Position,RomanNumeral,Ten,Test,90);
      AddLetter (Position,RomanNumeral,OneHundred,Test,0);
    END;

  IF Test >= 50 THEN AddLetter (Position,RomanNumeral,Fifty,Test,50);

  IF Test >= 40 THEN
    BEGIN
      AddLetter (Position,RomanNumeral,Ten,Test,40);
      AddLetter (Position,RomanNumeral,Fifty,Test,0);
    END;

  IF Test >= 30 THEN FOR Count := 1 TO 3 DO
      AddLetter (Position,RomanNumeral,Ten,Test,10);

  IF Test >= 20 THEN FOR Count := 1 TO 2 DO
      AddLetter (Position,RomanNumeral,Ten,Test,10);

  IF Test >= 10 THEN AddLetter (Position,RomanNumeral,Ten,Test,10);

  IF Test >= 9 THEN
    BEGIN
      AddLetter (Position,RomanNumeral,One,Test,9);
      AddLetter (Position,RomanNumeral,Ten,Test,0);
    END;

  IF Test >= 5 THEN AddLetter (Position,RomanNumeral,Five,Test,5);

  IF Test >= 4 THEN
    BEGIN
      AddLetter (Position,RomanNumeral,One,Test,4);
      AddLetter (Position,RomanNumeral,Five,Test,0);
    END;

  IF Test >= 3 THEN FOR Count := 1 TO 3 DO
      AddLetter (Position,RomanNumeral,One,Test,1);

  IF Test >= 2 THEN FOR Count := 1 TO 2 DO
      AddLetter (Position,RomanNumeral,One,Test,1);

  IF Test = 1 THEN AddLetter (Position,RomanNumeral,One,Test,1);

END; {* ConvertToRomanNumeral *}

{****************************************************************************}
{*                                                                          *}
{*  Procedure WriteRomanNumeral writes the roman numeral out for display    *}
{*  during the output.                                                      *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE WriteRomanNumeral (RomanNumeral : Roman);

BEGIN {* WriteRomanNumeral *}

  writeln;
  writeln ('Converted to a Roman Number ==> ', RomanNumeral);
  readln;

END; {* WriteRomanNumeral *}

{****************************************************************************}
{*                                                                          *}
{*  Procedure MainModule is the main controlling module that calls the      *}
{*  three main modules to run the program.                                  *}
{*                                                                          *}
{****************************************************************************}

PROCEDURE MainModule;

VAR
  DecimalNumeral : Integers;  {* Decimal representation of the number *}
  RomanNumeral   : Roman;     {* Roman numeral representation of the number *}

BEGIN {* MainModule *}

  {* Reads the decimal number *}
  ReadNumber (DecimalNumeral);

  {* Converts decimal number to roman number *}
  ConvertToRomanNumeral (DecimalNumeral,RomanNumeral);

  {* Prints the output *}
  WriteRomanNumeral (RomanNumeral);

END; {* MainModule *}

{****************************************************************************}

BEGIN {* Main *}

  {* Clears the screen -non standard pascal- *}
  clrscr;

  {* Introduction to the program *}
  writeln ('This program reads an integer between 1 and 1000 and prints it ',
           'in Roman Numeral format.');

  {* Runs the main module *}
  MainModule;

END. {* Main *}