PROGRAM VolleyballSimulation;

{*****************************************************************************}
{*                                                                           *}
{*  Author      : Charlie Wiederhold                                         *}
{*  Date        : October 6, 1995                                            *}
{*  Assignment  : Supplemental Problem #11                                   *}
{*  Purpose     : To simulate a volleyball game between two teams. The games *}
{*                continue for 500 times, and each game continues unil one   *}
{*                team scores 15 or more points and is more than 2 points    *}
{*                of the opponent                                            *}
{*                                                                           *}
{*****************************************************************************}

USES
  Crt;                           {* Unit used to clear the screen *}

CONST
  MinWin = 15;                   {* Minimum score for a team to have to win *}
  WinBy  = 2;                    {* Amount a team must be ahead by to win *}
  Games  = 500;                  {* Number of games played *}

VAR
  ProbabilityA,                  {* Numerator in Probability of winning *}
  ProbabilityB,                  {* Denominator in Probability of winning *}
  TexasScore,                    {* Texas score per game *}
  AandMScore,                    {* A&M score per game *}
  TexasWins,                     {* Number of Texas wins *}
  AandMWins,                     {* Number of A&M wins *}
  TexasShutouts,                 {* Number of times Texas shutout A&M *}
  AandMShutouts,                 {* Number of times A& shutout Texas *}
  Volley,                        {* Represents the Volley *}
  Count          : integer;      {* Loop control variable *}
  TexasPercent,                  {* Percentage of Texas wins *}
  AandMPercent   : real;         {* Percentage of A&M wins *}
  TexasServe     : boolean;      {* True if Texas serve, False if A&M serve *}

BEGIN {* VolleyballSimulation *}

  {* Gets a new random seed *}
  randomize;

  {* Clears the screen -not standard Pascal- *}
  clrscr;

  {* Reads the probability from the user in a/b form *}
  writeln ('Enter probability of Texas winning in form of a/b.');
  writeln ('Example ==> 1/2 = 50%');
  writeln;
  write ('Enter a ==> ');
  readln (ProbabilityA);
  write ('Enter b ==> ');
  readln (ProbabilityB);

  {* Initialize totals for wins *}
  TexasWins := 0;
  AandMWins := 0;
  TexasShutouts := 0;
  AandMShutouts := 0;

  {* Continues for 500 games the simulation of games *}
  FOR Count := 1 TO Games DO

    BEGIN {* For loop - count *}

      {* Initialize scores to 0 and Texas serves first every game *}
      TexasScore := 0;
      AandMScore := 0;
      TexasServe := true;

      {* Simulate the playing of each game *}
      {* Repeats until a team is ahead by at least 2 and score is 15 or more *}
      REPEAT

        {* Pick a random number to simulate the volley *}
        Volley := random(ProbabilityB) + 1;

        {* Determines action taken after each volley *}
        if (TexasServe = True)
          then

            {* If Texas served, then determine score for Texas or a turnover *}
            Begin
              if Volley <= ProbabilityA
                then TexasScore := TexasScore + 1
                else TexasServe := False;
            End

          else

            {* If A&M served, then determine a score for A&M or a turnover *}
            Begin
              if Volley > ProbabilityA
                then AandMScore := AandMScore + 1
                else TexasServe := True;
            End;

      {* Ending condition for the end of a game *}
      UNTIL (((TexasScore - 2 >= AandMScore) and (TexasScore >= 15)) or
             ((AandMScore - 2 >= TexasScore) and (AandMScore >= 15)));

      {* Adds one to team who won the game *}
      if (TexasScore > AandMScore)
        then TexasWins := TexasWins + 1
        else AandMWins := AandMWins + 1;

      {* Adds one to shutout total of Texas if Texas shutout A&M *}
      if AandMScore = 0
        then TexasShutouts := TexasShutouts + 1;

      {* Adds one to shutout total of A&M if A&M shutout Texas *}
      if TexasScore = 0
        then AandMShutouts := AandMShutouts + 1;

    END;

  {* Calulates win percentages *}
  TexasPercent := ((TexasWins/Games) * 100);
  AandMPercent := ((AandMWins/Games) * 100);

  {* Outputs all totals *}
  writeln;
  writeln ('Texas wins ==> ', TexasWins);
  writeln ('A & M wins ==> ', AandMWins);
  writeln;
  writeln ('Texas win percentage ==> ', TexasPercent:4:1, '%');
  writeln ('A & M win percentage ==> ', AandMPercent:4:1, '%');
  writeln;
  writeln ('Times Texas shutout A & M ==> ', TexasShutouts);
  writeln ('Times A & M shutout Texas ==> ', AandMShutouts);

  {* Hold output on screen *}
  readln;

END. {* VolleyballSimulation *}
