program Gas_Mileage;

{****************************************************************************
 *  Author  : Wiederman Windows Inc.                                        *
 *  Date    : 1/8/95                                                        *
 *  Purpose : This program allows the user to enter an initial odometer     *
 *            reading, and then enter the odometer reading and amount of    *
 *            fuel put into the car to fill up the gas tank. The user is    *
 *            supposed to enter the readings after every gas stop.          *
 *  Design  : This program consists of the following modules:               *
 *              writeat - writes a string at an indicated position          *
 *              writeatr - writes a real at an indicated position           *
 *              writeati - writes an integer at an indicated position       *
 *              Get_Initial_Data - reads beginning odometer reading         *
 *              Get_Data - gathers the current odometer and fuel reading    *
 *              Calculate_Data - calculates all attributes of the data      *
 *              Output_Current_Data - outputs the data                      *
 *                 win - simplifies the window command                      *
 *              Final_Status - allows a loop to output the final chart      *
 *              Output_Final_Data - outputs the final data chart            *
 *              Data - organizes all the overall loops for the main program *
 *              Initialize - initializes opening values for the arrays and  *
 *                           counters                                       *
 *              RUN - organizes everything into one procedure               *
 *              MAIN - Driver module                                        *
 ****************************************************************************}

uses
  crt, wieder;  {Screen manipulation}

type
  odo_array   = array [1..7] of longint;     {array of odometer readings}
  tank_array  = array [1..7] of real;        {array of tank readings}
  trip_array  = array [1..7] of real;        {array of trip readings}
  user_record = record                       {record of all information}
                  odo_begin,                 {begining odometer readings}
                  odo_end    : odo_array;    {ending odometer readings}
                  fuel_tank,                 {fuel put in at fill-up}
                  miles_tank,                {miles since last fill up}
                  mpg_tank   : tank_array;   {mpg since last fill up}
                  fuel_trip,                 {fuel over entire trip}
                  miles_trip,                {miles over entire trip}
                  mpg_trip   : trip_array;   {mpg over entire trip}
                end;  {uder_record}

var
  user     : user_record;   {stores all user information}
  stops,                    {counts the number of stops}
  counter,                  {counter variable for loops}
  count    : integer;       {counter variable for loops}
  cont     : char;          {controls program run-time}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Get_Initial_Data                                    *
 *  Purpose : To get the initial odometer reading from the user             *
 *  Method  : Write statements prompt and read statements read the values   *
 *                                                                          *
 ****************************************************************************}

procedure Get_Initial_Data;

begin   {get_initial_data}

  {sets up the main screen window}
  clear (1,1,1,1,80,25);
  makewin (0,7,14,8,66,12,true,true,14);
  writeln;

  {reads the beginning odometer reading}
  with user do

    begin   {with}

      write (' Enter the initial odometer reading: ');
      readln (odo_begin[1]);

    end;    {with}

end;   {get_initial_data}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Get_Data                                            *
 *  Purpose : To gather the odometer an fuel readings at each stop          *
 *  Method  : Write statemnts prompt and read statements read the values    *
 *                                                                          *
 ****************************************************************************}

procedure Get_Data;

begin   {get_data}

  {sets up the screen for the windows}
  clear (1,1,1,1,80,25);
  makewin (0,7,14,6,66,12,true,true,14);
  writeln;

  {reads the current odometer reading, and the fuel put in}
  with user do

    begin   {with}

      write (' Enter the current odometer reading: ');
      readln (odo_end[count]);
      writeln;
      write (' Enter the fuel purchased: ');
      readln (fuel_tank[count]);

      {increments the number of fuel stops made}
      inc(stops);

    end;    {with}

end;   {get_data}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Calculate_Data                                      *
 *  Purpose : To calculate al attributes of the data entered                *
 *  Method  : Several equations assign values to their respective array     *
 *                                                                          *
 ****************************************************************************}

procedure Calculate_Data;

begin   {calculate_data}

  {makes the calculations for the specific stop and assign them to positions}
  with user do

    begin   {with}

      odo_begin[count + 1] := odo_end[count];
      miles_tank[count] := odo_end[count] - odo_begin[count];

      {checks for invalid data or assign statements when count is 1}
      if count = 1
        then

          begin   {then}

            fuel_trip[count] := fuel_tank[count];
            miles_trip[count] := miles_tank[count];

          end   {then}

        else

          begin   {else}

            fuel_trip[count] := fuel_trip[count - 1] + fuel_tank[count];
            miles_trip[count] := miles_trip[count - 1] + miles_tank[count];

          end;   {else}

      mpg_tank[count] := miles_tank[count] / fuel_tank[count];
      mpg_trip[count] := miles_trip[count] / fuel_trip[count];

    end;   {with}

end;   {calculate_data}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Output_Current_Data                                 *
 *  Purpose : To output the current data, and running totals of the trip    *
 *  Method  : Write statements arrange the output in a formatted way        *
 *                                                                          *
 ****************************************************************************}

procedure Output_Current_Data;

begin   {output_current_data}

  {sets up the opening windows and output}
  clear (1,1,1,1,80,25);

  makewin (0,7,33,3,51,5,true,true,11);
  makewin (0,7,8,9,38,17,true,true,14);
  makewin (0,7,46,9,73,17,true,true,14);

  regwin;

  {outputs the data}
  with user do

    begin   {with}

      {screen title}
      wrts (34,4,'Performance Chart');

      {outputs current status chart}
      wrts (17,9, 'Current Status');
      wrts (10,11, 'Odometer (Begin)');
      wrts (10,12,'Odometer (End)');
      wrts (10,13,'Fuel     (Tank)');
      wrts (10,14,'Miles    (Tank)');
      wrts (10,15,'M.P.G.   (Tank)');
      frmti (25,11,odo_begin[count],12);
      frmti (25,12,odo_end[count],12);
      wrtr (25,13,fuel_tank[count],12,1);
      wrtr (25,14,miles_tank[count],12,1);
      wrtr (25,15,mpg_tank[count],12,1);

      {outputs total status chart}
      wrts (55,9,'Trip Status');
      wrts (48,13,'Fuel   (Trip)');
      wrts (48,14,'Miles  (Trip)');
      wrts (48,15,'M.P.G. (Trip)');
      wrtr (62,13,fuel_trip[count],10,1);
      wrtr (62,14,miles_trip[count],10,1);
      wrtr (62,15,mpg_trip[count],10,1);

      {continue option}
      linewin (0,7,32,47,21,1);
      write (' Continue (Y/N)');
      hidecursor (1,1,7);

    end;   {with}

  {reads continue option}
  cont := readkey;

  {increments the array assignment variable}
  inc(count);

end;   {output_current_data}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Final_Status                                        *
 *  Purpose : To allow a loop be used to output the final chart             *
 *  Method  : Write statements place the ouput where it belongs             *
 *                                                                          *
 ****************************************************************************}

procedure Final_Status (var x : integer);

begin   {final_status}

  {loop use in output}
  with user do

    begin   {with}

      {writes fuel stop number}
      textcolor (0);
      gotoXY (x,2);
      write ('Stop[',count,']');

      {writes data in chart form}
      frmti (x,5,odo_begin[count],7);
      frmti (x,7,odo_end[count],7);
      wrtr (x,9,fuel_tank[count],7,1);
      wrtr (x,11,miles_tank[count],7,1);
      wrtr (x,13,mpg_tank[count],7,1);
      wrtr (x,15,fuel_trip[count],7,1);
      wrtr (x,17,miles_trip[count],7,1);
      wrtr (x,19,mpg_trip[count],7,1);

    end;   {with}

  {increments the gotoXY position}
  x := x + 9;

end;   {final_status}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Output_Final_Data                                   *
 *  Purpose : To output the final chart                                     *
 *  Method  : Write statements place the output where it belongs            *
 *                                                                          *
 ****************************************************************************}

procedure Output_Final_Data;

var
  move : integer;   {the x value of the gotoXY command}

begin   {output_final_data}

  {sets up the screen}
  clear (1,1,1,1,80,25);

  makewin (0,3,5,2,75,23,true,true,14);

  {sets up the chart}
  wrts (2,5,'Odometer Begin');
  wrts (2,7,'Odometer End');
  wrts (2,9,'Fuel Tank');
  wrts (2,11,'Miles Tank');
  wrts (2,13,'M.P.G. Tank');
  wrts (2,15,'Fuel Trip');
  wrts (2,17,'Miles Trip');
  wrts (2,19,'M.P.G. Trip');

  {calls the procedure final_status to write the data chart}
  move := 17;
  for count := 1 to stops do
    final_status (move);

  {screen header}
  regwin;
  tctb (15,3);
  wrts (34,2,'Final Status Report');

  {hides cursor}
  hidecursor (1,1,1);
  readln;

end;   {output__final_data}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Data (I don't know why I named it that?)            *
 *  Purpose : To organize the main program into a more compact order        *
 *  Method  : Standard loops and simplification are used to organize        *
 *                                                                          *
 ****************************************************************************}

procedure Data;

begin   {data}

  {runs the program until the user selects No or they run it 6 times}
  repeat

    {checks to see if user wants to run it again}
    if cont <> 'n'

      then

        begin  {then}

          clrscr;
          get_data;
          calculate_data;

          {checks the number of stops made}
          if stops < 6 then output_current_data;

        end;  {then}

  until (cont = 'n') or (stops = 6);

end;  {data}

{****************************************************************************
 *                                                                          *
 *  Title   : Procedure Initialize                                          *
 *  Purpose : To initialize a few variables before the program runs         *
 *  Method  : Assign commands are used                                      *
 *                                                                          *
 ****************************************************************************}

procedure Initialize;

begin   {initialize}

  {assigns beginning values to all necessary values}
  count := 1;
  stops := 0;
  counter := 1;
  cont := 'y';
  textcolor (7);
  get_initial_data;

end;   {initialize}

{*****************************************************************************}

begin  {*** Main ***}

  initialize;
  data;
  output_final_data;

end.  {*** Main ***}