PROGRAM WPAINT_2 (painting);  { Wieder Paint version 2.0 }

{*****************************************************************************
 *                                                                           *
 *   Author   : Charles Thomas Wiederhold                                    *
 *   Date     : February 26, 1994                                            *
 *   Title    : WPAINT ver 2.0                                               *
 *   Purpose  : To allow a user to draw and save pictures drawn using the    *
 *              arrow keys and several other options dispayed at the top.    *
 *              These options allow thge user to change colors, print the    *
 *              picture drawn without the excess screen jargon around it,    *
 *              change  the background  color, and exit  the program. The    *
 *              goal was to offer as many options while still keeping the    *
 *              program simple enough for anyone to use.                     *
 *                                                                           *
 *****************************************************************************}

uses
  crt;    { allows the use of screen functions }

const
  center_x = 40;
  center_y = 9;
  name_pos = 37;
  purple   = 5;
  orange   = 6;
  start    = 'S';

type
  block_record = record  { block record }
                  color  : integer;    { element of each text pixel }
                end;  { block record }
  block_array = array [1..76,1..19] of block_record;   { canvas elements }
  paint_file  = file of block_array;

var
  block    : block_array;    { canvas elements }
  painting : paint_file;
  clr,                       { color variable changed through program }
  x,                         { x coordinate on picture }
  y,                         { y coordinate on pricture }
  canvas,                    { color of the canvas/background }
  counter,                   { loop variable }
  count    : integer;        { loop variable }
  name     : string;         { picture name }
  save,
  choice   : char;           { button pressed by user determining actions }
  clear    : boolean;        { determines whether the brush is clear or not }


{*****************************************************************************
 *    Name    : Procedure TCTB                                               *
 *    Purpose : Combines the textcolor/textbackground commands into one      *
 *              command to create the painted effect on the "canvas" since   *
 *              only one color is needed.                                    *
 *****************************************************************************}

Procedure TCTB (tc : integer);

BEGIN  {*** TCTB ***}

  textcolor (tc);
  textbackground (tc);

END;  {*** TCTB ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure TEXT                                               *
 *    Purpose : Combines the textcolor/textbackground commands into one      *
 *              one command with two parameters for the puropose of writing  *
 *              text, but also for multiple colors.                           *
 *****************************************************************************}

Procedure TEXT (tc, tb : integer);

BEGIN  {*** TEXT ***}

  textcolor (tc);
  textbackground (tb);

END;  {*** TEXT ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure WRTAT                                              *
 *    Purpose : Combines the write statement and the gotoXY command into     *
 *              one command so strings can be written at specified locations *
 *              on the screen with one command.                              *
 *****************************************************************************}

Procedure WRTAT (x_axis, y_axis : integer; str : string);

BEGIN  {*** WRTAT ***}

  gotoXY (x_axis,y_axis);
  write (str);

END;  {*** WRTAT ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure CANVASWIN                                          *
 *    Purpose : Ease of use of the window feature for the window needed for  *
 *              the canvas on which the user paints on.                      *
 *****************************************************************************}

Procedure CANVASWIN;

BEGIN  {*** CANVISWIN ***}

  window (3,4,79,23);

END;  {*** CANVASWIN ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure NORMWIN                                            *
 *    Purpose : Ease of use of the window feature for the window needed for  *
 *              the use of the full screen for text, printing, etc.          *
 *****************************************************************************}

Procedure NORMWIN;

BEGIN  {*** NORMWIN ***}

  window (1,1,80,25);

END;  {*** NORMWIN ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure INITIALIZE                                         *
 *    purpose : To initialize the variables at the beginning of the program. *
 *****************************************************************************}

Procedure INITIALIZE;

BEGIN  {*** INITIALIZE ***}

  clear := false;

  for count := 1 to 76 do
    for counter :=  1 to 19 do
      block[count,counter].color := canvas;

  tctb (canvas);

  window (3,4,78,22);

  clrscr;

  canvaswin;

  block[center_x,center_y].color := clr;
  tctb (clr + 24);

  wrtat (center_x,center_y,start);
  x := center_x;
  y := center_y;

  tctb (clr);

END;  {*** INITIALIZE ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure SET_UP                                             *
 *    Purpose : To set up the opening screen and prepare for the user to     *
 *              the interaction with the program.                            *
 *****************************************************************************}

Procedure SET_UP;

var
  number : char;         { the corosponding number written for the 8 colors }
  colors : integer;      { the color written below the numbers in color menu }

BEGIN  {*** SET_UP ***}

  normwin;

  tctb (0);
  clrscr;

  number := '1';
  colors := 0;
  count := 1;

  while count <= 24 do
    begin
      text (15,0);
      wrtat (count + 2,1,number);
      tctb (colors);
      wrtat (count + 2,2,'');
      inc(number);
      inc(colors);
      count := count + 3;
    end;

  text (15,0);
  wrtat (32,1,'<N>  <P>    <M>        <C>           <X>');
  wrtat (32,2,'New  Print  ClearMove  ChangeColors  Exit');

  text (11,0);
  wrtat (2,3,chr(201));
  wrtat (2,23,chr(200));
  wrtat (79,3,chr(187));
  wrtat (79,23,chr(188));

  for count := 3 to 78 do
    begin
      wrtat (count,3,chr(205));
      wrtat (count,23,chr(205));
    end;

  for count := 4 to 22 do
    begin
      wrtat (2,count,chr(186));
      wrtat (79,count,chr(186));
    end;

END;  {*** SET_UP ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure CHANGE_COLORS                                      *
 *    Purpose : On of the options in the menu that allows the user to change *
 *              any of the colors in the painting or the color of the        *
 *              canvas/background.                                           *
 *****************************************************************************}

Procedure CHANGE_COLORS;

var
  new_color_char : char;     { new color entered by user}
  new_color      : integer;  { conversion for the new color }
  old_color_char : char;     { old color entered by user }
  old_color      : integer;  { conversion for the old color }
  what_to_change : char;     { change paint or canvas decided by user }

BEGIN  {*** CHANGE_COLORS ***}

  text (7,0);

  normwin;

  wrtat (2,24,'What would you like to change: <P>aint or <C>anvas');

  repeat
    what_to_change := readkey;
  until (what_to_change = 'p') or (what_to_change = 'c');

  wrtat (2,24,'                                                       ');

  if (what_to_change = 'p')
    then
      begin

        wrtat (2,24,'What color would you like to change: ');

        repeat
          wrtat (39,24,'   ');
          gotoXY (39,24);
          readln (old_color_char);
        until (old_color_char > '0') and (old_color_char < '9');

        old_color := ord(old_color_char) - 49;

        wrtat (2,24,'What color would you like it to be changed to: ');

        repeat
          wrtat (49,24,'   ');
          gotoXY (49,24);
          readln (new_color_char);
        until (new_color_char > '0') and (new_color_char < '9');

        new_color := ord(new_color_char) - 49;

        for count := 1 to 76 do
          for counter := 1 to 19 do
            if block[count,counter].color = old_color
              then block[count,counter].color := new_color;

      end;

  if (what_to_change = 'c')
    then
      begin

        wrtat (2,24,'Select a new color: ');

        repeat
          wrtat (22,24,'   ');
          gotoXY (22,24);
          readln (new_color_char);
        until (new_color_char > '0') and (new_color_char < '9');

        new_color := ord(new_color_char) - 49;

        for count := 1 to 76 do
          for counter := 1 to 19 do
            if block[count,counter].color = canvas
              then block[count,counter].color := new_color;

        canvas := new_color;

      end;

  wrtat (2,24,'                                                           ');
  wrtat (name_pos ,24 ,name);

  canvaswin;

  for count := 1 to 76 do
    for counter :=  1 to 19 do
      begin
        clr := block[count,counter].color;
        tctb (clr);
        wrtat (count,counter,'');
      end;

  tctb (block[x,y].color + 24);
  wrtat (x,y,start);
  gotoXY (x - 2,y);
  clr := block[x,y].color;

  tctb (clr);

END;  {*** CHANGE_COLORS ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure REWRITE                                            *
 *    Purpose : To print the picture without the distarctions of the menu    *
 *              and canvas borders and clear the entire screen in the color  *
 *              of the canvas.                                               *
 *****************************************************************************}

Procedure REWRITEIT;

BEGIN  {*** REWRITE ***}

  block[x,y].color := clr;

  normwin;

  tctb (canvas);
  clrscr;

  text (canvas + 1, canvas);
  wrtat (name_pos,2,name);

  tctb (canvas);

  canvaswin;

  for count := 1 to 76 do
    for counter :=  1 to 19 do
      begin
        clr := block[count,counter].color;
        tctb (clr);
        wrtat (count,counter,'');
      end;

  readln;

  SET_UP;

  canvaswin;

  for count := 1 to 76 do
    for counter :=  1 to 19 do
      begin
        clr := block[count,counter].color;
        tctb (clr);
        wrtat (count,counter,'');
      end;

  normwin;

  text (15,0);
  wrtat (name_pos,24,name);

  canvaswin;

  tctb (block[x,y].color + 24);
  wrtat (x,y,start);
  gotoXY (x - 2,y);
  clr := block[x,y].color;
  tctb (clr);

END;  {*** REWRITE ***}

{/////////////////////////////////////////////////////////////////////////////}
{*****************************************************************************
 *    Name    : Procedure QUESTION                                           *
 *    Purpose : To ask the user what colors they would like the picture to   *
 *              begin with before the drawing starts                         *
 *****************************************************************************}

Procedure QUESTION;

var
  canvas_char,             { canvas color entered by user }
  color_char  : char;      { brush color entered by user }

BEGIN  {*** QUESTION ***}

  text (15,0);
  wrtat (2,24,'Select a color for the canvas: ');

  repeat
    wrtat (33,24,'   ');
    gotoXY (33,24);
    readln (canvas_char);
  until (canvas_char > '0') and (canvas_char < '9');

  canvas := ord(canvas_char) - 49;

  wrtat (2,24,'Select a beginning color for the painting: ');

  repeat
    wrtat (45,24,'   ');
    gotoXY (45,24);
    readln (color_char);
  until (color_char > '0') and (color_char < '9');

  clr := ord(color_char) - 49;

  window (2,24,80,24);

  clrscr;
  wrtat (2,24,'Select a name for your picture: ');
  readln (name);
  clrscr;

  wrtat (name_pos,1,name);

  INITIALIZE;

END;  {*** QUESTION ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure PAINT                                              *
 *    Purpose : Controls most of the program and calls most of the other     *
 *              procedures.  This is where the options are decided, and the  *
 *              different cases of painting modes are operated.              *
 *****************************************************************************}

Procedure PAINT;

var
  free : boolean;   { this is true when the user is moving about the screen }
                    { freely without changing the colors of the blocks as }
                    { they are passed over }

{^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 ^    Name    : Procedure MOVE                                               ^
 ^    Purpose : Controls the movement of the the paint brush, and the        ^
 ^              different directions the user can take.                      ^
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^}

Procedure MOVE;

BEGIN  {*** MOVE ***}

  wrtat (x,y,'');

  case choice of

    'G' : x := 1;

    'q' : begin
            if x <> 1
              then dec(y);
            if y <> 0
              then dec(x);
          end;

    'H' : dec(y);

    'I' : y := 1;

    'a' : begin
            if x <> 1
              then inc(y);
            if y <> 20
              then dec(x);
           end;

    'K' : dec(x);

    'w' : begin
            if x <> 76
              then dec(y);
            if y <> 0
              then inc(x);
          end;

    'M' : inc(x);

    'O' : x := 76;

    's' : begin
            if x <> 76
              then inc(y);
            if y <> 20
              then inc(x);
          end;

    'P' : inc(y);

    'Q' : y := 19;

  end;

  case ord(choice) of

    71, 73, 79, 81 : clear := true;

    else if free = true then clear := true;

  end;

  if y = 0 then inc(y);

  if x = 0 then inc(x);

  if x = 77 then dec(x);

  if y = 20 then dec(y);

  if clear = true
    then clr := block[x,y].color
    else block[x,y].color := clr;

  clear := false;

  tctb (clr + 24);

  case choice of
   'G' : wrtat (x,y,start);
   'q' : wrtat (x,y,chr(27));
   'H' : wrtat (x,y,chr(24));
   'I' : wrtat (x,y,start);
   'a' : wrtat (x,y,chr(27));
   'K' : wrtat (x,y,chr(27));
   'w' : wrtat (x,y,chr(26));
   'M' : wrtat (x,y,chr(26));
   'O' : wrtat (x,y,start);
   's' : wrtat (x,y,chr(26));
   'P' : wrtat (x,y,chr(25));
   'Q' : wrtat (x,y,start);
  end;

  tctb (clr);
  if x = 76
    then gotoXY (x-2,y);

  clear := false;

END;  {*** MOVE ***}

{/////////////////////////////////////////////////////////////////////////////}

{^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 ^    Name    : Procedure FREE_MOVE                                          ^
 ^    Purpose : Allows the user to move about the canvas without changing    ^
 ^              the color of the blocks they pass over.  Good for pictures   ^
 ^              with lots of different colors                                ^
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^}

Procedure FREE_MOVE;

BEGIN  {*** FREE_MOVE ***}

  free := true;

  normwin;

  text (7,0);
  wrtat (2,24,'Press <M> to exit free movement mode.                        ');

  canvaswin;

  gotoXY (x,y);
  tctb (clr);

  repeat
    choice := readkey;
    case choice of
      'a', 'q', 's', 'w',
      'G', 'H', 'I', 'K', 'M', 'O', 'P', 'Q' : MOVE;
    end;
  until (choice = 'm') or (choice = 'x');;

  free := false;
  clear := false;

  normwin;

  text (7,0);
  wrtat (2,24,'                                        ');
  wrtat (name_pos,24,name);

  canvaswin;

  gotoXY (x,y);
  tctb (clr);

END;  {*** FREE_MOVE ***}

{/////////////////////////////////////////////////////////////////////////////}

{^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 ^    Name    : Procedure PAINTING                                           ^
 ^    Purpose : Reads the users choice and then runs the paint module        ^
 ^              and repeats this until the user is ready to exit.            ^
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^}

Procedure PAINTING;

BEGIN  {*** PAINTING ***}

  repeat
    choice := readkey;
    PAINT;
  until (choice = 'x') or (choice = 'p');

  if choice = 'p'
    then
      PAINTING;

END;  {*** PAINTING ***}

{/////////////////////////////////////////////////////////////////////////////}


BEGIN  {*** PAINT ***}

  case choice of
    'a', 'q', 's', 'w',
    'G', 'H', 'I', 'K', 'M', 'O', 'P', 'Q' : MOVE;
    '1' : clr := black;
    '2' : clr := blue;
    '3' : clr := green;
    '4' : clr := cyan;
    '5' : clr := red;
    '6' : clr := purple;
    '7' : clr := orange;
    '8' : clr := white;
    'c' : CHANGE_COLORS;
    'm' : FREE_MOVE;
    'n' : begin
            SET_UP;
            QUESTION;
            PAINTING;
          end;
    'p' : REWRITEIT;
  end;

END;  {*** PAINT ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************
 *    Name    : Procedure PAINTING                                           *
 *    Purpose : Controls all the other modules until the user is ready to    *
 *              exit.  When they select the exit option they are asked if    *
 *              they are sure and responds to their selection.               *
 *****************************************************************************}

Procedure PAINTED;

var
  exit : char;  { exit option decided by user }

BEGIN  {*** PAINTING ***}

  repeat
    choice := readkey;
    PAINT;
  until (choice = 'x') or (choice = 'p');

  if choice = 'p'
    then
      PAINT
    else
      begin

        normwin;

        text (7,0);
        wrtat (36,24,'                  ');
        wrtat (2,24,'Are you sure you want to exit: (Y/N)');

        exit := readkey;

        if exit = 'n'
          then
            begin
              text (15,0);
              wrtat (2,24,'                                      ');
              wrtat (name_pos,24,name);

              canvaswin;

              tctb (clr);

              PAINTED;

            end

          else

            clrscr;
      end;

END;  {*** PAINTING ***}

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************}

BEGIN  {****  MAIN  ****}

  SET_UP;
  QUESTION;
  PAINTED;

END.  {****  MAIN  *****}