program payday;

{***************************************************************************
 *                                                                         *
 *   Author   :  Charlie Wiederhold                                        *
 *   Date     :  December 15, 1994                                         *
 *   Purpose  :  This program calculates and displays the projected pay    *
 *               for an employee.  It reads the name, wage, and hours the  *
 *               hours worked by the employee.  It also has a checking     *
 *               procedure so the user can correct any mistakes in the     *
 *               entering of the data.                                     *
 *   Design   :  The program is composed of the following modules:         *
 *                     Main - main, controlling module                     *
 *                     Open - promts user to enter name and wage           *
 *                     Schedule - calculates the pay                       *
 *                     Report - displays the pay statement                 *
 *                     Check - allows user to correct mistakes in data     *
 *                                                                         *
 ***************************************************************************}

uses
  crt, wieder;

const
  overtrate = 1.5;
  maxh      = 40;
  ficat     = 0.07657;
  fedwt     = 0.06693;

type
  wrkday = array [1..8] of string;
  entry  = array [1..8] of real;

var
  emp      : integer;
  name     : string;
  wage     : real;
  date     : wrkday;
  j        : integer;
  hours    : entry;
  cursor   : integer;
  decision : char;
  fedw     : real;
  fica     : real;
  weekh    : real;
  tothours : real;
  gross    : real;
  overtpay : real;
  overtime : real;
  regpay   : real;
  nett     : real;
  done     : char;
  correct  : integer;
  choice   : char;
  r        : integer;
  c        : integer;

{***************************************************************************}

procedure writeat (x,y : integer; strng : string);

begin

  gotoXY (x,y);
  write (strng);

end;

{***************************************************************************}

procedure linex (a,b,x,y : integer; strng : string);

begin

  for j := a to b do
    writeat ((j + x) - 1,y,strng);

end;

{***************************************************************************}

procedure liney (a,b,x,y : integer; strng : string);

begin

  for j := a to b do
    writeat (x,(j + y) - 1,strng);

end;

{***************************************************************************
 *                                                                         *
 *   Title    :  Opening                                                   *    *
 *   Purpose  :  This procedure provides the opening screen and credits.   *
 *   Method   :  Write and writeln output the data.                        *
 *   Usage    :  Main                                                      *
 *                                                                         *
 ***************************************************************************}

procedure opening;

var
  j : integer;

begin

  textbackground (1);
  clrscr;
  tctb (5,5);
  linex (1,70,5,2,'');
  linex (1,71,5,15,'');
  liney (1,14,5,2,'');
  liney (1,14,74,2,'');
  tctb (0,0);
  linex (1,69,8,16,'');
  liney (1,14,76,3,'');
  tctb (7,1);
  writeln;
  writeln;
  writeln;
  writeln ('    Welcome to       .  Press         to get started!');
  textcolor (2);
  writeat (16,19,'PAYDAY');
  textcolor (14);
  writeat (31,19,'<Enter>');
  tctb (1,15);
  window (7,3,73,14);
  clrscr;
  writeln;
  writeln;
  writeln ('       ܲ              ܲ        ');
  writeln ('                                                     ');
  writeln ('                                                     ');
  writeln ('                            ');
  writeln ('                                                     ');
  writeln ('                                                     ');
  writeln ('                                                     ');
  write   ('                                               ');
  gotoXY (12,3);
  readln;

end;

{***************************************************************************
 *                                                                         *
 *   Title    :  Rep                                                       *
 *   Purpose  :  This procedure asks the user how many times the program   *
 *               needs to be run by the number of employees entered.       *
 *   Method   :  Write and writeln statements promt user to enter data.    *
 *               Readln statements read the entered data.                  *
 *   Usage    :  Main                                                      *
 *                                                                         *
 ***************************************************************************}

procedure rep;

begin  { rep }

  tctb (7,1);
  window (1,1,80,25);
  clrscr;
  write ('Thank you for selecting ...');
  textcolor (2);
  writeln ('  PAYDAY');
  writeln;
  writeln;
  textcolor (15);
  write ('Written by ':47);
  textcolor (14);
  writeln ('Charlie Wiederhold');
  writeln;
  writeln;
  writeln;
  writeln;
  textcolor (7);
  write ('How many employees do you wish to print a report for: ');
  readln (emp);

end;  { rep }

{***************************************************************************
 *                                                                         *
 *   Title    :  Open                                                      *
 *   Purpose  :  This procedure gathers the employee's name and wage.      *
 *   Method   :  Write and writeln statements promt user to enter data.    *
 *               Readln statements read the entered data.                  *
 *   Usage    :  Main                                                      *
 *                                                                         *
 ***************************************************************************}

procedure open;

begin  { open }

  { Gathers the name and wage of employee. }

  window (1,1,80,25);
  textbackground (1);
  clrscr;
  tctb (0,0);
  linex (1,51,17,14,'');
  liney (1,8,66,6,'');
  tctb (0,7);
  window (15,5,65,13);
  clrscr;
  writeln;
  writeln (' Employee ', c);
  for j := 1 to 3 do
    writeln;
  write (' Enter the employee''s name: ');
  readln (name);
  writeln;
  write (' Enter the employee''s wage: ');
  readln (wage);

end;  { open }

{***************************************************************************
 *                                                                         *
 *   Title    :  Schedule                                                  *
 *   Purpose  :  Calculate the data entered by the user to print in        *
 *               the report.                                               *
 *   Method   :  Write and writeln statements promt the user to enter      *
 *               data and readln statements read the data.  Arrays and     *
 *               loops manipulate the data for user-friendliness.  Case    *
 *               statements give the user many options.                    *
 *   Usage    :  Main                                                      *
 *                                                                         *
 ***************************************************************************}

procedure schedule;

begin  { schedule }

  { Calculates the data enter for the wage and hours. }

  date[1] := ' Thursday  ';
  date[2] := ' Friday    ';
  date[3] := ' Saturday  ';
  date[4] := ' Sunday    ';
  date[5] := ' Monday    ';
  date[6] := ' Tuesday   ';
  date[7] := ' Wednesday ';
  date[8] := ' Report    ';
  window (1,1,80,25);
  for j := 1 to 7 do
    hours[j] := 0.0;
  tctb (15,1);
  clrscr;
  cursor := 1;
  writeln ('Weekly schedule : ', name, '.');
  writeln ('Select a date using the up and down arrows and enter the hours',
    ' worked that day.');
  writeln ('Go to Report to print out the pay statement.');
  textcolor (14);
  linex (1,80,0,4,'');
  tctb (0,0);
  window (1,1,80,25);
  linex (1,24,7,16,'');
  liney (1,7,31,10,'');
  linex (1,19,42,19,'');
  liney (1,10,61,10,'');
  tctb (0,7);
  window (5,9,30,15);
  clrscr;
  writeln;
  writeln (' Employee ', c);
  writeln;
  writeln (' Name: ', name);
  writeln (' Wage: ', wage:0:2);
  tctb (0,7);
  window (40,9,60,18);
  cursor := 1;
  repeat
    begin
      clrscr;
      for j := 1 to 8 do
        writeat (2,j + 1,date[j]);
      for j := 1 to 7 do
        begin  { for }
          gotoXY (12,j + 1);
          write (' =', hours[j]:6:2);
        end;  { for }
      selectbar (4,7,2,12,2,9,cursor,chr(16),chr(17));
      repeat
        begin
          tctb (0,7);
          writeat (14,cursor + 1,'       ');
          gotoXY (16,cursor + 1);
          case cursor of
            1..7 : read (hours[cursor]);
          end;  { case }
          if hours[cursor] > 24
            then
              begin
                tctb (4,0);
                writeat (14,cursor + 1,'Invalid');
                delay (1000);
              end;
        end;
      until hours[cursor] <= 24;
      tctb (0,7);
      gotoXY (1,cursor + 1);
    end;  { repeat }
  until cursor = 8;
  tctb (7,1);
  window (1,1,80,25);

end;  { schedule }

{***************************************************************************
 *                                                                         *
 *   Title    :  Report                                                    *
 *   Purpose  :  Display the completed calculations in an easy to read     *
 *               format.                                                   *
 *   Method   :  Write and writeln statements display the output for the   *
 *               user.  Loops and control statements manipulate the data   *
 *               to make it easier to understand.                          *
 *   Usage    :  Main                                                      *
 *                                                                         *
 ***************************************************************************}

procedure report;

begin  { report }

  { Outputs the pay statement using the user's data. }

  clrscr;
  tctb (0,0);
  linex (1,36,5,23,'');
  liney (1,19,39,4,'');
  tctb (0,7);
  window (3,3,38,22);
  clrscr;
  writeln;
  writeln (' Employee: ', name);
  writeln (' Wage    : $', wage:0:2);
  writeln;
  weekh := 0;
  tothours := 0;
  for j := 1 to 7 do
    weekh := weekh + hours[j];
  if weekh > 40
    then
      begin
        overtpay := wage * overtrate;
        overtime := (weekh - maxh) * overtpay;
        regpay := maxh * wage;
        gross := regpay + overtime;
      end
    else
      begin
        overtime := 0;
        gross := weekh * wage;
        regpay := gross;
      end;
  fica := gross * ficat;
  if gross > 50
    then
      fedw := gross * fedwt
    else
      fedw := 0;
  nett := gross - fica - fedw;
  writeln;
  writeln (' Day          | Hours | Pay for day');
  linex (1,36,1,7,'');
  for j := 1 to 7 do
    writeln (date[j], hours[j]:10:2, ' ':5, '$', (wage * hours[j]):7:2);
  linex (1,36,1,15,'');
  writeln;
  writeln (' Total Hours :', ' ':13, weekh:7:2);
  writeln (' Regular Pay :', ' ':12, '$', regpay:7:2);
  writeln (' Overtime    :', ' ':12, '$', overtime:7:2);
  window (1,1,80,25);
  tctb (14,1);
  write ('Is this information correct? (Y/N)');
  tctb (0,0);
  linex (1,31,47,20,'');
  liney (1,9,77,12,'');
  textbackground (7);
  window (44,11,76,19);
  clrscr;
  writeln;
  writeln (' Regular Pay         :  $', regpay:7:2);
  writeln (' Overtime Pay        : +$', overtime:7:2);
  writeln (' Gross Pay           :  $', gross:7:2);
  writeln (' F.I.C.A.            : -$', fica:7:2);
  writeln (' Federal Withholding : -$', fedw:7:2);
  linex (1,33,1,7,'');
  writeln (' Nett Pay            :  $', nett:7:2);
  tctb (7,7);
  writeat (1,1,' ');
  gotoXY (1,1);
  done := upcase(readkey);

end;  { report }

{***************************************************************************
 *                                                                         *
 *   Title    :  Check                                                     *
 *   Purpose  :  Allow the user to change any incorrect data.              *
 *   Method   :  Write and writeln statements display the output for the   *
 *               user.  Loops and control statements manipulate the data   *
 *               to make it easier to understand.  Promts allow the user   *
 *               select different portions of the program the change.      *
 *               Read statements read the new data.                        *
 *   Usage    :  Main                                                      *
 *                                                                         *
 ***************************************************************************}

procedure check;

type
  wrong = array [1..4] of string;

var
  crrct : wrong;
  cursor2 : integer;

begin  { check }

  { Allows the user to correct any mistakes done while entering data }

  crrct[1] := '1: Name  ';
  crrct[2] := '2: Wage  ';
  crrct[3] := '3: Hours ';
  crrct[4] := '4: Done  ';
  window (1,1,80,25);
  cursor := 1;
  if done = 'N'
    then
      begin  { if }
        repeat
          begin  { repeat }
            tctb (14,1);
            clrscr;
            writeln (' Which section is wrong.');
            writeln;
            tctb (7,1);
            for j := 1 to 4 do
              writeln ('  ',crrct[j]);
            tctb (0,0);
            linex (1,31,14,22,'');
            liney (1,11,43,11,'');
            window  (12,10,42,21);
            tctb (0,7);
            clrscr;
            writeln;
            writeln (' Name : ', name);
            writeln (' Wage : $', wage:0:2);
            writeln;
            for j := 1 to 7 do
              writeln (date[j]:15, ' =', hours[j]:7:2);
            window (1,1,80,25);
            selectbar (15,1,2,11,3,6,cursor,chr(16),chr(17));
            tctb (7,1);
            writeat (3,cursor + 2,crrct[cursor]);
            case cursor of
              1 : begin  { case 1 }
                    textcolor (15);
                    writeat (25,5,'Enter correct name: ');
                    readln (name);
                  end;  { case 1 }
              2 : begin  { case 2 }
                    textcolor (15);
                    writeat (25,5,'Enter correct wage: ');
                    readln (wage);
                  end;  { case 2 }
              3 : begin  { case 3 }
                    cursor2 := 1;
                    repeat
                      begin  { repeat }
                        tctb (0,0);
                        linex (1,22,52,14,'');
                        liney (1,12,74,3,'');
                        textbackground (7);
                        window (50,2,73,13);
                        clrscr;
                        writeln;
                        writeln (' Which day is wrong.');
                        writeln;
                        tctb (0,7);
                        date[8] := ' Done  ';
                        for j := 1 to 8 do
                          writeln ('  ', date[j], '= ',
                            hours[j]:6:2);
                        linex (1,13,10,11,' ');
                        selectbar (4,7,3,13,4,11,cursor2,chr(16),chr(17));
                        repeat
                          begin
                            tctb (0,7);
                            writeat (16,cursor2 + 3,'       ');
                            gotoXY (18,cursor2 + 3);
                            case cursor2 of
                              1..7 : read (hours[cursor2]);
                            end;  { case }
                            if hours[cursor2] > 24
                              then
                                begin
                                  tctb (4,0);
                                  writeat (16,cursor2 + 3,'Invalid');
                                  delay (1000);
                                end;
                          end;
                        until hours[cursor2] <= 24;
                      gotoXY (20,cursor2 + 2);
                      end;  { repeat }
                    until cursor2 = 8;
                  tctb (7,1);
                  clrscr;
                  window (1,1,80,25);
                  end;  { repeat }
            end;  { case }
          end;  { repeat }
        until cursor = 4;
      end;  { then }

end;  { check }

{ ************************************************************************ }

begin  {main}

  opening;
  rep;
  c := 1;
  repeat
    begin  { for }
      open;
      schedule;
      report;
      repeat
        begin  { repeat }
          check;
          if cursor = 4
            then
              report;
        end;  { repeat }
      until done = 'Y';
    c := c + 1;
    end;  { for }
  until c = emp + 1;

end.  { main }


