PROGRAM WIEDERGRADE (Class_file, Students_file, Grade_file, Teacher_file);

USES DOS, CRT, UTILITY, FNCTIONS, MENUS, WGHELP, PRINTER;

Const
  leftc          = #75;                 {* left arrow char value       *}
  rightc         = #77;                 {* right arrow char value      *}
  upc            = #72;                 {* up arrow char value         *}
  dnc            = #80;                 {* down arrow char value       *}
  pgupc          = #73;                 {* page up char value          *}
  pgdnc          = #81;                 {* page down char value        *}
  homec          = #71;                 {* home char value             *}
  endc           = #79;                 {* end char value              *}
  enterc         = #13;                 {* enter char value            *}
  escc           = #27;                 {* escape char value           *}
  wincolor       = 7;                   {* window background color     *}
  wintext        = 0;                   {* window text color           *}
  winborder      = 15;                  {* window border color         *}
  winselec       = 9;                   {* selection bar color         *}
  nul            = 0;                   {* used in place of 0          *}
  nograde        = -999;                {* represents no grade         *}
  badgrade       = -666;                {* represents an invalid value *}
  maxdgrades     = 30;                  {* max daily grades            *}
  maxtgrades     = 15;                  {* max test grades             *}
  maxstudents    = 40;                  {* max number of students      *}
  maxgrade       = 100;                 {* max daily grade value       *}
  maxidnum       = 20000;               {* max id number value         *}
  exitchar       = 'X';                 {* exit option character       *}
  fix            = '';       {* window border fix           *}

Type
  grade_range    = nograde..maxgrade;   {* range of grade values       *}
  daily_range    = nul..maxdgrades;     {* range of daily grades       *}
  test_range     = nul..maxtgrades;     {* range of test grades        *}
  class_range    = nul..maxstudents;    {* range of class size         *}
  idnum_range    = nul..maxidnum;       {* range of id number value    *}

  {* arrays of different student storage structures *}
  dgrade_array   = array [1..maxdgrades] of grade_range;
  tgrade_array   = array [1..maxtgrades] of grade_range;
  dgname_array   = array [1..maxdgrades] of string;
  tgname_array   = array [1..maxtgrades] of string;

  {* record of teacher information *}
  teacher_record = record
                     name      : string[22];
                   end;

  {* record of class information *}
  class_record   = record
                     classname : string[11];
                     size      : class_range;
                     name      : array [1..maxstudents] of string;
                     idnum     : array [1..maxstudents] of idnum_range;
                   end;

  {* record of grade information *}
  grade_record   = record
                     dnum      : daily_range;
                     tnum      : test_range;
                     dnames    : dgname_array;
                     tnames    : tgname_array;
                     classave  : real;
                   end;

  {* record of student information *}
  student_record = record
                     dgrades   : dgrade_array;
                     tgrades   : tgrade_array;
                     currentd  : daily_range;
                     currentt  : test_range;
                     dailysum,
                     testsum   : integer;
                     daily,
                     test,
                     total     : real;
                   end;

  {* array of students in class *}
  student_array  = array [1..maxstudents] of student_record;

  {* data files of grade book information *}
  teacher_info   = file of teacher_record;
  classes        = file of class_record;
  grades         = file of grade_record;
  students       = file of student_array;

Var
  {* files *}
  teacher_file : teacher_info;
  class_file   : classes;
  grade_file   : grades;
  student_file : students;

  {* structures *}
  teacher      : teacher_record;
  class        : class_record;
  grade        : grade_record;
  student      : student_array;

  {* keep track of status *}
  exit,
  openwin      : boolean;

  {* used for choices *}
  chrc,
  choice       : char;

  {* used for names and sting to integer conversions *}
  first,
  last,
  intstring    : string;

  {* keep track of status of program and what actions to take *}
  gradepos,
  studentpos,
  oldpos,
  menupos,
  cursor,
  long,
  period,
  curve,
  selec,
  selec2,
  count,
  counter,
  x_pos,
  y_pos,
  seed,
  seed2        : byte;


{/////////////////////////////////////////////////////////////////////////////}

{** Averages daily average **}

Function DAVE (sum, student_num : integer) : real;

var
  num : integer;  {* Helps prevent division by zero errors *}

Begin
  num := nul;
  for counter := 1 to student[student_num].currentd do
    if student[student_num].dgrades[counter] <> nograde
      then inc(num);
  if num = nul
    then DAVE := nul
    else DAVE := sum / num;
End;

{/////////////////////////////////////////////////////////////////////////////}

{** Averages test average **}

Function TAVE (sum, student_num : integer) : real;

var
  num : integer;  {* Helps prevent division by zero erros *}

Begin
  num := nul;
  for counter := 1 to student[student_num].currentt do
    if student[student_num].tgrades[counter] <> nograde
      then inc(num);
  if num = nul
    then TAVE := nul
    else TAVE := sum / num;
End;

{/////////////////////////////////////////////////////////////////////////////}

{** Averages total average **}

Function TOTAVE (student_num : byte; dave, tave : real) : real;

Begin
  if student[student_num].currentd = nul then TOTAVE := tave;
  if student[student_num].currentt = nul then TOTAVE := dave;
  if (student[student_num].currentd <> nul) and
     (student[student_num].currentt <> nul) then TOTAVE := (dave + tave) / 2;
End;

{/////////////////////////////////////////////////////////////////////////////}

{*****************************************************************************}

{** Assigns period data to its data files **}

Procedure ASSIGNFILE (period : byte);

Begin
  case period of
    1 : begin  {* Period 1 *}
          assign (class_file, 'c:\classes\class1.dat');
          assign (grade_file, 'c:\grades\grade1.dat');
          assign (student_file, 'c:\periods\period1.dat');
        end;
    2 : begin  {* Period 2 *}
          assign (class_file, 'c:\classes\class2.dat');
          assign (grade_file, 'c:\grades\grade2.dat');
          assign (student_file, 'c:\periods\period2.dat');
        end;
    3 : begin  {* Period 3 *}
          assign (class_file, 'c:\classes\class3.dat');
          assign (grade_file, 'c:\grades\rade3.dat');
          assign (student_file, 'c:\periods\period3.dat');
        end;
    4 : begin  {* Period 4 *}
          assign (class_file, 'c:\classes\class4.dat');
          assign (grade_file, 'c:\grades\grade4.dat');
          assign (student_file, 'c:\periods\period4.dat');
        end;
    5 : begin  {* Period 5 *}
          assign (class_file, 'c:\classes\class5.dat');
          assign (grade_file, 'c:\grades\grade5.dat');
          assign (student_file, 'c:\periods\period5.dat');
        end;
    6 : begin  {* Period 6 *}
          assign (class_file, 'c:\classes\class6.dat');
          assign (grade_file, 'c:\grades\grade6.dat');
          assign (student_file, 'c:\periods\period6.dat');
        end;
    7 : begin  {* Period 7 *}
          assign (class_file, 'c:\classes\class7.dat');
          assign (grade_file, 'c:\grades\grade7.dat');
          assign (student_file, 'c:\periods\period7.dat');
        end;
  end;
End;

{*****************************************************************************}

{** Saves the period information **}

Procedure SAVEFILE (period : byte);

Begin
  ASSIGNFILE (period);
  rewrite (class_file);
  write (class_file, class);
  close (class_file);
  rewrite (grade_file);
  write (grade_file, grade);
  close (grade_file);
  rewrite (student_file);
  write (student_file, student);
  close (student_file);
End;

{*****************************************************************************}

{** Loads the period information **}

Procedure LOADFILE (period : byte);

Begin
  ASSIGNFILE (period);
  reset (class_file);
  read (class_file, class);
  close (class_file);
  reset (grade_file);
  read (grade_file, grade);
  close (grade_file);
  reset (student_file);
  read (student_file, student);
  close (student_file);
End;

{*****************************************************************************}

{** Averages everyones grades **}

Procedure AVERAGE;

Begin
  for count := 1 to class.size do
    begin
      student[count].dailysum := nul;
      for counter := 1 to grade.dnum do
        if student[count].dgrades[counter] <> nograde
          then student[count].dailysum := student[count].dailysum +
                                            student[count].dgrades[counter];
      student[count].testsum := nul;
      for counter := 1 to grade.tnum do
        if student[count].tgrades[counter] <> nograde
          then student[count].testsum := student[count].testsum +
                                            student[count].tgrades[counter];
      student[count].daily := DAVE(student[count].dailysum, count);
      student[count].test := TAVE(student[count].testsum, count);
      student[count].total := TOTAVE(count,student[count].daily,
                                     student[count].test);
    end;
  grade.classave := nul;
  for count := 1 to class.size do
    grade.classave := grade.classave + student[count].total;
  grade.classave := grade.classave / class.size
End;

{*****************************************************************************}

{** Drops the lowest grade of all students in period **}

Procedure DROPLOWGRADE;

var
  lowestpos,
  lowest : integer;

Begin
  AYSURE (wincolor,wintext,winborder,chrc);
  if (chrc = 'Y') and (class.size > nul)
    then
      begin
        for count := 1 to class.size do
          begin
            lowestpos := nul;
            lowest := 1000;
            for counter := 1 to grade.dnum do
              if (student[count].dgrades[counter] < lowest) and
                 (student[count].dgrades[counter] <> nograde)
                then
                  begin
                    lowest := student[count].dgrades[counter];
                    lowestpos := counter;
                  end;
            student[count].dgrades[lowestpos] := nograde;
          end;
        AVERAGE;
        SAVEFILE (period);
        DONE (wincolor,wintext,winborder);;
      end;
  if (chrc = 'Y') and (class.size = nul)
    then NOSTUDWIN (wincolor,wintext,winborder);
  CLEAR (15,1,4,3,77,24);
End;

{*****************************************************************************}

{** Swaps the information up for a sort **}

Procedure SWAPUP;

var
  temp1 : student_record;  {* temporary student_record *}
  temp2 : string;          {* Temporary name *}
  temp3 : longint;         {* Temporary id number *}

Begin
  temp1 := student[count];
  student[count] := student[count + 1];
  student[count + 1] := temp1;
  temp2 := class.name[count];
  class.name[count] := class.name[count + 1];
  class.name[count + 1] := temp2;
  temp3 := class.idnum[count];
  class.idnum[count] := class.idnum[count + 1];
  class.idnum[count + 1] := temp3;
End;

{*****************************************************************************}

{** Sorts students by last name **}

Procedure SLAST;

Begin
  for count := 1 to class.size do
    class.name[count] := LASTFIRST(class.name[count]);
  for counter := 1 to class.size do
    for count := 1 to (class.size - 1) do
      if class.name[count] > class.name[count + 1] then SWAPUP;
End;

{*****************************************************************************}

{** Gets the student information **}

Procedure GETSTUD;

var
  name : string;

Begin
  CLEAR (15,1,4,3,77,24);
  if (class.size = nul) and (class.classname = '')
    then
      begin
        write ('Enter the name of PERIOD ', period, ' ==> ');
        name := UPSTRING;
        if (name <> 'x') and (name <> 'X')
          then
            begin
              long := length(name);
              if long > 10 then long := 10;
              for count := 1 to long do
                class.classname := class.classname + name[count];
              class.classname := class.classname + ' ';
              REGWIN;
              TCTB (11,1);
              WRTS (14,25,fix);
              TCTB (15,1);
              WRTS (14,25,class.classname);
              CLEAR (15,1,4,3,77,24);
            end;
      end;
  if (name <> 'x') and (name <> 'X')
    then
      begin
        write ('Enter student <', (class.size + 1), '> name ==> ');
        name := UPSTRING;
      end;
  while (name <> exitchar) and (class.size < maxstudents) do
    begin
      inc(class.size);
      class.name[class.size] := name;
      write ('Enter the I.D. number ==> ');
      repeat
        readln (intstring);
        class.idnum[class.size] := STRINT(intstring,'i');
        if class.idnum[class.size] = badgrade
          then
            begin
              TCTB (14,1);
              write ('Invalid! Try again ==> ');
              TCTB (15,1);
            end;
      until class.idnum[class.size] <> badgrade;
      writeln;
      if class.size < maxstudents
        then
          begin
            write ('Enter student <', (class.size + 1), '> name ==> ');
            name := UPSTRING;
          end;
    end;
  clrscr;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Gets the daily grade names **}

Procedure GET_DGRADE_NAMES;

var
  name : string;

Begin
  CLEAR (15,1,4,3,77,24);
  write ('Enter daily grade name <', grade.dnum + 1, '> ==> ');
  name := UPSTRING;
  while (name <> exitchar) and (grade.dnum < maxdgrades) do
    begin
      inc(grade.dnum);
      grade.dnames[grade.dnum] := name;
      writeln;
      if grade.dnum < maxdgrades
        then
          begin
            write ('Enter daily grade name <', (grade.dnum + 1), '> ==> ');
            name := UPSTRING;
          end;
    end;
  clrscr;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Gets the test grades names **}

Procedure GET_TGRADE_NAMES;

var
  name : string;

Begin
  CLEAR (15,1,4,3,77,24);
  write ('Enter test grade name <', (grade.tnum + 1), '> ==> ');
  name := UPSTRING;
  while (name <> exitchar) and (grade.tnum < maxtgrades) do
    begin
      inc(grade.tnum);
      grade.tnames[grade.tnum] := name;
      writeln;
      if grade.tnum < maxtgrades
        then
          begin
            write ('Enter test grade name <', (grade.tnum + 1), '> ==> ');
            name := UPSTRING;
          end;
    end;
  clrscr;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Gets daily grades for the students **}

Procedure ADD_DGRADES;

Begin
  SLAST;
  for count := 1 to class.size do
    class.name[count] := FIRSTLAST(class.name[count]);
  count := 1;
  intstring := '';
  CLEAR (15,1,4,3,77,24);
  repeat
    counter := student[count].currentd + 1;
    clrscr;
    writeln ('Enter daily grades for ', class.name[count]);
    writeln;
    repeat
      if counter = 21
        then
          begin
            clrscr;
            writeln ('Enter daily grades for ', class.name[count]);
            writeln;
          end;
      if counter <= grade.dnum
        then
          begin
            write (grade.dnames[counter], ' ==> ');
            intstring := UPSTRING;
            if intstring <> exitchar
              then
                begin
                  if STRINT(intstring,'g') <> badgrade
                    then
                      begin
                        student[count].dgrades[counter] :=
                          STRINT(intstring,'g');
                        student[count].currentd := counter;
                        inc(counter);
                      end
                    else
                      begin
                        TCTB (14,1);
                        write ('Invalid! Try again: ');
                        TCTB (15,1);
                      end;
                end;
          end;
    until (intstring = exitchar) or (counter > grade.dnum);
    inc(count);
  until (count > class.size) or (intstring = exitchar);
  clrscr;
  if class.size > nul then AVERAGE;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Gets test grades for students **}

Procedure ADD_TGRADES;

Begin
  SLAST;
  for count := 1 to class.size do
    class.name[count] := FIRSTLAST(class.name[count]);
  count := 1;
  intstring := '';
  CLEAR (15,1,4,3,77,24);
  repeat
    counter := student[count].currentt + 1;
    clrscr;
    writeln ('Enter test grades for ', class.name[count]);
    writeln;
    repeat
      if counter = 21
        then
          begin
            clrscr;
            writeln ('Enter test grades for ', class.name[count]);
            writeln;
          end;
      if counter <= grade.tnum
        then
          begin
            write (grade.tnames[counter], ' ==> ');
            intstring := UPSTRING;
            if intstring <> exitchar
              then
                begin
                  if STRINT(intstring,'g') <> badgrade
                    then
                      begin
                        student[count].tgrades[counter] :=
                          STRINT(intstring,'g');
                        student[count].currentt := counter;
                        inc(counter);
                      end
                    else
                      begin
                        TCTB (14,1);
                        write ('Invalid! Try again: ');
                        TCTB (15,1);
                      end;
                end;
          end;
    until (intstring = exitchar) or (counter > grade.tnum);
    inc(count);
  until (count > class.size) or (intstring = exitchar);
  clrscr;
  if class.size > nul then AVERAGE;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Displays the results of a search **}

Procedure SEARCHDP;

Begin
  writeln (' ', TRUNCSTR(LASTFIRST(class.name[count])), class.idnum[count]:9,
           student[count].daily:15:1, student[count].test:13:1,
           student[count].total:12:1);
End;

{*****************************************************************************}

{** Searches for a student by first name **}

Procedure SRCHFIRST;

var
  first  : array [1..maxstudents] of string;
  space  : array [1..maxstudents] of integer;
  selec  : integer;
  target : string;
  found,
  hit    : boolean;

Begin
  for count := 1 to class.size do
    begin
      class.name[count] := FIRSTLAST(class.name[count]);
      space[count] := pos(' ', class.name[count]);
      first[count] := copy(class.name[count], 1, (space[count] - 1));
    end;
  CLEAR (15,1,4,3,77,24);
  write ('Type the first name: ');
  target := UPSTRING;
  writeln;
  writeln;
  clrscr;
  found := false;
  HEADER;
  for count := 1 to class.size do
    begin
      hit := false;
      if first[count] = target then hit := true;
      if hit then found := true;
      if hit then SEARCHDP;
    end;
  if not found
    then
      begin
        TCTB (14,1);
        WRTS (22,6,'Match not found: ? ');
        write (target, ' ?');
      end;
  HIDECURSOR (1,1,1);
  repeat
    chrc := readkey;
  until (chrc = #27);
  clrscr;
End;

{*****************************************************************************}

{** Searches for a student by last name **}

Procedure SRCHLAST;

var
  last   : array [1..maxstudents] of string;
  comma  : array [1..maxstudents] of integer;
  selec  : integer;
  target : string;
  found,
  hit    : boolean;

Begin
  for count := 1 to class.size do
    begin
      class.name[count] := LASTFIRST(class.name[count]);
      comma[count] := pos(',', class.name[count]);
      last[count] := copy(class.name[count],1,(comma[count] - 1));
    end;
  CLEAR (15,1,4,3,77,24);
  write ('Type the last name: ');
  target := UPSTRING;
  writeln;
  writeln;
  found := false;
  clrscr;
  HEADER;
  for count := 1 to class.size do
    begin
      hit := false;
      if last[count] = target then hit := true;
      if hit then found := true;
      if hit then SEARCHDP;
    end;
  if not found
    then
      begin
        TCTB (14,1);
        WRTS (22,6,'Match not found: ? ');
        write (target, ' ?');
      end;
  HIDECURSOR (1,1,1);
  repeat
    chrc := readkey;
  until (chrc = #27);
  clrscr;
End;

{*****************************************************************************}

{** Search for a student by id number **}

Procedure SRCHIDNUM;

var
  found,
  hit    : boolean;
  selec,
  target : integer;

Begin
  for count := 1 to class.size do
    class.name[count] := FIRSTLAST(class.name[count]);
  CLEAR (15,1,4,3,77,24);
  write ('Type the ID number: ');
  repeat
    readln (intstring);
    target := STRINT(intstring,'i');
    if target = badgrade
      then
        begin
          TCTB (14,1);
          write ('Invalid! Try again ==> ');
          TCTB (15,1);
        end;
  until target <> badgrade;
  writeln;
  writeln;
  found := false;
  clrscr;
  HEADER;
  for count := 1 to class.size do
    begin
      hit := false;
      if class.idnum[count] = target then hit := true;
      if hit then found := true;
      if hit then SEARCHDP;
    end;
  if not found
    then
      begin
        TCTB (14,1);
        WRTS (22,6,'Match not found: ? ');
        write (target, ' ?');
      end;
  HIDECURSOR (1,1,1);
  repeat
    chrc := readkey;
  until (chrc = #27);
  clrscr;
End;

{*****************************************************************************}

{** Displays the names for all averages view option **}

Procedure DISPLAY;

Begin
  CLEAR (15,1,4,3,77,24);
  SLAST;
  cursor := 1;
  chrc := #73;
  repeat
    if chrc = #73
      then
        begin
          seed := 1;
          seed2 := 20;
        end;
    if chrc = #81
      then
        begin
          seed := 21;
          seed2 := 40;
        end;
    HEADER;
    if (chrc = #73) or (chrc = #81)
      then
        for count := seed to seed2 do
          begin
            if class.name[count] <> ''
              then
                begin
                  if chrc = #73 then y_pos := count + 2;
                  if chrc = #81 then y_pos := (count - 20) + 2;
                  WRTS (2,y_pos,class.name[count]);
                  FRMTI (25,y_pos,class.idnum[count],9);
                  WRTR (40,y_pos,student[count].daily,9,1);
                  WRTR (53,y_pos,student[count].test,9,1);
                  WRTR (65,y_pos,student[count].total,9,1);
                end;
            if count < seed2 then writeln;
          end;
    TCTB (5,1);
    if chrc = #73 then wrts (1,1,'PgDn');
    if chrc = #81 then wrts (1,1,'PgUp');
    HIDECURSOR (5,1,1);
    repeat
      chrc := readkey;
    until (chrc = #27) or (chrc = #73) or ((chrc = #81) and (class.size > 20));
    TCTB (15,1);
    clrscr;
  until chrc = #27;
End;

{*****************************************************************************}

{** Writes the grades for view all option **}

Procedure VAWRITE (studentpos, gradepos : integer);

Begin
  if studentpos <= maxdgrades
    then
      begin
        if student[studentpos].dgrades[gradepos] <> nograde
          then write (student[studentpos].dgrades[gradepos]:4, ' ')
          else write (' --- ');
      end
    else
      begin
        if student[studentpos].tgrades[gradepos - maxdgrades] <> nograde
          then write (student[studentpos].tgrades[gradepos - maxdgrades]:4,
                      ' ')
          else write (' --- ');
      end;
End;

{*****************************************************************************}

{** Produces the screen for the view all option **}

Procedure VASCREEN;

var
  range : byte;

Begin
  SLAST;
  VABACKGROUND (gradepos,maxdgrades,seed,seed2);
  WRTR (70,1,grade.classave,5,1);
  gotoXY (1,3);
  TCTB (15,1);
  if studentpos <= 20
    then range := 1
    else range := 21;
  for count := range to (range + 19) do
    begin
      if class.name[count] <> ''
        then
          begin
            write (' ', TRUNCSTR(class.name[count]), ' ');
            for counter := seed to seed2 do
              VAWRITE (count,counter);
            if count < (range + 19) then writeln;
          end;
    end;
End;

{*****************************************************************************}

{** Allows the user to edit grades in the view all option **}

Procedure VAENTER;

Begin
  repeat
    window (3,3,78,25);
    TCTB (15,0);
    WRTS (x_pos,y_pos,'     ');
    gotoXY (x_pos,y_pos);
    read (intstring);
    if gradepos <= maxdgrades
      then student[studentpos].dgrades[gradepos] := STRINT(intstring,'g')
      else student[studentpos].tgrades[gradepos- maxdgrades] :=
             STRINT(intstring,'g');
    if (student[studentpos].dgrades[gradepos] = badgrade) or
       (student[studentpos].tgrades[gradepos - maxdgrades] = badgrade)
      then
        begin
          WRTS (x_pos,y_pos,'Error');
          delay(2000);
        end;
    readln;
  until (student[studentpos].dgrades[gradepos] <> badgrade) and
        (student[studentpos].tgrades[gradepos - maxdgrades] <> badgrade);

  AVERAGE;
  SAVEFILE (period);
  VASCREEN;
End;

{*****************************************************************************}

{** View and edit all grades (view all option) **}

Procedure VIEWALL;

var
  oldpos1,
  oldpos2 : integer;

Begin
  oldpos1 := gradepos;
  oldpos2 := studentpos;
  TCTB (15,1);
  gotoXY (x_pos,y_pos);
  VAWRITE (studentpos,gradepos);
  case chrc of
     leftc : if gradepos > 1 then dec(gradepos);
    rightc : if gradepos < 45 then inc(gradepos);
       upc : if studentpos > 1 then dec(studentpos);
       dnc : if studentpos < class.size then inc(studentpos);
     pgupc : studentpos := 1;
     pgdnc : if class.size > 20 then studentpos := 21;
     homec : gradepos := 1;
      endc : gradepos := 31;
    enterc : if gradepos <= maxdgrades
               then
                 begin
                   if grade.dnames[gradepos] <> '' then VAENTER;
                 end
               else
                 begin
                   if grade.tnames[gradepos - maxdgrades] <> '' then VAENTER;
                 end;
  end;
  if gradepos > 45 then gradepos := 45;
  if (chrc = pgupc) or (chrc = pgdnc) or (chrc = homec) or (chrc = endc) or
     ((studentpos = 20) and (oldpos2 = 21)) or
     ((studentpos = 21) and (oldpos2 = 20)) or
     ((gradepos mod 10 = 0) and (oldpos1 mod 10 = 1)) or
     ((gradepos mod 10 = 1) and (oldpos1 mod 10 = nul))
     then VASCREEN;
  WRTS (26,1,'                       ');
  TCTB (14,1);
  if gradepos <= maxdgrades
    then WRTS (26,1,grade.dnames[gradepos])
    else WRTS (26,1,grade.tnames[gradepos - maxdgrades]);
  if (grade.dnames[gradepos] = '') or
     (grade.tnames[gradepos - maxdgrades] = '')
    then WRTS (26,1,'No grade');
  if class.name[studentpos] <> ''
    then WRTR (13,2,student[studentpos].total,5,1);
  TCTB (15,1);
  case gradepos mod 10 of
    1 : x_pos := 26;
    2 : x_pos := 31;
    3 : x_pos := 36;
    4 : x_pos := 41;
    5 : x_pos := 46;
    6 : x_pos := 51;
    7 : x_pos := 56;
    8 : x_pos := 61;
    9 : x_pos := 66;
    0 : x_pos := 71;
  end;
  if studentpos <= 20
    then y_pos := (studentpos + 2)
    else y_pos := (studentpos - 20 + 2);
  gotoXY (x_pos,y_pos);
  TCTB (15,0);
  VAWRITE (studentpos,gradepos);
  HIDECURSOR (1,1,1);
  TCTB (15,1);
  repeat
    chrc := readkey;
  until (chrc = leftc) or (chrc = rightc) or (chrc = upc) or (chrc = dnc) or
        (chrc = pgupc) or (chrc = pgdnc) or (chrc = homec) or (chrc = endc) or
        (chrc = enterc) or (chrc = escc);
  if chrc <> escc
    then VIEWALL
    else CLEAR (15,1,3,3,78,24);
End;

{*****************************************************************************}

{** Displays all the names in the class on one screen **}

Procedure NAMEDISPLAY;

Begin
  repeat
    clrscr;
    writeln (' 0: EXIT');
    if class.size <= 20
      then
        begin
          seed := class.size;
          for count := 1 to class.size do
            begin
              write (count:2, ': ', TRUNCSTR(class.name[count]));
              if cursor = 2 then write (class.idnum[count]:7);
              writeln;
            end;
          end
        else
          begin
            if class.size mod 2 = nul
            then seed := (class.size div 2)
            else seed := (class.size div 2) + 1;
            for count := 1 to seed do
              begin
                write (count:2, ': ', TRUNCSTR(class.name[count]));
                if cursor = 2 then write (class.idnum[count]:7);
                if (count * 2) <= class.size
                  then
                    begin
                      write ((count + seed):7, ': ',
                              TRUNCSTR(class.name[count + seed]));
                      if cursor = 2 then write (class.idnum[count + seed]:7);
                    end;
                writeln;
              end;
          end;
    TCTB (14,1);
    write ('Which student? (number) ==> ');
    TCTB (15,1);
    readln (intstring);
    selec := STRINT (intstring,'g');
    if (selec = badgrade) or (selec < nul) or (selec > class.size)
      then
        begin
          TCTB (4,1);
          WRTS (29,(seed + 2),'Invalid Entry!');
          delay (1000);
          TCTB (15,1);
        end;
  until (selec >= nul) and (selec <= class.size);
End;

{*****************************************************************************}

{** Displays all the daily grade names on one screen **}

Procedure DGRADESDISPLAY;

Begin
  repeat
    clrscr;
    if cursor = 3
      then
        begin
          writeln ('Student: ', class.name[selec], '  (', class.idnum[selec],
                   ')');
          writeln;
        end;
    writeln (' 0: EXIT');
    if grade.dnum <= 15
      then
        begin
          seed := grade.dnum;
          for count := 1 to grade.dnum do
            begin
              write (count:2, ': ', TRUNCSTR(grade.dnames[count]));
              if cursor = 3 then write (': ', student[selec].dgrades[count]:3);
              writeln;
            end;
          end
        else
          begin
            if grade.dnum mod 2 = nul
            then seed := (grade.dnum div 2)
            else seed := (grade.dnum div 2) + 1;
            for count := 1 to seed do
              begin
                write (count:2, ': ', TRUNCSTR(grade.dnames[count]));
                if cursor = 3 then write (': ',
                                          student[selec].dgrades[count]:3);
                if (count * 2) <= grade.dnum
                  then
                    begin
                      write ((count + seed):7, ': ',
                             TRUNCSTR(grade.dnames[count + seed]));
                      if cursor = 3
                        then write (': ', student[selec].dgrades[count]:3);
                    end;
                writeln;
              end;
          end;
    writeln;
    TCTB (14,1);
    write ('Which grade? (number) ==> ');
    TCTB (15,1);
    readln (intstring);
    selec2 := STRINT(intstring,'g');
    if (selec2 = badgrade) or (selec2 < nul) or (selec2 > grade.dnum)
      then
        begin
          TCTB (4,1);
          if (cursor = 7) or (cursor = 5)
            then WRTS (27,(seed + 3),'Invalid Entry!')
            else WRTS (27,(seed + 5),'Invalid Entry!');
          delay (1000);
          TCTB (15,1);
        end;
  until (selec2 >= nul) and (selec2 <= grade.dnum);
End;

{*****************************************************************************}

{** Displays all the test grades **}

Procedure TGRADESDISPLAY;

var
  seed : integer;

Begin
  seed := 3;
  repeat
    clrscr;
    if cursor = 4
      then
        begin
          seed := seed + 2;
          writeln ('Student: ', class.name[selec], '  (', class.idnum[selec],
                   ')');
          writeln;
        end;
    writeln (' 0: EXIT');
    for count := 1 to grade.tnum do
      begin
        write (count:2, ': ', TRUNCSTR(grade.tnames[count]));
        if cursor = 4  then write (': ', student[selec].tgrades[count]:3);
        writeln;
      end;
    writeln;
    TCTB (14,1);
    write ('Which grade? (number) ==> ');
    TCTB (15,1);
    readln (intstring);
    selec2 := STRINT(intstring,'g');
    if (selec2 = badgrade) or (selec2 < nul) or (selec2 > grade.tnum)
      then
        begin
          TCTB (4,1);
          WRTS (27,(grade.tnum + seed),'Invalid Entry!');
          delay (1000);
          TCTB (15,1);
        end;
  until (selec2 >= nul) and (selec2 <= grade.tnum);
End;

{*****************************************************************************}

{** Allows user to edit student names **}

Procedure ENAME;

var
  exit  : boolean;

Begin
  SLAST;
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    NAMEDISPLAY;
    if selec = nul
      then exit := true
      else
        begin
          class.name[selec] := FIRSTLAST(class.name[selec]);
          clrscr;
          write ('Enter new name for ', class.name[selec], ' ==> ');
          class.name[selec] := UPSTRING;
          class.name[selec] := LASTFIRST(class.name[selec]);
        end;
    clrscr;
  until exit;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to edit student id numbers **}

Procedure EIDNUM;

var
  exit  : boolean;

Begin
  SLAST;
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    NAMEDISPLAY;
    if selec = nul
      then exit := true
      else
        begin
          clrscr;
          class.name[selec] := FIRSTLAST(class.name[selec]);
          write ('Enter new ID number for ', class.name[selec], ' (',
                  class.idnum[selec], ') ==> ');
          repeat
            readln (intstring);
            class.idnum[selec] := STRINT (intstring,'i');
            if class.idnum[selec] = badgrade
              then
                begin
                  TCTB (14,1);
                  write ('Invalid! Try again ==> ');
                  TCTB (15,1);
                end;
          until class.idnum[selec] <> badgrade;
          class.name[selec] := LASTFIRST(class.name[selec]);
        end;
    clrscr;
  until exit;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to edit students daily grades **}

Procedure EDAILY;

var
  exit   : boolean;

Begin
  SLAST;
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    NAMEDISPLAY;
    if selec = nul
      then exit := true
      else
        begin
          clrscr;
          class.name[selec] := FIRSTLAST(class.name[selec]);
          writeln ('Student: ', class.name[selec]);
          writeln;
          DGRADESDISPLAY;
          if selec2 <> nul
            then
              begin
                clrscr;
                write ('Change ', grade.dnames[selec2], ' (',
                       student[selec].dgrades[selec2], ') to ==> ');
                repeat
                  readln (intstring);
                  student[selec].dgrades[selec2] := STRINT (intstring,'g');
                  if student[selec].dgrades[selec2] = badgrade
                    then
                      begin
                        TCTB (14,1);
                        write ('Invalid! Try again ==> ');
                        TCTB (15,1);
                      end;
                until student[selec].dgrades[selec2] <> badgrade;
              end;
          class.name[selec] := LASTFIRST(class.name[selec]);
        end;
    clrscr;
  until exit;
  AVERAGE;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to edit students test grades **}

Procedure ETEST;

var
  exit   : boolean;

Begin
  SLAST;
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    NAMEDISPLAY;
    if selec = nul
      then exit := true
      else
        begin
          clrscr;
          class.name[selec] := FIRSTLAST(class.name[selec]);
          writeln ('Student: ', class.name[selec]);
          writeln;
          TGRADESDISPLAY;
          if selec2 <> nul
            then
              begin
                clrscr;
                write ('Change ', grade.tnames[selec2], ' (',
                       student[selec].tgrades[selec2], ') to ==> ');
                repeat
                  readln (intstring);
                  student[selec].tgrades[selec2] := STRINT (intstring,'g');
                  if student[selec].tgrades[selec2] = badgrade
                    then
                      begin
                        TCTB (14,1);
                        write ('Invalid! Try again ==> ');
                        TCTB (15,1);
                      end;
                until student[selec].tgrades[selec2] <> badgrade;
              end;
          class.name[selec] := LASTFIRST(class.name[selec]);
        end;
    clrscr;
  until exit;
  AVERAGE;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to add a curve to a daily grade **}

Procedure DCURVE;

var
  exit  : boolean;

Begin
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    DGRADESDISPLAY;
    if selec2 = nul
      then exit := true
      else
        begin
          clrscr;
          write ('How much would you like to curve ',
                  grade.dnames[selec2], ' ==> ');
          repeat
            readln (intstring);
            curve := STRINT (intstring,'g');
            if curve = badgrade
              then
                begin
                  TCTB (14,1);
                  write ('Invalid! Try again ==> ');
                  TCTB (15,1);
                end;
          until curve <> badgrade;
          for count := 1 to class.size do
            if student[count].dgrades[selec2] <> nograde
              then student[count].dgrades[selec2] :=
                   student[count].dgrades[selec2] + curve;
        end;
    clrscr;
  until exit;
  AVERAGE;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to add a curve to a test grade **}

Procedure TCURVE;

var
  exit  : boolean;

Begin
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    TGRADESDISPLAY;
    if selec2 = nul
      then exit := true
      else
        begin
          clrscr;
          write ('How much would you like to curve ',
                  grade.tnames[selec2], ' ==> ');
          repeat
            readln (intstring);
            curve := STRINT (intstring,'g');
            if curve = badgrade
              then
                begin
                  TCTB (14,1);
                  write ('Invalid! Try again ==> ');
                  TCTB (15,1);
                end;
          until curve <> badgrade;
          for count := 1 to class.size do
            if student[count].tgrades[selec2] <> nograde
              then student[count].tgrades[selec2] :=
                   student[count].tgrades[selec2] + curve;
        end;
    clrscr;
  until exit;
  AVERAGE;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to edit a daily grade name **}

Procedure EDAILYNAME;

var
  exit  : boolean;

Begin
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    DGRADESDISPLAY;
    if selec2 = nul
      then exit := true
      else
        begin
          clrscr;
          write ('Change ', grade.dnames[selec2], ' to: ');
          grade.dnames[selec2] := UPSTRING;
        end;
    clrscr;
  until exit;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Allows user to edit a test grade name **}

Procedure ETESTNAME;

var
  exit  : boolean;

Begin
  CLEAR (15,1,4,3,77,24);
  repeat
    exit := false;
    TGRADESDISPLAY;
    if selec2 = nul
      then exit := true
      else
        begin
          clrscr;
          write ('Change ', grade.tnames[selec2], ' to: ');
          grade.tnames[selec2] := UPSTRING;
        end;
    clrscr;
  until exit;
  SAVEFILE (period);
End;

{*****************************************************************************}

{** Produces the period menu option **}

Procedure PERIODWIN;

Begin
  PERIODMENU (wincolor,wintext,winborder);
  cursor := period;
  repeat
    SELECTBAR (winselec,wincolor,8,7,cursor,choice);
  until (choice = #13) or (choice = #27);
  if choice = #13 then period := cursor;
End;

{*****************************************************************************}

{** Allows user to edit the name of the class **}

Procedure ECLASSNAME;

var
  name : string;

Begin
  CLEAR (15,1,4,3,77,24);
  write ('Enter the new name of PERIOD ', period, ' ', class.classname,
         ' ==> ');
  name := UPSTRING;
  if (name <> 'x') and (name <> 'X')
    then
      begin
        class.classname := '';
        long := length(name);
        if long > 10 then long := 10;
        for count := 1 to long do
          class.classname := class.classname + name[count];
        class.classname := class.classname + ' ';
        clrscr;
        SAVEFILE (period);
        REGWIN;
        TCTB (11,1);
        WRTS (14,25,fix);
        TCTB (15,1);
        WRTS (14,25,class.classname);
      end;
  CLEAR (15,1,4,3,77,24);
  cursor := 10;
End;

{*****************************************************************************}

{** User selects a period to load **}

Procedure LOAD;

Begin
  PERIODWIN;
  if choice = #13
    then
      begin
        LOADFILE (period);
        REGWIN;
        TCTB (11,1);
        WRTS (4,25,' PERIOD   ');
        WRTI (12,25,period);
        WRTS (14,25,fix);
        TCTB (15,1);
        if class.classname <> '' then WRTS (14,25,class.classname);
        DONE (wincolor,wintext,winborder);;
      end;
  cursor := 1;
  CLEAR (15,1,4,3,77,24);
End;

{*****************************************************************************}

{** Prints out the class report **}

Procedure PRINTCLASSREPORT;

Begin
  write (lst, '');
  long := length(class.classname) + 2;
  for counter := 1 to long do write (lst, '');
  writeln (lst, '');
  writeln (lst, ' Class: ', class.classname, ' ');
  write (lst, '');
  for counter := 1 to long do write (lst, '');
  writeln (lst, '');
  writeln (lst);
  writeln (lst, '   ',
                '͸');
  writeln (lst, '             Name               I.D.    DailyAvg ',
                '  TestAvg   Average ');
  writeln (lst, '   ',
                '͵');
  for count := 1 to class.size do
    begin
      writeln (lst, '    ', TRUNCSTR(class.name[count]),
                    '  ', class.idnum[count]:7,
                    '    ', student[count].daily:7:1,
                    '   ', student[count].test:7:1,
                    '   ', student[count].total:7:1, '  ');
    end;
  writeln (lst, '   ',
                ';');
End;

{*****************************************************************************}

{** Prints out the individual reports **}

Procedure PRINTINDIREPORT;

var
  counted : byte;

Begin
  class.name[count] := FIRSTLAST(class.name[count]);
  CLEAR (15,1,4,3,77,24);
  writeln (lst, 'Name: ', FIRSTLAST(class.name[count]),
                'ID Number: ':20, class.idnum[count]);
  writeln (lst);
  write (lst, '');
  long := length(class.classname) + 2;
  for counter := 1 to long do write (lst, '');
  writeln (lst, '');
  writeln (lst, ' Class: ', class.classname, ' ');
  write (lst, '');
  for counter := 1 to long do write (lst, '');
  writeln (lst, '');
  writeln (lst);
  writeln (lst, 'Daily Grades');
  writeln (lst, '');
  for counter := 1 to grade.dnum do
    begin
      write (lst, grade.dnames[counter]);
      long := length(grade.dnames[counter]);
      for counted := long to 20 do write (lst, '.');
      if student[count].dgrades[counter] <> nograde
        then writeln (lst, ' ', student[count].dgrades[counter]:3)
        else writeln (lst, ' ---');
    end;
  writeln (lst);
  writeln (lst, 'Daily Average: ', student[count].daily:5:1);
  writeln (lst);
  writeln (lst);
  writeln (lst, 'Test Grades');
  writeln (lst, '');
  for counter := 1 to grade.tnum do
    begin
      write (lst, grade.tnames[counter]);
      long := length(grade.tnames[counter]);
      for counted := long to 20 do write (lst, '.');
      if student[count].tgrades[counter] <> nograde
        then writeln (lst, ' ', student[count].tgrades[counter]:3)
        else writeln (lst, ' ---');
    end;
  writeln (lst);
  writeln (lst, 'Test Average: ', student[count].test:5:1);
  writeln (lst);
  writeln (lst);
  writeln (lst, 'Total Average: ', student[count].total:5:1);
  for counter := (grade.dnum + grade.tnum + 19)to 59 do
    writeln (lst);
End;

{*****************************************************************************}

{** Runs the printing of the class report **}

Procedure CLASSREPORT;

Begin
  AYSURE (wincolor,wintext,winborder,chrc);
  if chrc = 'Y'
    then
      begin
        SLAST;
        PRINTCLASSREPORT;
        DONE (wincolor,wintext,winborder);
      end;
End;

{*****************************************************************************}

{** Runs the printing of the individual reports **}

Procedure INDIREPORT;

var
  oldcursor : byte;

Begin
  if cursor = 2
    then
      begin
        CLEAR (15,1,4,3,77,24);
        NAMEDISPLAY;
      end;
  if selec <> nul then AYSURE (wincolor,wintext,winborder,chrc);
  if chrc = 'Y'
    then
      begin
        if cursor = 2
          then seed := selec
          else seed := 1;
        if cursor = 2
          then seed2 := selec
          else seed2 := class.size;
        if selec <> nul
          then
            begin
              for count := seed to seed2 do PRINTINDIREPORT;
              DONE (wincolor,wintext,winborder);
            end;
      end;
End;

{*****************************************************************************}

{** Runs the over all printing option **}

Procedure PRINT;

Begin
  cursor := 1;
  SLAST;
  PRINTMENU (wincolor,wintext,winborder);
  repeat
    SELECTBAR (winselec,wincolor,11,3,cursor,choice);
  until (choice = #13) or (choice = #27);
  if choice = #13
    then
      case cursor of
          1 : CLASSREPORT;
        2,3 : INDIREPORT;
      end;
  CLEAR (15,1,4,3,77,24);
  cursor := 2;
End;

{*****************************************************************************}

{** Clears all the grades in the gradebook **}

Procedure CLEARGRADES;

Begin
  for count := 1 to maxstudents do
    begin
      student[count].currentd := nul;
      student[count].currentt := nul;
      student[count].daily := nul;
      student[count].test := nul;
      student[count].total := nul;
      student[count].dailysum := nul;
      student[count].testsum := nul;
      for counter := 1 to maxdgrades do student[count].dgrades[counter] := nul;
      for counter := 1 to maxtgrades do student[count].tgrades[counter] := nul;
    end;
  grade.dnum := nul;
  grade.tnum := nul;
  for count := 1 to maxdgrades do grade.dnames[count] := '';
  for count := 1 to maxtgrades do grade.tnames[count] := '';
End;

{*****************************************************************************}

{** Clears all the students in the gradebook **}

Procedure CLEARCLASS;

Begin
  class.size := nul;
  class.classname := '';
  for count := 1 to maxstudents do
    begin
      class.name[count] := '';
      class.idnum[count] := nul;
    end;
End;

{*****************************************************************************}

{** Erases the grades of all the students in the gradebook **}

Procedure NEWSIXWEEKS;

Begin
  AYSURE (wincolor,wintext,winborder,chrc);
  if chrc = 'Y'
    then
      begin
        CLEARGRADES;
        for period := 1 to 7 do SAVEFILE (period);
        DONE (wincolor,wintext,winborder);;
      end;
End;

{*****************************************************************************}

{** Erases the grades and students in the gradebook **}

Procedure NEWYEAR;

Begin
  AYSURE (wincolor,wintext,winborder,chrc);
  if chrc = 'Y'
    then
      begin
        CLEARGRADES;
        CLEARCLASS;
        for period := 1 to 7 do SAVEFILE (period);
        REGWIN;
        TCTB (11,1);
        WRTS (14,25,fix);
        DONE (wincolor,wintext,winborder);;
      end;
End;

{*****************************************************************************}

{** Runs the new option **}

Procedure NEW;

Begin
  NEWMENU (wincolor,wintext,winborder);
  cursor := 1;
  repeat
    SELECTBAR (winselec,wincolor,11,2,cursor,choice);
  until (choice = #13) or (choice = #27);
  if choice = #13 then
     case cursor of
       1 : NEWSIXWEEKS;
       2 : NEWYEAR;
     end;
  CLEAR (15,1,4,3,77,24);
  period := 1;
  cursor := 3;
End;

{*****************************************************************************}

{** Controls the direction of the program after a selection is made **}

Procedure ENTER;

Begin
  case menupos of
    1 : case cursor of
          1 : LOAD;
          2 : if class.size > nul
                then PRINT
                else NOSTUDWIN (wincolor,wintext,winborder);
          3 : NEW;
        end;
    2 : case cursor of
          1 : if class.size < maxstudents then GETSTUD;
          2 : if grade.dnum < maxdgrades then GET_DGRADE_NAMES;
          3 : if grade.tnum < maxtgrades then GET_TGRADE_NAMES;
          4 : if grade.dnum > nul
                then ADD_DGRADES
                else NOGRADEWIN (wincolor,wintext,winborder);
          5 : if grade.tnum > nul
                then ADD_TGRADES
                else NOGRADEWIN (wincolor,wintext,winborder);
        end;
    3 : case cursor of
          1 : if class.size > nul
                then SRCHFIRST
                else NOSTUDWIN (wincolor,wintext,winborder);
          2 : if class.size > nul
                then SRCHLAST
                else NOSTUDWIN (wincolor,wintext,winborder);
          3 : if class.size > nul
                then SRCHIDNUM
                else NOSTUDWIN (wincolor,wintext,winborder);
        end;
    5 : case cursor of
           1 : if class.size > nul
                 then ENAME
                 else NOSTUDWIN (wincolor,wintext,winborder);
           2 : if class.size > nul
                 then EIDNUM
                 else NOSTUDWIN (wincolor,wintext,winborder);
           3 : if class.size > nul
                 then EDAILY
                 else NOSTUDWIN (wincolor,wintext,winborder);
           4 : if class.size > nul
                 then ETEST
                 else NOSTUDWIN (wincolor,wintext,winborder);
           5 : if class.size > nul
                 then EDAILYNAME
                 else NOSTUDWIN (wincolor,wintext,winborder);
           6 : if class.size > nul
                 then ETESTNAME
                 else NOSTUDWIN (wincolor,wintext,winborder);
           7 : if class.size > nul
                 then DCURVE
                 else NOSTUDWIN (wincolor,wintext,winborder);
           8 : if class.size > nul
                 then TCURVE
                 else NOSTUDWIN (wincolor,wintext,winborder);
           9 : if class.size > nul
                 then DROPLOWGRADE
                 else NOSTUDWIN (wincolor,wintext,winborder);
          10 : ECLASSNAME;
        end;
    6 : case cursor of
           1 : HELP (wincolor,wintext,winborder);
           2 : ABOUT (wincolor,wintext,winborder,menupos);
        end;
  end;
End;

{*****************************************************************************}

{** The file option **}

Procedure ALTF;

Begin
  FILEMENU (wincolor,wintext,winborder);
  menupos := 1;
  openwin := true;
  SELECTBAR (winselec,wincolor,5,4,cursor,choice);
  if choice = #13
    then
      begin
        if cursor <> 4
          then
            begin
              ENTER;
              ALTF;
            end
          else exit := true;
      end;
End;

{*****************************************************************************}

{** The add option **}

Procedure ALTA;

Begin
  ADDMENU (wincolor,wintext,winborder);
  menupos := 2;
  openwin := true;
  SELECTBAR (winselec,wincolor,11,5,cursor,choice);
  if choice = #13
    then
      begin
        ENTER;
        ALTA;
      end;
End;

{*****************************************************************************}

{** The search option **}

Procedure ALTS;

Begin
  SEARCHMENU (wincolor,wintext,winborder);
  menupos := 3;
  openwin := true;
  SELECTBAR (winselec,wincolor,9,3,cursor,choice);
  if choice = #13
    then
      begin
        ENTER;
        ALTS;
      end;
End;

{*****************************************************************************}

{** The view option **}

Procedure ALTV;

Begin
  VIEWMENU (wincolor,wintext,winborder);
  menupos := 4;
  openwin := true;
  cursor := oldpos;
  SELECTBAR (winselec,wincolor,11,2,cursor,choice);
  oldpos := cursor;
  if (choice = #13) and (cursor = 1) and (class.size > nul)
    then
      begin
        DISPLAY;
        ALTV;
      end;
  if (choice = #13) and (cursor = 2) and (class.size > nul)
    then
      begin
        if class.size > nul then AVERAGE;
        SLAST;
        for count := 1 to class.size do
          class.name[count] := FIRSTLAST(class.name[count]);
        x_pos := 26;
        y_pos := 3;
        studentpos := 1;
        gradepos := 1;
        VASCREEN;
        if class.size > nul
          then
            begin
              VIEWALL;
              ALTV;
            end
          else
            begin
              WRTS (25,10,'No students, Press <ESC>.');
              HIDECURSOR (1,1,1);
              repeat
                chrc := readkey;
              until chrc = escc;
              ALTV;
            end;
      end;
  if (choice = #13) and (class.size = nul)
    then
      begin
        NOSTUDWIN (wincolor,wintext,winborder);
        ALTV;
      end;
  oldpos := 1;
End;

{*****************************************************************************}

{** The edit option **}

Procedure ALTE;

Begin
  EDITMENU (wincolor,wintext,winborder);
  menupos := 5;
  openwin := true;
  SELECTBAR (winselec,wincolor,11,10,cursor,choice);
  if (choice = #13)
    then
      begin
        ENTER;
        ALTE;
      end;
End;

{*****************************************************************************}

{** The help option **}

Procedure ALTH;

Begin
  HELPMENU (wincolor,wintext,winborder);
  menupos := 6;
  openwin := true;
  SELECTBAR (winselec,wincolor,5,2,cursor,choice);
  if (choice = #13)
    then
      begin
        ENTER;
        ALTH;
      end;
End;

{*****************************************************************************}

{** Moves to the left in the menu system **}

Procedure LEFT;

Begin
  if openwin = true then
    begin
      dec(menupos);
      if menupos = nul then menupos := 6;
      case menupos of
        1 : ALTF;
        2 : ALTA;
        3 : ALTS;
        4 : ALTV;
        5 : ALTE;
        6 : ALTH;
      end;
    end;
End;

{*****************************************************************************}

{** Moves to the right in the menu system **}

Procedure RIGHT;

Begin
  if openwin = true then
    begin
      inc(menupos);
      if menupos = 7 then menupos := 1;
      case menupos of
        1 : ALTF;
        2 : ALTA;
        3 : ALTS;
        4 : ALTV;
        5 : ALTE;
        6 : ALTH;
      end;
    end;
End;

{*****************************************************************************}

{** Creates the background for the entire program and kicks everything off **}

Procedure RUNWIN;

Begin
  BACKGROUND;
  assign (teacher_file, 'c:teacher.dat');
  reset (teacher_file);
  read (teacher_file, teacher);
  close (teacher_file);
  tctb (15,1);
  wrts (13,2,teacher.name);
  wrts (14,25,class.classname);
  window (3,4,77,24);
End;

{*****************************************************************************}

{** Initializes the beggining values **}

Procedure INITIALIZE;

Begin
  period := 1;
  cursor := 1;
  menupos := nul;
  choice := #255;
  openwin := false;
  exit := false;
  oldpos := 1;
  LOADFILE (period);
  RUNWIN;
  ABOUT (wincolor,wintext,winborder,menupos);
End;

{*****************************************************************************}

{** Runs the entire program. Everything returns to here and awaits the next **}
{** command from the user. It can best be visualized as the wise man who is **}
{** unable to actually do anything, but can tell everyone else what to do!! **}

Procedure WIEDERGRADES;

Begin
  repeat
    cursor := 1;
    CLEAR (15,1,4,3,77,24);
    if openwin = false
      then
        begin
          HIDECURSOR (1,1,1);
          choice := readkey;
        end;
    case choice of
      #27 : if (openwin = true)
              then
                begin
                  HEADING;
                  CLEAR (15,1,4,3,77,24);
                  openwin := false;
                end;
      #75 : LEFT;
      #77 : RIGHT;
      #33,
      #59 : ALTF;
      #30,
      #60 : ALTA;
      #31,
      #61 : ALTS;
      #47,
      #62 : ALTV;
      #18,
      #63 : ALTE;
      #35,
      #64 : ALTH;
      #45 : exit := true;
    end;
    if exit = true
      then
        begin
          HEADING;
          EXITCHECK (wincolor,wintext,winborder,chrc);
          if chrc = 'N'
            then
              begin
                exit := false;
                openwin := false;
                HEADING;
              end;
        end;
  until exit;
  CLEAR (7,0,1,1,80,25);
End;

{*****************************************************************************}

BEGIN

checkbreak := false;
{** This is where the magic begins... **}

INITIALIZE;
WIEDERGRADES;

{** ... and ends. **}

END.