{-----------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} UNIT UTILITY; {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{-----------------------------------------------------------------------------}



{-----------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} INTERFACE {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{-----------------------------------------------------------------------------}



USES CRT;

  Procedure TCTB       (tc, tb : byte);
  Procedure NOTE       (tone, time : integer);
  Procedure WRTC       (x, y : byte; chr : char);
  Procedure WRTS       (x, y : byte; str : string);
  Procedure WRTI       (x, y : byte; int : integer);
  Procedure WRTR       (x, y : byte; reel : real; field, decimal : byte);
  Procedure FRMTI      (x, y : byte; int : integer; field : byte);
  Procedure REGWIN;
  Procedure CLEAR      (text, backcolor, ulx, uly, lrx, lry : byte);
  Procedure MAKEWIN    (shadow, border : boolean;
                        bordercolor, borderstyle,
                        text, background, ulx, uly, width, height : byte);
  Procedure HIDECURSOR (x, y, color : byte);
  Procedure REGSELECT  (text, background, width, height : byte;
                        var cursor : byte; var choice : char);
  Procedure MENUSELECT (text, background, width, height : byte;
                        var cursor : byte; var choice : char);



{-----------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} IMPLEMENTATION {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{-----------------------------------------------------------------------------}



{=============================================================================}

{****** Combines the textcolor textbackground commands into one command ******}

Procedure TCTB (tc, tb : byte);

Begin
  textcolor (tc);
  textbackground (tb);
End;

{=============================================================================}

{************* Plays a sound for the designated time period ******************}

Procedure NOTE (tone, time : integer);

BEGIN
  sound (tone);
  delay (time);
END;

{=============================================================================}

{******** Combines the gotoXY and the write commands into one command ********}

Procedure WRTC (x, y : byte; chr: char);

Begin
  gotoXY (x, y);
  write (chr);
End;

{=============================================================================}

{******** Combines the gotoXY and the write commands into one command ********}

Procedure WRTS (x, y : byte; str : string);
Begin
  gotoXY (x, y);
  write (str);
End;

{=============================================================================}

{******** Combines the gotoXY and the write commands into one command ********}

Procedure WRTI (x, y : byte; int : integer);

Begin
  gotoXY (x, y);
  write (int);
End;

{=============================================================================}

{******** Combines the gotoXY and the write commands into one command ********}

Procedure WRTR (x, y : byte; reel : real; field, decimal : byte);

Begin
  gotoXY (x, y);
  write (reel:field:decimal);
End;

{=============================================================================}

{******** Combines the gotoXY and the write commands into one command ********}

Procedure FRMTI (x, y : byte; int: integer; field : byte);

Begin
  gotoXY (x,y);
  write (int:field);
End;

{=============================================================================}

{*************** Resizes to standard window without clearing it **************}

Procedure REGWIN;

Begin
  window (1,1,80,25);
End;

{=============================================================================}

{***************** Redraws the standard window and clears it *****************}

Procedure CLEAR (text, backcolor, ulx, uly, lrx, lry : byte);

Begin
  TCTB (text, backcolor);
  window (ulx,uly,lrx,lry);
  clrscr;
End;

{=============================================================================}

{***************** Redraws the specified window and clears it ****************}

Procedure MAKEWIN (shadow, border : boolean;
                   bordercolor, borderstyle,
                   text, background, ulx, uly, width, height : byte);
var
  vertical,
  horizontal,
  topleft,
  topright,
  bottomleft,
  bottomright : char;
  count       : byte;

begin
  if shadow
    then CLEAR (0, 0, (ulx + 2), (uly + 1),
                (ulx + (width + 1)), (uly + height));
  CLEAR (text, background, ulx, uly, ulx + (width - 1), uly + (height - 1));
  if border
    then
      begin
        case borderstyle of
          1 : begin
                vertical := #179;
                horizontal := #196;
                topleft := #218;
                topright := #191;
                bottomleft := #192;
                bottomright := #217;
              end;
          2 : begin
                vertical := #186;
                horizontal := #205;
                topleft := #201;
                topright := #187;
                bottomleft := #200;
                bottomright := #188;
              end;
          3 : begin
                vertical := #179;
                horizontal := #205;
                topleft := #213;
                topright := #184;
                bottomleft := #212;
                bottomright := #190;
              end;
          4 : begin
                vertical := #186;
                horizontal := #196;
                topleft := #214;
                topright := #183;
                bottomleft := #211;
                bottomright := #189;
              end;
        end;
        TCTB (bordercolor,background);
        REGWIN;
        for count := (ulx + 1) to (ulx + width - 2) do
          begin
            WRTC (count,uly,horizontal);
            WRTC (count,(uly + height - 1),horizontal);
          end;
        for count := (uly + 1) to (uly + height - 2) do
          begin
            WRTC (ulx,count,vertical);
            WRTC ((ulx + width - 1),count,vertical);
          end;
        WRTC (ulx,uly,topleft);
        WRTC ((ulx + width - 1),uly,topright);
        WRTC (ulx,(uly + height - 1),bottomleft);
        WRTC ((ulx + width - 1),(uly + height - 1),bottomright);
      end;
    window ((ulx + 1), (uly + 1), (ulx + width - 2), (uly  + height - 2));
    TCTB (text,background);

end;

{=============================================================================}

{****************************** Hides the cursor *****************************}

Procedure HIDECURSOR (x, y, color : byte);

Begin
  TCTB (color, color);
  WRTC (x, y, ' ');
  gotoXY (x,y);
End;

{=============================================================================}

Procedure REGSELECT (text, background, width, height : byte;
                     var cursor : byte; var choice : char);

Begin
  TCTB (text, background);
  WRTC (2, cursor, #16);
  WRTC ((width + 3), cursor, #17);
  repeat
    HIDECURSOR (1, cursor, background);
    choice := readkey;
    case choice of
      #80 : begin
              TCTB (background, background);
              WRTC (2, cursor, ' ');
              WRTC ((width + 3), cursor, ' ');
              if cursor = height
                then cursor := 1
                else inc(cursor);
              TCTB (text, background);
              WRTC (2, cursor, #16);
              WRTC ((width + 3), cursor, #17);
            end;
      #72 : begin
              TCTB (background, background);
              WRTC (2, cursor, ' ');
              WRTC ((width + 3), cursor, ' ');
              if cursor = 1
                then cursor := height
                else dec(cursor);
              TCTB (text, background);
              WRTC (2, cursor, #16);
              WRTC ((width + 3), cursor, #17);
            end;
    end;  { case }
  until (choice = #13) or (choice = #27);

End;

{=============================================================================}
{******************* Moves about in a vertical selection bar *****************}

Procedure MENUSELECT (text, background, width, height : byte;
                      var cursor : byte; var choice : char);

Begin
  TCTB (text, background);
  WRTC (2, cursor, #16);
  WRTC ((width + 3), cursor, #17);
  repeat
    HIDECURSOR (1, cursor, background);
    choice := readkey;
    case choice of
      #80 : begin
              TCTB (background, background);
              WRTC (2, cursor, ' ');
              WRTC ((width + 3), cursor, ' ');
              if cursor = height
                then cursor := 1
                else inc(cursor);
              TCTB (text, background);
              WRTC (2, cursor, #16);
              WRTC ((width + 3), cursor, #17);
            end;
      #72 : begin
              TCTB (background, background);
              WRTC (2, cursor, ' ');
              WRTC ((width + 3), cursor, ' ');
              if cursor = 1
                then cursor := height
                else dec(cursor);
              TCTB (text, background);
              WRTC (2, cursor, #16);
              WRTC ((width + 3), cursor, #17);
            end;
    end;  { case }
  until (choice = #13) or (choice = #27) or (choice = #75)  or (choice = #77)
        or (choice = #33) or (choice = #59) or (choice = #30) or
        (choice = #60) or (choice = #31) or (choice = #61) or (choice = #47)
        or (choice = #62) or (choice = #18) or (choice = #63) or
        (choice = #35) or (choice = #64);

End;

{=============================================================================}

END.  {*** UNIT UTILITY ***}