UNIT Menus;

INTERFACE

USES
  NewDelay,
  Crt,
  Utility;

CONST
  MaxMenu  = 20;

TYPE
  ExitCharSet = SET OF Char;
  MenuListElementType = RECORD
                      Position   : Integer;
                      OptionName : STRING;
                    END;

  MenuRelationType = (Less, Greater, Equal);

  MenuPointerType = ^MenuNodeType;
  MenuNodeType = RECORD
    Info   : MenuListElementType;
    Next   : MenuPointerType
  END;
  MenuListType = RECORD
    Length       : Integer;
    CurrentItem  : MenuPointerType;
    ListData     : MenuPointerType
  END;
  MenuType = ARRAY [1..MaxMenu] OF STRING;

FUNCTION MenuEmptyList (List : MenuListType ) : Boolean;
FUNCTION MenuFullList (List : MenuListType) : Boolean;
FUNCTION MenuListLength (List : MenuListType) : Integer;
PROCEDURE MenuCreateList (VAR List : MenuListType);
PROCEDURE MenuDestroyList (VAR List : MenuListType);
PROCEDURE MenuRetrieveElement (List : MenuListType; VAR Element : MenuListElementType);
PROCEDURE MenuInsertElement (VAR List : MenuListType; NewElement : MenuListElementType);
PROCEDURE MenuDeleteElement (VAR List : MenuListType; Element  : MenuListElementType);
PROCEDURE MenuResetList (VAR List : MenuListType);
PROCEDURE MenuGetNextItem (VAR List : MenuListType; VAR Element  : MenuListElementType);
PROCEDURE LListSelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; DirList : MenuListType; VAR Selected : MenuListElementType);

PROCEDURE ArraySelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; Menu : MenuType; ExitVals : ExitCharSet);

IMPLEMENTATION

{***************************************************************************}
FUNCTION CompareName (Item1, Item2 : MenuListElementType) : MenuRelationType;

BEGIN
  IF Item1.OptionName < Item2.OptionName THEN
      CompareName := Less
    ELSE IF Item1.OptionName > Item2.OptionName THEN
      CompareName := Greater
    ELSE CompareName := Equal;
END;

{***************************************************************************}
FUNCTION ComparePos (Item1, Item2 : MenuListElementType) : MenuRelationType;

BEGIN
  IF Item1.Position < Item2.Position THEN
      ComparePos := Less
    ELSE IF Item1.Position > Item2.Position THEN
      ComparePos := Greater
    ELSE ComparePos := Equal;
END;

(*************************************************************)
FUNCTION MenuEmptyList (List : MenuListType) : Boolean;

BEGIN
  MenuEmptyList := (List.ListData =  NIL)
END;

(*************************************************************)
FUNCTION MenuFullList (List : MenuListType) : Boolean;

BEGIN
  MenuFullList := False
END;

(*************************************************************)
FUNCTION MenuListLength (List : MenuListType) : Integer;

BEGIN
  MenuListLength := List.Length
END;

(*************************************************************)
PROCEDURE MenuCreateList (VAR List : MenuListType);

BEGIN
  List.ListData := NIL;
  List.Length := 0
END;

(*************************************************************)
PROCEDURE MenuDestroyList (VAR List : MenuListType);

VAR
  TempPtr : MenuPointerType;

BEGIN
  WITH List DO
    BEGIN
      List.Length := 0;
      WHILE ListData <>  NIL DO
        BEGIN
          TempPtr := ListData;
          ListData := ListData^.Next;
          Dispose (TempPtr)
        END
      END
END;

(*************************************************************)
PROCEDURE MenuRetrieveElement (List : MenuListType; VAR Element : MenuListElementType);

VAR
  Location     : MenuPointerType;

BEGIN
  Location := List.ListData;
  WHILE Location^.Info.Position <> Element.Position DO
    Location := Location^.Next;
  Element := Location^.Info;
END;

(*************************************************************)
PROCEDURE MenuInsertElement (VAR List : MenuListType; NewElement : MenuListElementType);

VAR
  NewNode      : MenuPointerType;
  PredLoc      : MenuPointerType;
  Location     : MenuPointerType;
  MoreToSearch : Boolean;

BEGIN
  WITH List DO
    BEGIN
      Location := List.ListData;
      PredLoc := NIL;
      MoreToSearch := Location <> NIL;
      List.Length := List.Length + 1;
      WHILE MoreToSearch DO
        CASE CompareName (NewElement, Location^.Info) OF
          Greater : BEGIN
                      PredLoc := Location;
                      Location := Location^.Next;
                      MoreToSearch := Location <> NIL
                    END;
          Less    : MoreToSearch := False
        END;
      New(NewNode);
      NewNode^.Info := NewElement;
      IF PredLoc = NIL
        THEN
          BEGIN
            NewNode^.Next := ListData;
            ListData := NewNode
          END
        ELSE
          BEGIN
            NewNode^.Next := PredLoc^.Next;
            PredLoc^.Next := NewNode
          END
    END
END;

(*************************************************************)
PROCEDURE MenuDeleteElement (VAR List : MenuListType; Element : MenuListElementType);

VAR
  PredLoc   : MenuPointerType;
  Location  : MenuPointerType;

  TempPtr   : MenuPointerType;
  MoreToSearch : Boolean;

BEGIN
  WITH List DO
    BEGIN
      Location := ListData;
      PredLoc := NIL;
      MoreToSearch := Location <> NIL;
      Length := Length - 1;
      WHILE MoreToSearch DO
        CASE CompareName (Element, Location^.Info) OF
          Greater: BEGIN
                     PredLoc := Location;
                     Location := Location^.Next;
                   END;
          Equal  : MoreToSearch := False
        END;
      IF PredLoc = NIL
        THEN
          BEGIN
            TempPtr := Location;
            ListData := Location^.Next;
          END
        ELSE
          BEGIN
            TempPtr := Location;
            PredLoc^.Next := Location^.Next
          END;
      Dispose (TempPtr)
    END
END;

(*************************************************************)
PROCEDURE MenuResetList (VAR List : MenuListType);

BEGIN
  List.CurrentItem := List.ListData
END;

(***********************************************************)
PROCEDURE MenuGetNextItem (VAR List : MenuListType; VAR Element : MenuListElementType);

BEGIN
  Element := List.CurrentItem^.Info;
  List.CurrentItem := List.CurrentItem^.Next
END;

{***************************************************************************}
PROCEDURE LListSelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; DirList : MenuListType; VAR Selected : MenuListElementType);

VAR
  Top      : Integer;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
PROCEDURE Move
  (Choice     : Char;
   VAR Cursor : Byte);

VAR
  Count : Byte;

BEGIN
  CASE Choice OF
    #72 : IF Cursor > 1 THEN dec(Cursor)
            ELSE IF (Top + Cursor - 1) > 1 THEN dec(Top);
    #80 : IF Cursor < Height THEN inc(Cursor)
            ELSE IF (Top + Cursor - 1) < MenuListLength(DirList) THEN inc(Top);
    #71 : BEGIN
            Cursor := 1;
            Top := 1;
          END;
    #79 : BEGIN
            Cursor := Height;
            Top := (MenuListLength(DirList) - Height + 1);
          END;
  END;
  TcTb (RegText, RegBack);
  FOR Count := 1 TO Height DO
    BEGIN
      Selected.Position := (Count + Top - 1);
      MenuRetrieveElement (DirList,Selected);
      WrtS (StartX,Count,Selected.OptionName);
    END;
  TcTb (SelText, SelBack);
  Selected.Position := Cursor + Top - 1;
  MenuRetrieveElement (DirList,Selected);
  WrtS (StartX, (Cursor + StartY - 1), Selected.OptionName);
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

BEGIN
  TcTb (SelText, SelBack);
  {* Fix this next time around.... *}
  IF MenuListLength(DirList) <= 18 THEN
      Top := 1
    ELSE Top := Cursor;
  Selected.Position := Cursor;
  MenuRetrieveElement (DirList,Selected);
  WrtS (StartX, (Cursor + StartY - 1), Selected.OptionName);
  Choice := readkey;
  WHILE NOT(Choice IN [#13,#27,#9]) DO
    BEGIN
      IF ((Choice = #72) AND ((Cursor + Top - 1) > 1)) OR
         ((Choice = #80) AND ((Cursor + Top - 1) < MenuListLength(DirList))) OR
         (Choice IN [#71,#79]) THEN Move (Choice,Cursor);
      Choice := readkey;
    END;
END;

{***************************************************************************}
PROCEDURE ArraySelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; Menu : MenuType; ExitVals : ExitCharSet);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
PROCEDURE Move
  (Choice : char;
   VAR Cursor : byte);

BEGIN
  TcTb (RegText, RegBack);
  WrtS (StartX, (Cursor + StartY - 1), Menu[Cursor]);
  CASE Choice OF
    #72 : IF Cursor > 1 THEN dec(Cursor);
    #80 : IF Cursor < Height THEN inc(Cursor);
  END;
  TcTb (SelText, SelBack);
  WrtS (StartX, (Cursor + StartY - 1), Menu[Cursor]);
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

BEGIN
  TcTb (SelText, SelBack);
  WrtS (StartX, (Cursor + StartY - 1), Menu[Cursor]);
  Choice := readkey;
  WHILE NOT(Choice IN ExitVals) DO
    BEGIN
      IF ((Choice = #72) AND (Cursor > 1)) OR ((Choice = #80) AND (Cursor < Height)) THEN
        BEGIN
          Move (Choice,Cursor);
        END;
      Choice := readkey;
    END;
END;

(***********************************************************)

END.




