UNIT SSData;

INTERFACE

USES
  NewDelay,
  Crt,
  Dos,
  Utility,
  Menus;

CONST
  MaxGames     = 18;
  GamerVersion = 1.01;
  Developer    = 'Sunstorm Interactive, Inc.';
  Year         = '(1997)';
  FilesMark    = 'SSFILES.MRK';
  HackMark     = 'SSUNHACK.MRK';
  BackupDir    = 'SSIFILES.BAK';

TYPE
  HackRec   = RECORD
                ByteLoc : LongInt;
                CASE VarType : 0..7 OF
                  0 : (StrLength  : Byte;
                       StringVal  : STRING);
                  1 : (CharVal    : Char);
                  2 : (BooleanVal : Boolean);
                  3 : (ByteVal    : Byte);
                  4 : (IntegerVal : Integer);
                  5 : (WordVal    : Word);
                  6 : (LongIntVal : LongInt);
                  7 : (RealVal    : Real);
              END;

VAR
  DirInfo  : SearchRec;

PROCEDURE DrawStatusBar (TextColor,BGColor : Byte; Message : STRING);
PROCEDURE MainScreenSetup;
PROCEDURE DrawInfoWindow (Addon : MenuElementType; DrawWin : Boolean);
PROCEDURE DrawMenuWindow (GameMenu : MenuList; DrawWin : Boolean; StartPos : Integer);
PROCEDURE MenuSelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; DirList : MenuList; VAR Selected : MenuElementType;
   ExitVals : ExitCharSet);

IMPLEMENTATION

{***************************************************************************}
PROCEDURE DrawStatusBar (TextColor,BGColor : Byte; Message : STRING);

BEGIN
  Clear (TextColor,BGColor,1,25,80,25);
  WrtS (1,1,Message);
END;

{***************************************************************************}
PROCEDURE MainScreenSetup;

VAR
  Count : Byte;

BEGIN
  Clear (1 + 16,7,1,1,80,1);
  WrtS (40 - (length('GAMER Developed By ' + Developer + ' ' + Year) DIV  2),1,
                     'GAMER Developed By ' + Developer + ' ' +Year);
  TcTb (7 + 16,0);
  window (1,2,80,25);
  FOR Count := 2 TO 24 DO
    WrtS (1,Count,'');
  TcTb (14 + 16,0);
  WrtS (72,23,' ver1.01 ');
  DrawStatusBar (1 + 16,7,'  ' + #24 + #25 + '-Select Game/Addon   <ENTER>-Activate/Deactivate   <ESC>-Exit SSI Manager');
END;

{***************************************************************************}
PROCEDURE DrawInfoWindow (Addon : MenuElementType; DrawWin : Boolean);

VAR
  Count : Byte;

BEGIN
  IF DrawWin THEN
      BEGIN
        MakeWin (9,1,3,17,74,6,True,True);
        window (27,17,53,17);
        TcTb (14,1);
        WrtS (1,1,' Add-on File Information ');
      END;

  Clear (7,1,4,18,75,21);
  FOR Count := 1 TO 3 DO
    WrtS (2,Count + 1,Addon.LongDescrpt[Count]);

  TcTb (14,1);
  WrtS (2,1,'Add-on Title:');
  WrtS (51,1,'Status:');

  TcTb (15,1);
  delete (Addon.Title,1,pos('-',Addon.Title));
  WrtS (15,1,Addon.Title);

  IF Addon.Activated THEN
      BEGIN
        TcTb (10,1);
        WrtS (59,1,'ACTIVATED');
      END
    ELSE
      BEGIN
        TcTb (12,1);
        WrtS (59,1,'NOT ACTIVATED');
      END;

  window (4,4,75,13);
END;

{***************************************************************************}
PROCEDURE DrawMenuWindow (GameMenu : MenuList; DrawWin : Boolean; StartPos : Integer);

VAR
  Count,
  Height : Byte;
  Addon  : MenuElementType;

BEGIN
  IF DrawWin THEN
      BEGIN
        MakeWin (9,1,3,3,74,12,True,True);
        window (22,3,59,3);
        TcTb (14,1);
        WrtS (1,1,' Game Add-on Manager : Add-on Menu ');
      END;
  Clear (15,1,4,4,75,13);

  IF MenuListLength(GameMenu) >= 10 THEN
      BEGIN
        Height := 10;
        IF (MenuListLength(GameMenu) - StartPos - 1) < Height THEN
            StartPos := MenuListLength(GameMenu) - Height + 1;
      END
    ELSE
      BEGIN
        Height := MenuListLength(GameMenu);
        StartPos := 1;
      END;

  FOR Count := StartPos TO (StartPos + Height - 1) DO
    BEGIN
      Addon.Position := Count;
      MenuRetrieveElement (GameMenu,Addon);
      IF Addon.Activated THEN TcTb (10,1);
      WrtS (37 - (length(Addon.Title) DIV 2) - (length(Addon.Title) MOD 2),Count,Addon.Title);
      IF Addon.Activated THEN TcTb (15,1);
    END;
END;

{***************************************************************************}
PROCEDURE MenuSelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; DirList : MenuList; VAR Selected : MenuElementType;
   ExitVals : ExitCharSet);

VAR
  Top      : Integer;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
PROCEDURE Move
  (Choice     : Char;
   VAR Cursor : Byte);

VAR
  Scroll : Boolean; {* Use for when scrolling, so that it only updates
  the entire thing when needed, rather than everytime... later add this *}
  Count : Byte;

BEGIN
  CASE Choice OF
    #72 : IF Cursor > 1 THEN dec(Cursor)
            ELSE IF (Top + Cursor - 1) > 1 THEN dec(Top);
    #80 : IF Cursor < Height THEN inc(Cursor)
            ELSE IF (Top + Cursor - 1) < MenuListLength(DirList) THEN inc(Top);
    #71 : BEGIN
            Cursor := 1;
            Top := 1;
          END;
    #79 : BEGIN
            Cursor := Height;
            Top := (MenuListLength(DirList) - Height + 1);
          END;
  END;
  TcTb (RegText, RegBack);
  FOR Count := 1 TO Height DO
    BEGIN
      Selected.Position := (Count + Top - 1);
      MenuRetrieveElement (DirList,Selected);
      IF Selected.Activated THEN TcTb (10,RegBack);
      WrtS (37 - (length(Selected.Title) DIV 2) - (length(Selected.Title) MOD 2),Count,Selected.Title);
      IF Selected.Activated THEN TcTb (RegText,RegBack);
    END;
  Selected.Position := Cursor + Top - 1;
  MenuRetrieveElement (DirList,Selected);
  IF Selected.Activated THEN
      TcTb (8 + 16,6)
    ELSE
      TcTb (SelText,SelBack);
  WrtS (37 - (length(Selected.Title) DIV 2) - (length(Selected.Title) MOD 2), (Cursor + StartY - 1), Selected.Title);
  DrawInfoWindow (Selected,False);
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

BEGIN
  IF MenuListLength(DirList) <= 10 THEN
      Top := 1
    ELSE Top := Cursor;
  Selected.Position := Cursor;
  MenuRetrieveElement (DirList,Selected);
  IF Selected.Activated THEN
      TcTb (8 + 16,6)
    ELSE
      TcTb (SelText,SelBack);
  WrtS (37 - (length(Selected.Title) DIV 2) - (length(Selected.Title) MOD 2), (Cursor + StartY - 1), Selected.Title);
  Choice := readkey;
  WHILE NOT(Choice IN ExitVals) DO
    BEGIN
      IF ((Choice = #72) AND ((Cursor + Top - 1) > 1)) OR
         ((Choice = #80) AND ((Cursor + Top - 1) < MenuListLength(DirList))) OR
         (Choice IN [#71,#79]) THEN Move (Choice,Cursor);
      Choice := readkey;
    END;
END;

{***************************************************************************}

END.